-- --------------------------------------------------------
-- Host:                         127.0.0.1
-- Versión del servidor:         5.6.21 - MySQL Community Server (GPL)
-- SO del servidor:              Win32
-- HeidiSQL Versión:             9.2.0.4947
-- --------------------------------------------------------

/*!40101 SET @OLD_CHARACTER_SET_CLIENT=@@CHARACTER_SET_CLIENT */;
/*!40101 SET NAMES utf8mb4 */;
/*!40014 SET @OLD_FOREIGN_KEY_CHECKS=@@FOREIGN_KEY_CHECKS, FOREIGN_KEY_CHECKS=0 */;
/*!40101 SET @OLD_SQL_MODE=@@SQL_MODE, SQL_MODE='NO_AUTO_VALUE_ON_ZERO' */;

-- Volcando estructura de base de datos para fia
CREATE DATABASE IF NOT EXISTS `fia` /*!40100 DEFAULT CHARACTER SET latin1 */;
USE `fia`;


-- Volcando estructura para tabla fia.circuit
CREATE TABLE IF NOT EXISTS `circuit` (
  `codi_circuit` mediumint(8) unsigned NOT NULL AUTO_INCREMENT,
  `nom` varchar(50) NOT NULL,
  `sobrenom` varchar(50) DEFAULT NULL,
  `longitud` smallint(6) unsigned DEFAULT NULL,
  `pais` char(2) NOT NULL,
  PRIMARY KEY (`codi_circuit`),
  UNIQUE KEY `nom` (`nom`),
  UNIQUE KEY `sobrenom` (`sobrenom`),
  KEY `FK_circuit_pais` (`pais`),
  CONSTRAINT `FK_circuit_pais` FOREIGN KEY (`pais`) REFERENCES `pais` (`codi_pais`)
) ENGINE=InnoDB DEFAULT CHARSET=latin1;




-- Volcando estructura para tabla fia.competicio
CREATE TABLE IF NOT EXISTS `competicio` (
  `codi_competicio` smallint(5) unsigned NOT NULL AUTO_INCREMENT,
  `nom` varchar(50) NOT NULL,
  `acronim` varchar(10) NOT NULL,
  `descripcio` text NOT NULL,
  PRIMARY KEY (`codi_competicio`),
  UNIQUE KEY `temporada` (`nom`),
  UNIQUE KEY `acronim` (`acronim`)
) ENGINE=InnoDB DEFAULT CHARSET=latin1;




-- Volcando estructura para tabla fia.competicio_temporada
CREATE TABLE IF NOT EXISTS `competicio_temporada` (
  `competicio` smallint(5) unsigned NOT NULL,
  `temporada` year(4) NOT NULL,
  PRIMARY KEY (`competicio`,`temporada`),
  KEY `FK_competicio_temporada_temporada` (`temporada`),
  CONSTRAINT `FK_competicio_temporada_competicio` FOREIGN KEY (`competicio`) REFERENCES `competicio` (`codi_competicio`),
  CONSTRAINT `FK_competicio_temporada_temporada` FOREIGN KEY (`temporada`) REFERENCES `temporada` (`any`)
) ENGINE=InnoDB DEFAULT CHARSET=latin1;




-- Volcando estructura para tabla fia.component
CREATE TABLE IF NOT EXISTS `component` (
  `codi_component` mediumint(8) unsigned NOT NULL AUTO_INCREMENT,
  `fabricant` mediumint(8) unsigned NOT NULL,
  `cotxe` mediumint(8) unsigned DEFAULT NULL,
  `descripcio` text NOT NULL,
  `utilitat` text NOT NULL,
  PRIMARY KEY (`codi_component`),
  KEY `FK_component_fabricant` (`fabricant`),
  KEY `FK_component_cotxe` (`cotxe`),
  CONSTRAINT `FK_component_cotxe` FOREIGN KEY (`cotxe`) REFERENCES `cotxe` (`codi_cotxe`),
  CONSTRAINT `FK_component_fabricant` FOREIGN KEY (`fabricant`) REFERENCES `fabricant` (`codi_fabricant`)
) ENGINE=InnoDB DEFAULT CHARSET=latin1;




-- Volcando estructura para tabla fia.cotxe
CREATE TABLE IF NOT EXISTS `cotxe` (
  `codi_cotxe` mediumint(8) unsigned NOT NULL AUTO_INCREMENT,
  `model` varchar(25) NOT NULL,
  `any_debut` year(4) NOT NULL,
  PRIMARY KEY (`codi_cotxe`),
  UNIQUE KEY `model` (`model`)
) ENGINE=InnoDB DEFAULT CHARSET=latin1;




-- Volcando estructura para tabla fia.cotxe_equip
CREATE TABLE IF NOT EXISTS `cotxe_equip` (
  `cotxe` mediumint(8) unsigned NOT NULL,
  `equip` smallint(5) unsigned NOT NULL,
  `temporada` year(4) NOT NULL,
  PRIMARY KEY (`cotxe`,`equip`,`temporada`),
  KEY `FK_cotxe_equip_equip` (`equip`),
  KEY `FK_cotxe_equip_temporada` (`temporada`),
  CONSTRAINT `FK_cotxe_equip_cotxe` FOREIGN KEY (`cotxe`) REFERENCES `cotxe` (`codi_cotxe`),
  CONSTRAINT `FK_cotxe_equip_equip` FOREIGN KEY (`equip`) REFERENCES `equip` (`codi_equip`),
  CONSTRAINT `FK_cotxe_equip_temporada` FOREIGN KEY (`temporada`) REFERENCES `temporada` (`any`)
) ENGINE=InnoDB DEFAULT CHARSET=latin1;




-- Volcando estructura para tabla fia.cursa
CREATE TABLE IF NOT EXISTS `cursa` (
  `circuit` mediumint(8) unsigned NOT NULL,
  `pilot` mediumint(8) unsigned NOT NULL,
  `competicio` smallint(5) unsigned NOT NULL,
  `posicio` smallint(5) unsigned NOT NULL,
  `data_cursa` date NOT NULL,
  PRIMARY KEY (`circuit`,`pilot`,`competicio`,`data_cursa`),
  KEY `FK_cursa_pilot` (`pilot`),
  KEY `FK_cursa_competicio` (`competicio`),
  CONSTRAINT `FK_cursa_circuit` FOREIGN KEY (`circuit`) REFERENCES `circuit` (`codi_circuit`),
  CONSTRAINT `FK_cursa_competicio` FOREIGN KEY (`competicio`) REFERENCES `competicio` (`codi_competicio`),
  CONSTRAINT `FK_cursa_pilot` FOREIGN KEY (`pilot`) REFERENCES `pilot` (`codi_persona`)
) ENGINE=InnoDB DEFAULT CHARSET=latin1;




-- Volcando estructura para tabla fia.dim_classificacio
CREATE TABLE IF NOT EXISTS `dim_classificacio` (
  `competicio` smallint(5) unsigned NOT NULL,
  `temporada` year(4) NOT NULL,
  `pilot` mediumint(8) unsigned NOT NULL,
  `punts` smallint(5) unsigned NOT NULL DEFAULT '0'
) ENGINE=InnoDB DEFAULT CHARSET=latin1;




-- Volcando estructura para tabla fia.dim_component
CREATE TABLE IF NOT EXISTS `dim_component` (
  `fabricant` mediumint(8) unsigned NOT NULL,
  `components` mediumint(8) unsigned NOT NULL
) ENGINE=InnoDB DEFAULT CHARSET=latin1;




-- Volcando estructura para tabla fia.dim_consum
CREATE TABLE IF NOT EXISTS `dim_consum` (
  `competicio` smallint(5) unsigned NOT NULL,
  `temporada` year(4) NOT NULL,
  `cotxe` mediumint(8) unsigned NOT NULL,
  `consum` float(8,4) unsigned NOT NULL
) ENGINE=InnoDB DEFAULT CHARSET=latin1;




-- Volcando estructura para tabla fia.dim_defectuos
CREATE TABLE IF NOT EXISTS `dim_defectuos` (
  `cotxe` mediumint(8) unsigned NOT NULL,
  `component` mediumint(8) unsigned NOT NULL,
  `defectuos` bit(1) NOT NULL DEFAULT b'0'
) ENGINE=InnoDB DEFAULT CHARSET=latin1;




-- Volcando estructura para tabla fia.dim_patrocini
CREATE TABLE IF NOT EXISTS `dim_patrocini` (
  `patrocinador` mediumint(9) unsigned NOT NULL,
  `cost` int(11) unsigned NOT NULL
) ENGINE=InnoDB DEFAULT CHARSET=latin1;




-- Volcando estructura para tabla fia.dim_resultat
CREATE TABLE IF NOT EXISTS `dim_resultat` (
  `pilot` mediumint(8) unsigned NOT NULL,
  `guanyades` smallint(5) unsigned NOT NULL DEFAULT '0',
  `finalitzades` smallint(5) unsigned NOT NULL DEFAULT '0'
) ENGINE=InnoDB DEFAULT CHARSET=latin1;




-- Volcando estructura para tabla fia.dim_telemetria
CREATE TABLE IF NOT EXISTS `dim_telemetria` (
  `temporada` year(4) NOT NULL,
  `equip` smallint(6) unsigned NOT NULL,
  `dades_any` mediumint(8) unsigned NOT NULL
) ENGINE=InnoDB DEFAULT CHARSET=latin1;




-- Volcando estructura para tabla fia.dim_volta_rapida
CREATE TABLE IF NOT EXISTS `dim_volta_rapida` (
  `circuit` mediumint(8) unsigned NOT NULL,
  `pilot` mediumint(8) unsigned NOT NULL,
  `equip` smallint(5) unsigned NOT NULL,
  `data` date NOT NULL,
  `temps` time NOT NULL
) ENGINE=InnoDB DEFAULT CHARSET=latin1;




-- Volcando estructura para tabla fia.director
CREATE TABLE IF NOT EXISTS `director` (
  `codi_persona` mediumint(8) unsigned NOT NULL,
  `equip` smallint(5) unsigned NOT NULL,
  `estudis` text NOT NULL,
  `any_inici` year(4) NOT NULL,
  PRIMARY KEY (`codi_persona`),
  UNIQUE KEY `codi_equip` (`equip`),
  CONSTRAINT `FK_director_equip` FOREIGN KEY (`equip`) REFERENCES `equip` (`codi_equip`),
  CONSTRAINT `FK_director_persona` FOREIGN KEY (`codi_persona`) REFERENCES `persona` (`codi_persona`)
) ENGINE=InnoDB DEFAULT CHARSET=latin1;




-- Volcando estructura para tabla fia.emissora
CREATE TABLE IF NOT EXISTS `emissora` (
  `codi_emissora` smallint(5) unsigned NOT NULL AUTO_INCREMENT,
  `nom` varchar(50) NOT NULL,
  `pais` char(2) NOT NULL,
  PRIMARY KEY (`codi_emissora`),
  UNIQUE KEY `nom` (`nom`,`pais`),
  KEY `FK_emisora_pais` (`pais`),
  CONSTRAINT `FK_emisora_pais` FOREIGN KEY (`pais`) REFERENCES `pais` (`codi_pais`)
) ENGINE=InnoDB DEFAULT CHARSET=latin1;




-- Volcando estructura para tabla fia.equip
CREATE TABLE IF NOT EXISTS `equip` (
  `codi_equip` smallint(5) unsigned NOT NULL AUTO_INCREMENT,
  `nom` varchar(50) NOT NULL,
  `seu` varchar(100) NOT NULL,
  `data_debut` date NOT NULL,
  PRIMARY KEY (`codi_equip`),
  UNIQUE KEY `nom` (`nom`)
) ENGINE=InnoDB DEFAULT CHARSET=latin1;




-- Volcando estructura para tabla fia.fabricant
CREATE TABLE IF NOT EXISTS `fabricant` (
  `codi_fabricant` mediumint(9) unsigned NOT NULL AUTO_INCREMENT,
  `nom` varchar(50) NOT NULL,
  `any_debut` year(4) NOT NULL,
  PRIMARY KEY (`codi_fabricant`),
  UNIQUE KEY `nom` (`nom`)
) ENGINE=InnoDB DEFAULT CHARSET=latin1;




-- Volcando estructura para tabla fia.fact_table_1
CREATE TABLE IF NOT EXISTS `fact_table_1` (
  `competicio` smallint(5) unsigned DEFAULT NULL,
  `temporada` year(4) DEFAULT NULL,
  `pilot` mediumint(8) unsigned DEFAULT NULL,
  `equip` smallint(5) unsigned DEFAULT NULL,
  `cotxe` mediumint(8) unsigned DEFAULT NULL,
  `circuit` mediumint(8) unsigned DEFAULT NULL,
  `patrocinador` mediumint(8) unsigned DEFAULT NULL,
  `fabricant` mediumint(8) unsigned DEFAULT NULL,
  `data_cursa` date DEFAULT NULL,
  `data_millora` date DEFAULT NULL,
  `quantitat_components` mediumint(8) unsigned DEFAULT NULL,
  `cost_patrocini` int(10) unsigned DEFAULT NULL,
  `volta_rapida` time DEFAULT NULL,
  `temperatura_maxima` float(8,2) unsigned DEFAULT NULL,
  `dades_telemetriques` mediumint(8) unsigned DEFAULT NULL,
  `dades_telemetriques_equip` mediumint(8) unsigned DEFAULT NULL,
  `consum_cotxe` float(8,2) unsigned DEFAULT NULL,
  `punts_pilot` mediumint(8) unsigned DEFAULT NULL,
  `posicio_pilot` smallint(5) unsigned DEFAULT NULL,
  `curses_finalitzades` smallint(5) unsigned DEFAULT NULL,
  `curses_guanyades` smallint(5) unsigned DEFAULT NULL,
  `components_defectuosos` decimal(4,1) DEFAULT NULL,
  `camp_control` char(4) DEFAULT NULL,
  KEY `FK_fact_table_1_temporada` (`temporada`),
  KEY `FK_fact_table_1_pilot` (`pilot`),
  KEY `FK_fact_table_1_competicio` (`competicio`),
  KEY `FK_fact_table_1_equip` (`equip`),
  KEY `FK_fact_table_1_cotxe` (`cotxe`),
  KEY `FK_fact_table_1_circuit` (`circuit`),
  KEY `FK_fact_table_1_patrocinador` (`patrocinador`),
  KEY `FK_fact_table_1_fabricant` (`fabricant`),
  CONSTRAINT `FK_fact_table_1_circuit` FOREIGN KEY (`circuit`) REFERENCES `circuit` (`codi_circuit`) ON DELETE CASCADE,
  CONSTRAINT `FK_fact_table_1_competicio` FOREIGN KEY (`competicio`) REFERENCES `competicio` (`codi_competicio`) ON DELETE CASCADE,
  CONSTRAINT `FK_fact_table_1_cotxe` FOREIGN KEY (`cotxe`) REFERENCES `cotxe` (`codi_cotxe`) ON DELETE CASCADE,
  CONSTRAINT `FK_fact_table_1_equip` FOREIGN KEY (`equip`) REFERENCES `equip` (`codi_equip`) ON DELETE CASCADE,
  CONSTRAINT `FK_fact_table_1_fabricant` FOREIGN KEY (`fabricant`) REFERENCES `fabricant` (`codi_fabricant`) ON DELETE CASCADE,
  CONSTRAINT `FK_fact_table_1_patrocinador` FOREIGN KEY (`patrocinador`) REFERENCES `patrocinador` (`codi_patrocinador`) ON DELETE CASCADE,
  CONSTRAINT `FK_fact_table_1_pilot` FOREIGN KEY (`pilot`) REFERENCES `pilot` (`codi_persona`) ON DELETE CASCADE,
  CONSTRAINT `FK_fact_table_1_temporada` FOREIGN KEY (`temporada`) REFERENCES `temporada` (`any`) ON DELETE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=latin1;




-- Volcando estructura para tabla fia.log
CREATE TABLE IF NOT EXISTS `log` (
  `codi_log` bigint(20) unsigned NOT NULL AUTO_INCREMENT,
  `operacio` enum('ALTA','BAIXA','MODIFICACIÓ') NOT NULL,
  `taula` varchar(100) NOT NULL,
  `antic_valor` varchar(512) DEFAULT NULL,
  `nou_valor` varchar(512) DEFAULT NULL,
  `data` datetime NOT NULL,
  `consulta` varchar(512) DEFAULT NULL,
  PRIMARY KEY (`codi_log`)
) ENGINE=InnoDB DEFAULT CHARSET=latin1;




-- Volcando estructura para tabla fia.pais
CREATE TABLE IF NOT EXISTS `pais` (
  `codi_pais` char(2) NOT NULL,
  `nom` varchar(50) NOT NULL,
  PRIMARY KEY (`codi_pais`),
  UNIQUE KEY `nom` (`nom`)
) ENGINE=InnoDB DEFAULT CHARSET=latin1;




-- Volcando estructura para tabla fia.patrocinador
CREATE TABLE IF NOT EXISTS `patrocinador` (
  `codi_patrocinador` mediumint(8) unsigned NOT NULL AUTO_INCREMENT,
  `sector` smallint(5) unsigned DEFAULT '0',
  `nom` varchar(50) NOT NULL DEFAULT '0',
  PRIMARY KEY (`codi_patrocinador`),
  UNIQUE KEY `nom` (`nom`),
  KEY `FK__sector` (`sector`),
  CONSTRAINT `FK__sector` FOREIGN KEY (`sector`) REFERENCES `sector` (`codi_sector`) ON DELETE SET NULL
) ENGINE=InnoDB DEFAULT CHARSET=latin1;




-- Volcando estructura para tabla fia.patrocini
CREATE TABLE IF NOT EXISTS `patrocini` (
  `patrocinador` mediumint(9) unsigned NOT NULL AUTO_INCREMENT,
  `equip` smallint(5) unsigned NOT NULL,
  `temporada` year(4) NOT NULL,
  `cost` int(10) unsigned NOT NULL,
  PRIMARY KEY (`patrocinador`,`equip`,`temporada`),
  KEY `FK_patrocini_equip` (`equip`),
  KEY `FK_patrocini_temporada` (`temporada`),
  CONSTRAINT `FK_patrocini_equip` FOREIGN KEY (`equip`) REFERENCES `equip` (`codi_equip`),
  CONSTRAINT `FK_patrocini_patrocinador` FOREIGN KEY (`patrocinador`) REFERENCES `patrocinador` (`codi_patrocinador`),
  CONSTRAINT `FK_patrocini_temporada` FOREIGN KEY (`temporada`) REFERENCES `temporada` (`any`)
) ENGINE=InnoDB DEFAULT CHARSET=latin1;




-- Volcando estructura para tabla fia.persona
CREATE TABLE IF NOT EXISTS `persona` (
  `codi_persona` mediumint(8) unsigned NOT NULL AUTO_INCREMENT,
  `nom` varchar(100) NOT NULL,
  `nacionalitat` char(2) NOT NULL,
  PRIMARY KEY (`codi_persona`),
  UNIQUE KEY `nom` (`nom`),
  KEY `FK_persona_pais` (`nacionalitat`),
  CONSTRAINT `FK_persona_pais` FOREIGN KEY (`nacionalitat`) REFERENCES `pais` (`codi_pais`)
) ENGINE=InnoDB DEFAULT CHARSET=latin1;




-- Volcando estructura para tabla fia.pilot
CREATE TABLE IF NOT EXISTS `pilot` (
  `codi_persona` mediumint(8) unsigned NOT NULL,
  `equip` smallint(6) unsigned NOT NULL,
  `codi_llicencia` varchar(8) NOT NULL,
  `any_inici` year(4) NOT NULL,
  PRIMARY KEY (`codi_persona`),
  UNIQUE KEY `codi_llicencia` (`codi_llicencia`),
  KEY `FK_pilot_equip` (`equip`),
  CONSTRAINT `FK_pilot_equip` FOREIGN KEY (`equip`) REFERENCES `equip` (`codi_equip`),
  CONSTRAINT `pilot_ibfk_1` FOREIGN KEY (`codi_persona`) REFERENCES `persona` (`codi_persona`)
) ENGINE=InnoDB DEFAULT CHARSET=latin1 ROW_FORMAT=COMPACT;




-- Volcando estructura para tabla fia.pilot_cotxe
CREATE TABLE IF NOT EXISTS `pilot_cotxe` (
  `pilot` mediumint(8) unsigned NOT NULL,
  `cotxe` mediumint(8) unsigned NOT NULL,
  `temporada` year(4) NOT NULL,
  PRIMARY KEY (`pilot`,`cotxe`,`temporada`),
  KEY `FK_pilot_cotxe_cotxe` (`cotxe`),
  KEY `FK_pilot_cotxe_temporada` (`temporada`),
  CONSTRAINT `FK_pilot_cotxe_cotxe` FOREIGN KEY (`cotxe`) REFERENCES `cotxe` (`codi_cotxe`),
  CONSTRAINT `FK_pilot_cotxe_pilot` FOREIGN KEY (`pilot`) REFERENCES `pilot` (`codi_persona`),
  CONSTRAINT `FK_pilot_cotxe_temporada` FOREIGN KEY (`temporada`) REFERENCES `temporada` (`any`)
) ENGINE=InnoDB DEFAULT CHARSET=latin1;




-- Volcando estructura para tabla fia.retransmissio
CREATE TABLE IF NOT EXISTS `retransmissio` (
  `emissora` smallint(5) unsigned NOT NULL,
  `competicio` smallint(5) unsigned NOT NULL,
  PRIMARY KEY (`emissora`,`competicio`),
  KEY `FK_retransmissio_competicio` (`competicio`),
  CONSTRAINT `FK_retransmissio_competicio` FOREIGN KEY (`competicio`) REFERENCES `competicio` (`codi_competicio`),
  CONSTRAINT `FK_retransmissio_emisora` FOREIGN KEY (`emissora`) REFERENCES `emissora` (`codi_emissora`)
) ENGINE=InnoDB DEFAULT CHARSET=latin1;




-- Volcando estructura para tabla fia.sector
CREATE TABLE IF NOT EXISTS `sector` (
  `codi_sector` smallint(5) unsigned NOT NULL AUTO_INCREMENT,
  `nom` varchar(50) NOT NULL,
  PRIMARY KEY (`codi_sector`),
  UNIQUE KEY `nom` (`nom`)
) ENGINE=InnoDB DEFAULT CHARSET=latin1;




-- Volcando estructura para tabla fia.telemetria
CREATE TABLE IF NOT EXISTS `telemetria` (
  `codi_dada` bigint(20) unsigned NOT NULL AUTO_INCREMENT,
  `component` mediumint(8) unsigned NOT NULL,
  `moment` datetime(6) NOT NULL,
  `ambit` enum('marxa','velocitat','temperatura','bateria','frenada','consum','temps per volta') NOT NULL,
  `valor` float(8,4) NOT NULL,
  `unitat_de_mesura` enum(' -','kmh','ºC','%','N','lps','ms') NOT NULL,
  PRIMARY KEY (`codi_dada`),
  KEY `FK_telemetria_component` (`component`),
  KEY `Índice 3` (`moment`),
  CONSTRAINT `FK_telemetria_component` FOREIGN KEY (`component`) REFERENCES `component` (`codi_component`)
) ENGINE=InnoDB DEFAULT CHARSET=latin1;




-- Volcando estructura para tabla fia.temporada
CREATE TABLE IF NOT EXISTS `temporada` (
  `any` year(4) NOT NULL,
  PRIMARY KEY (`any`)
) ENGINE=InnoDB DEFAULT CHARSET=latin1;


/*!40101 SET SQL_MODE=IFNULL(@OLD_SQL_MODE, '') */;
/*!40014 SET FOREIGN_KEY_CHECKS=IF(@OLD_FOREIGN_KEY_CHECKS IS NULL, 1, @OLD_FOREIGN_KEY_CHECKS) */;
/*!40101 SET CHARACTER_SET_CLIENT=@OLD_CHARACTER_SET_CLIENT */;






-- --------------------------------------------------------
-- Host:                         127.0.0.1
-- Versión del servidor:         5.6.21 - MySQL Community Server (GPL)
-- SO del servidor:              Win32
-- HeidiSQL Versión:             9.2.0.4947
-- --------------------------------------------------------

/*!40101 SET @OLD_CHARACTER_SET_CLIENT=@@CHARACTER_SET_CLIENT */;
/*!40101 SET NAMES utf8mb4 */;
/*!40014 SET @OLD_FOREIGN_KEY_CHECKS=@@FOREIGN_KEY_CHECKS, FOREIGN_KEY_CHECKS=0 */;
/*!40101 SET @OLD_SQL_MODE=@@SQL_MODE, SQL_MODE='NO_AUTO_VALUE_ON_ZERO' */;

-- Volcando estructura de base de datos para fia

USE `fia`;


-- Volcando estructura para procedimiento fia.alta_circuit
DELIMITER //
CREATE DEFINER=`root`@`localhost` PROCEDURE `alta_circuit`(IN `nom` VARCHAR(50), IN `sobrenom` VARCHAR(50), IN `longitud` SMALLINT, IN `nacionalitat` CHAR(2), OUT `RSP` VARCHAR(512))
BEGIN

declare valor_antic VARCHAR (10);
declare valor_nou VARCHAR (512);
declare consulta VARCHAR(512);
declare msg VARCHAR(250);
declare state VARCHAR(50);

-- Control d'errors

DECLARE EXIT HANDLER FOR 1644 
BEGIN
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT("ERROR: Codi ",state,". ",msg);
END;

DECLARE EXIT HANDLER FOR 1062 
BEGIN
GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
SET RSP := CONCAT('ERROR: Codi ',state,'.Nom de circuit (',nom,') o sobrenom (',IFNULL(sobrenom,"NULL"),') duplicat.');
END;

DECLARE EXIT HANDLER FOR 1452
BEGIN
GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
SET RSP := CONCAT('ERROR: Codi ',state,'. La clau forana (',nacionalitat,') no es correcte.');
END;

DECLARE EXIT HANDLER FOR SQLEXCEPTION
BEGIN
GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
SET RSP := CONCAT('ERROR: Desconegut. Codi ',state,'. Missatge original: ',msg);
END;

DECLARE EXIT HANDLER FOR SQLWARNING 
BEGIN 
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Warning. Codi ',state,'. Missatge original: ',msg);
END;


	SET RSP := 'OK';
	
	
	-- Inserta a la taula
	
	INSERT INTO circuit(nom, sobrenom, longitud, pais) VALUES (nom, sobrenom, longitud, nacionalitat);
	

	-- Actualitzar log
		
	SET valor_antic = " - ";
	SET valor_nou = CONCAT_WS(', ',LAST_INSERT_ID(),nom, sobrenom, longitud, nacionalitat);
	SET consulta = CONCAT("alta_circuit ('",nom,"', '",(IFNULL(sobrenom,"NULL")),"', ",(IFNULL(longitud,"NULL")),", '",nacionalitat,"')");
	
	INSERT INTO log(operacio, taula,antic_valor,nou_valor,data,consulta) VALUES ('ALTA', 'circuit', valor_antic, valor_nou, NOW(), consulta);

END//
DELIMITER ;


-- Volcando estructura para procedimiento fia.alta_competicio
DELIMITER //
CREATE DEFINER=`root`@`localhost` PROCEDURE `alta_competicio`(IN `nom` VARCHAR(50), IN `acronim` VARCHAR(10), IN `descripcio` TEXT, OUT `RSP` VARCHAR(250))
BEGIN

declare valor_antic VARCHAR (10);
declare valor_nou VARCHAR (512);
declare consulta VARCHAR(512);
declare msg VARCHAR(250);
declare state VARCHAR(50);

declare new_competicio SMALLINT;


-- Control d'errors

DECLARE EXIT HANDLER FOR 1062 
BEGIN
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT("ERROR: Codi ",state,". Possiblement la competicio (",nom,") o l'acronim (",acronim,") ja existeix a la taula \'competicio\'.");
END;

DECLARE EXIT HANDLER FOR SQLWARNING 
BEGIN 
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Warning. Codi ',state,'. Missatge original: ',msg);
END;

DECLARE EXIT HANDLER FOR SQLEXCEPTION
BEGIN
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Desconegut. Codi ',state,'. Missatge original: ',msg);
END;

	
	SET RSP := 'OK';
	
	
	-- Insertar a la taula
	
	INSERT INTO competicio(nom, acronim, descripcio) VALUES (nom, acronim, descripcio);

	SET new_competicio = LAST_INSERT_ID();
	
	
	-- Actualitzar log
	
	set valor_antic = " - ";
	set valor_nou = CONCAT_WS(', ', new_competicio, nom, acronim, descripcio);
	SET consulta = CONCAT("alta_competicio ('",nom,"', '",acronim,"', '",descripcio,"')");
	
	INSERT INTO log(operacio, taula,antic_valor,nou_valor,data,consulta) VALUES ('ALTA', 'competicio', valor_antic, valor_nou, NOW(), consulta);
	
END//
DELIMITER ;


-- Volcando estructura para procedimiento fia.alta_component
DELIMITER //
CREATE DEFINER=`root`@`localhost` PROCEDURE `alta_component`(IN `in_fabricant` MEDIUMINT, IN `in_cotxe` MEDIUMINT, IN `descripcio` TEXT, IN `utilitat` TEXT, OUT `RSP` VARCHAR(512))
BEGIN

declare valor_antic VARCHAR (10);
declare valor_nou VARCHAR (512);
declare codi_fabricant MEDIUMINT;
declare codi_cotxe MEDIUMINT;
declare consulta VARCHAR(512);
declare msg VARCHAR(250);
declare state VARCHAR(50);

declare aux_existeix MEDIUMINT;
declare codi_component MEDIUMINT;
declare percentatge_defectuos DECIMAL(4,1);


-- Control d'errors

DECLARE EXIT HANDLER FOR 1452
BEGIN
GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
SET RSP := CONCAT('ERROR: Codi ',state,'. Possiblement el fabricant (',in_fabricant,') o el cotxe (',in_cotxe,') no son correctes.');
END;


DECLARE EXIT HANDLER FOR NOT FOUND
 BEGIN
 	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
   SET RSP := CONCAT('ERROR: Not found. Codi ',state,'. El nom del fabricant o del cotxe no son correctes.' );
 END;

DECLARE EXIT HANDLER FOR SQLWARNING 
BEGIN 
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Warning. Codi ',state,'. Missatge original: ',msg);
END;

DECLARE CONTINUE HANDLER FOR SQLEXCEPTION
BEGIN
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Desconegut. Codi ',state,'. Missatge original: ',msg);
END;


	SET RSP := 'OK';
	
	
	-- Es fa servir SELECT INTO per tal de llençar l'excepcio  NOT FOUND si no es troba el codi a la taula
	
	SELECT f.codi_fabricant FROM fabricant f WHERE f.codi_fabricant = in_fabricant INTO codi_fabricant;
	SELECT c.codi_cotxe FROM cotxe c WHERE c.codi_cotxe = in_cotxe INTO codi_cotxe;
	
	
	-- Insertar a la taula
	
	INSERT INTO component (fabricant,cotxe,descripcio,utilitat) VALUES (codi_fabricant, codi_cotxe, descripcio, utilitat);

	
	-- Actualitzar log
	
	SET codi_component = LAST_INSERT_ID();
	SET valor_antic = " - ";
	SET consulta = CONCAT("alta_component ('",in_fabricant,"','",in_cotxe,"', '",descripcio,"', '",utilitat,"')");
	SET valor_nou = CONCAT_WS(', ', codi_component, codi_fabricant, codi_cotxe, descripcio, utilitat);
	
	INSERT INTO log(operacio, taula,antic_valor,nou_valor,data,consulta) VALUES ('ALTA', 'component', valor_antic, valor_nou, NOW(),consulta);
	

	-- Actualitzar estadistiques sobre quantitat de components aportats per fabricant
	
	SET aux_existeix = (SELECT components FROM dim_component dc WHERE dc.fabricant = codi_fabricant);				
	
	IF aux_existeix IS NULL THEN
		INSERT INTO dim_component (fabricant, components) 
		VALUES (codi_fabricant, 1);
	ELSE
		UPDATE dim_component dc
		SET dc.components = dc.components + 1
		WHERE dc.fabricant = codi_fabricant;
	END IF;
	
	DELETE FROM fact_table_1 WHERE camp_control = '0xQC';
   INSERT INTO fact_table_1(fabricant, quantitat_components, camp_control) 
   SELECT dc.fabricant, dc.components, '0xQC' FROM dim_component dc ORDER BY dc.components DESC LIMIT 5;
	
	SET aux_existeix = NULL;
	
	
	-- Actualitzar estadistiques sobre quantitat de components defectuosos

	INSERT INTO dim_defectuos (cotxe, component) 
	VALUES (in_cotxe, codi_component);
	
	IF (SELECT camp_control FROM fact_table_1 WHERE camp_control = '0xCD' AND cotxe = in_cotxe) IS NULL THEN
		INSERT INTO fact_table_1 (cotxe, components_defectuosos, camp_control)
		VALUES (in_cotxe, 0, '0xCD');
	ELSE
		SELECT ROUND((SUM(CASE WHEN defectuos = 1 THEN 1 ELSE 0 END)/COUNT(*)*100),1) 
		FROM dim_defectuos WHERE cotxe = in_cotxe
		INTO percentatge_defectuos;

		UPDATE fact_table_1
		SET components_defectuosos = percentatge_defectuos
		WHERE cotxe = in_cotxe
		AND camp_control = '0xCD';
 	END IF;
	
END//
DELIMITER ;


-- Volcando estructura para procedimiento fia.alta_cotxe
DELIMITER //
CREATE DEFINER=`root`@`localhost` PROCEDURE `alta_cotxe`(IN `model` VARCHAR(25), IN `any_debut` YEAR, IN `in_equip` MEDIUMINT, IN `temporada` YEAR, OUT `RSP` VARCHAR(250))
BEGIN

declare codi_equip SMALLINT(5);
declare codi_cotxe_afegit MEDIUMINT(8);
declare valor_antic VARCHAR (10);
declare valor_nou VARCHAR (512);
declare consulta_cotxe VARCHAR(512);
declare consulta_equip VARCHAR(512);
declare state VARCHAR(50);
declare msg VARCHAR(250);

declare check_temporada YEAR;


-- Control d'errors

DECLARE EXIT HANDLER FOR 1452
BEGIN
GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
SET RSP := CONCAT('ERROR: Codi ',state,'. Possiblement la relacio ',model,'-',in_equip,'-',temporada,' ja existeix a la taula \'cotxe_equip\'');
END;

DECLARE EXIT HANDLER FOR 1062 
BEGIN
GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
SET RSP := CONCAT('ERROR: Codi ',state,'. Possiblement el model del cotxe (',model,') ja existeix a la taula \'cotxe\'');
END;

DECLARE EXIT HANDLER FOR NOT FOUND
 BEGIN
 	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
   SET RSP := CONCAT('ERROR: Not found. Codi ',state,'. Pot ser que el codi de l\'equip (',in_equip,') no sigui correcte o que la temporada (',temporada,') no existeixi a la taula \'temporada\'');
 END;

DECLARE EXIT HANDLER FOR SQLWARNING 
BEGIN 
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Warning. Codi ',state,'. Missatge original: ',msg);
END;

DECLARE CONTINUE HANDLER FOR SQLEXCEPTION
BEGIN
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Desconegut. Codi ',state,'. Missatge original: ',msg);
END;


	SET RSP := 'OK';
	
	-- Es fa servir SELECT INTO per tal de llençar l'excepcio  NOT FOUND si no es troba el codi a la taula
	SELECT e.codi_equip 
	FROM equip e 
	WHERE e.codi_equip = in_equip 
	INTO codi_equip;
	
	SELECT t.any
	FROM temporada t
	WHERE t.any = temporada
	INTO check_temporada;
	
	-- Insertar a la taula
	
	INSERT INTO cotxe(model,any_debut) VALUES (model, any_debut);
	
	
	-- Actualitzar log
	
	SET valor_antic = " - ";
	SET codi_cotxe_afegit = LAST_INSERT_ID();
	SET valor_nou = CONCAT_WS(', ', codi_cotxe_afegit, model, any_debut);
	SET consulta_cotxe = CONCAT("alta_cotxe ('",model,"',",any_debut," '",in_equip,"')");
	
	INSERT INTO log(operacio, taula,antic_valor,nou_valor,data,consulta) VALUES ('ALTA', 'cotxe', valor_antic, valor_nou, NOW(), consulta_cotxe);
	
	
	-- Insertar a la taula
	
	INSERT INTO cotxe_equip VALUES (codi_cotxe_afegit,codi_equip, temporada);
	
	
	-- Actualitzar log
	
	SET valor_nou = CONCAT_WS(', ', codi_cotxe_afegit, codi_equip, temporada);
	SET consulta_equip = CONCAT("INDIRECTE -> alta_cotxe ('",model,"',",any_debut," '",in_equip,"')");
	
	INSERT INTO log(operacio, taula,antic_valor,nou_valor,data,consulta) VALUES ('ALTA', 'cotxe_equip', valor_antic, valor_nou, NOW(), consulta_equip);


END//
DELIMITER ;


-- Volcando estructura para procedimiento fia.alta_cursa
DELIMITER //
CREATE DEFINER=`root`@`localhost` PROCEDURE `alta_cursa`(IN `in_circuit` MEDIUMINT(8), IN `in_pilot` MEDIUMINT(8), IN `in_competicio` SMALLINT(5), IN `in_posicio` SMALLINT, IN `data_cursa` DATE, OUT `RSP` VARCHAR(250))
BEGIN

declare check_circuit MEDIUMINT(8);
declare check_pilot MEDIUMINT(8);
declare check_competicio SMALLINT(5);
declare check_temporada YEAR;
declare check_posicio SMALLINT;
declare valor_antic VARCHAR (10);
declare valor_nou VARCHAR (512);
declare any_temporada YEAR;
declare consulta VARCHAR(512);
declare state VARCHAR(50);
declare msg VARCHAR(250);

declare aux_punts MEDIUMINT;
declare total_punts MEDIUMINT;

declare max_data SMALLINT;
declare loop_data SMALLINT;
declare current_data DATE;
declare current_punts SMALLINT;
declare current_total_punts SMALLINT;

declare pilot_retirat BOOL;
declare current_curses_finalitzades MEDIUMINT;
declare current_victories MEDIUMINT;


-- Control d'errors

DECLARE EXIT HANDLER FOR 1062 
BEGIN
GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
SET RSP := CONCAT('ERROR: Codi ',state,'. Possiblement la cursa ja existeix a la taula \'cursa\'');
END;

DECLARE EXIT HANDLER FOR NOT FOUND
 BEGIN
 	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO;
   SET RSP := CONCAT('ERROR: Not found. Codi ',state,' .Possiblement el pilot (\'',in_pilot,'\'), el codi del circuit (\'',in_circuit,'\') o el codi de la competicio (\'',in_competicio,',', any_temporada,'\') no sigui correcte.');
 END;

DECLARE EXIT HANDLER FOR SQLWARNING 
BEGIN 
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Warning. Codi ',state,'. Missatge original: ',msg);
END;

DECLARE CONTINUE HANDLER FOR SQLEXCEPTION
BEGIN
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Desconegut. Codi ',state,'. Missatge original: ',msg);
END;

	SET RSP := 'OK';
	SET pilot_retirat = 0;
	SET any_temporada = EXTRACT(YEAR from data_cursa);
	
	-- Es fa servir SELECT INTO per tal de llençar l'excepcio  NOT FOUND si no es troba el codi a la taula
	SELECT ci.codi_circuit 
	FROM circuit ci 
	WHERE ci.codi_circuit = in_circuit 
	INTO check_circuit;
	
	SELECT pi.codi_persona 
	FROM pilot pi 
	JOIN persona pe 
	ON pe.codi_persona = pi.codi_persona 
	WHERE pe.codi_persona = in_pilot 
	INTO check_pilot;
	
	SELECT co.codi_competicio 
	FROM competicio co 
	WHERE co.codi_competicio = in_competicio 
	INTO check_competicio;
	
	SELECT temporada 
	FROM competicio_temporada 
	WHERE temporada = any_temporada
	AND competicio = in_competicio 
	LIMIT 1 
	INTO check_temporada;
	
		
	-- Insertar a la taula
	
	INSERT INTO cursa(circuit,pilot,competicio,posicio,data_cursa) VALUES (in_circuit, in_pilot, in_competicio, in_posicio, data_cursa);

		
	-- Actualitzar log
	
	SET consulta = CONCAT("alta_cursa ('",in_circuit,"', '",in_pilot,"', '",in_competicio,"', ",in_posicio,", ",data_cursa,")");
	SET valor_antic = " - ";
	SET valor_nou = CONCAT_WS(', ', in_circuit, in_pilot, in_competicio, in_posicio, data_cursa);
	
	INSERT INTO log(operacio, taula,antic_valor,nou_valor,data,consulta) VALUES ('ALTA', 'cursa', valor_antic, valor_nou, NOW(),consulta);
	
	 CASE in_posicio
      WHEN 1 THEN 
			SET total_punts = 25;
			
			
			-- Actualitzar estadistiques sobre curses guanyades
			
			SET current_victories = (SELECT guanyades FROM dim_resultat dr WHERE dr.pilot = check_pilot );
			
			IF current_victories IS NULL THEN
				INSERT INTO dim_resultat (pilot, guanyades) 
				VALUES (check_pilot, 1);
			ELSE
				UPDATE dim_resultat 
				SET guanyades = guanyades + 1 
				WHERE pilot = check_pilot;
		   END IF;	
			
		   DELETE FROM fact_table_1
			WHERE camp_control = '0xCG';
			
			INSERT INTO fact_table_1(pilot, curses_guanyades, camp_control) 
			SELECT dr.pilot, dr.guanyades, '0xCG' 
			FROM dim_resultat dr 
			ORDER BY dr.guanyades DESC
		LIMIT 10;

      WHEN 2 THEN SET total_punts = 15;
      WHEN 3 THEN SET total_punts = 5;
      WHEN 0 THEN 
      	SET pilot_retirat = 1;
      	SET total_punts = 0;
   ELSE
   	BEGIN
   	SET total_punts = 0;
   	END;
   END CASE;

	
	-- Actualitzar estadistiques sobre curses finalitzades
	
	IF pilot_retirat = 0 THEN
		
		SET current_curses_finalitzades = (SELECT finalitzades FROM dim_resultat dr WHERE dr.pilot = check_pilot );
		
		IF current_curses_finalitzades IS NULL THEN
			INSERT INTO dim_resultat (pilot, finalitzades) 
			VALUES (check_pilot, 1);
		ELSE
			UPDATE dim_resultat 
			SET finalitzades = finalitzades + 1 
			WHERE pilot = check_pilot;
	   END IF;	
		
	   DELETE FROM fact_table_1 
		WHERE camp_control = '0xCF';
		
		INSERT INTO fact_table_1(pilot, curses_finalitzades, camp_control) 
		SELECT dr.pilot, dr.finalitzades, '0xCF' 
		FROM dim_resultat dr 
		ORDER BY dr.finalitzades DESC
		LIMIT 10;
		
	END IF;
	

	-- Actualitzar estadistica classificacio pilots & posicio pilots
	-- La posicio s'escriu directament a la fact table mitjançant un comptador a la select de la taula dim_classificacio a l'hora d'insertar
		
	SET aux_punts = (SELECT punts FROM dim_classificacio WHERE pilot = in_pilot AND competicio = in_competicio AND temporada = any_temporada);
	
	IF aux_punts IS NULL THEN
		INSERT INTO dim_classificacio (competicio, temporada, pilot, punts) 
		VALUES (in_competicio, any_temporada, in_pilot, total_punts);
	ELSE
		UPDATE dim_classificacio 
		SET punts = punts + total_punts
		WHERE pilot = in_pilot 
		AND competicio = in_competicio 
		AND temporada = any_temporada;
   END IF;	
	
   DELETE FROM fact_table_1 
	WHERE camp_control = '0xCP' 
	AND competicio = in_competicio 
	AND temporada = any_temporada ;
   
	INSERT INTO fact_table_1(competicio, temporada, pilot, punts_pilot, posicio_pilot, camp_control) 
   SELECT dc.competicio, dc.temporada, dc.pilot, dc.punts, @rownum:=@rownum+1 posicio, '0xCP' 
	FROM (SELECT @rownum:=0) r, dim_classificacio dc 
	WHERE competicio = in_competicio 
	AND temporada = any_temporada 
	ORDER BY dc.punts DESC;
	
		
	-- Actualitzar estadistica de millor evolucio d'un pilot

	SELECT punts
	INTO current_total_punts
	FROM dim_classificacio
	WHERE temporada = any_temporada
	AND competicio = in_competicio
	AND pilot = in_pilot;
		
	IF(SELECT punts_pilot FROM fact_table_1 ft WHERE ft.camp_control = '0xEP' AND ft.data_cursa = data_cursa AND ft.pilot = in_pilot) IS NULL THEN
		INSERT INTO fact_table_1 (competicio, temporada, pilot, data_cursa, punts_pilot,camp_control) VALUES (in_competicio, any_temporada, in_pilot, data_cursa, current_total_punts, '0xEP');
	END IF;
	
	IF(SELECT punts_pilot FROM fact_table_1 ft WHERE ft.camp_control = '0xEP' AND ft.pilot = in_pilot AND ft.data_cursa != data_cursa AND YEAR(ft.data_cursa) = any_temporada LIMIT 1) IS NOT NULL THEN
		SELECT count(*) 
		INTO max_data 
		FROM (
			SELECT ft.data_cursa FROM fact_table_1 ft 
			WHERE ft.camp_control = '0xEP' 
			AND ft.pilot = in_pilot 
			AND ft.data_cursa != data_cursa 
			AND data_millora IS NULL
			AND YEAR(ft.data_cursa) = any_temporada
		)count_dates;	
				
		SET loop_data = 0;
			
		WHILE loop_data < max_data DO
			
			SELECT ft.data_cursa, ft.punts_pilot
			INTO current_data, current_punts
			FROM fact_table_1 ft 
			WHERE ft.camp_control = '0xEP' 
			AND ft.pilot = in_pilot 
			AND ft.data_cursa != data_cursa 
			AND data_millora IS NULL
			AND YEAR(ft.data_cursa) = any_temporada
			LIMIT loop_data,1;
			
			INSERT INTO fact_table_1 (competicio, temporada, pilot, data_cursa, data_millora, punts_pilot, camp_control) 
			VALUES (in_competicio, any_temporada, in_pilot, current_data, data_cursa, current_total_punts - current_punts, '0xEP');
			
			SET loop_data = loop_data +1;
		END WHILE;
	END IF;
	
END//
DELIMITER ;


-- Volcando estructura para procedimiento fia.alta_director
DELIMITER //
CREATE DEFINER=`root`@`localhost` PROCEDURE `alta_director`(IN `in_nom` VARCHAR(100), IN `in_pais` CHAR(2), IN `in_equip` MEDIUMINT, IN `in_estudis` TEXT, IN `in_debut` YEAR, OUT `RSP` VARCHAR(250))
BEGIN

declare codi_equip SMALLINT(5);
declare codi_nacionalitat CHAR(2);
declare codi_persona_afegida MEDIUMINT(8);
declare valor_antic VARCHAR (10);
declare valor_nou VARCHAR (512);
declare consulta_director VARCHAR(512);
declare consulta_persona VARCHAR(512);
declare state VARCHAR(50);
declare msg VARCHAR(250);


-- Control d'errors

DECLARE EXIT HANDLER FOR 1062 
BEGIN
GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
SET RSP := CONCAT('ERROR: Codi ',state,'. Possiblement el director ',codi_persona_afegida,' (',in_nom,') o l\'equip (',in_equip,') ja existeix a la taula \'director\'');
END;

DECLARE EXIT HANDLER FOR NOT FOUND
 BEGIN
 	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
   SET RSP := CONCAT('ERROR: Not found. Codi ',state,'.Possiblement el nom de l\'equip (',in_equip,') o el codi del pais (',in_pais,') no sigui correcte.');
 END;

DECLARE EXIT HANDLER FOR SQLWARNING 
BEGIN 
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Warning. Codi ',state,'. Missatge original: ',msg);
END;

DECLARE CONTINUE HANDLER FOR SQLEXCEPTION
BEGIN
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Desconegut. Codi ',state,'. Missatge original: ',msg);
END;


	SET RSP := 'OK';
	
	-- Es fa servir SELECT INTO per tal de llençar l'excepcio  NOT FOUND si no es troba el codi a la taula.
	SELECT e.codi_equip 
	FROM equip e 
	WHERE e.codi_equip = in_equip 
	INTO codi_equip;
	
	SELECT p.codi_pais 
	FROM pais p 
	WHERE p.codi_pais = in_pais 
	INTO codi_nacionalitat;
	
	-- Si la persona no existeix, cal afegir-la abans que director.
	IF in_nom NOT IN (SELECT nom FROM persona p WHERE p.nom = in_nom) THEN
		INSERT INTO persona(nom,nacionalitat) VALUES (in_nom, codi_nacionalitat);
		SET codi_persona_afegida = LAST_INSERT_ID();
	ELSE
		SET codi_persona_afegida = (SELECT p.codi_persona FROM persona p WHERE p.nom = in_nom);
	END IF;

	
	-- Insertar a la taula
	
	INSERT INTO director VALUES (codi_persona_afegida, codi_equip, in_estudis, in_debut);
	SET valor_nou = CONCAT_WS(', ', codi_persona_afegida, codi_equip, in_estudis, in_debut);
	INSERT INTO log(operacio, taula,antic_valor,nou_valor,data,consulta) VALUES ('ALTA', 'pilot', valor_antic, valor_nou, NOW(), consulta_director);
		
		
	-- Actualitzar log
	
	SET consulta_persona = CONCAT("INDIRECTE -> alta_director ('",in_nom,"', '",in_pais,"', '",in_equip,"', '",in_estudis,"', ",in_debut,")");
	SET consulta_director = CONCAT("alta_director ('",in_nom,"', '",in_pais,"', '",in_equip,"', '",in_estudis,"', ",in_debut,")");
	SET valor_antic = " - ";
	SET valor_nou = CONCAT_WS(', ', codi_persona_afegida, in_nom, codi_nacionalitat);
	
	INSERT INTO log(operacio, taula,antic_valor,nou_valor,data,consulta) VALUES ('ALTA', 'persona', valor_antic, valor_nou, NOW(), consulta_persona);
	
END//
DELIMITER ;


-- Volcando estructura para procedimiento fia.alta_emissora
DELIMITER //
CREATE DEFINER=`root`@`localhost` PROCEDURE `alta_emissora`(IN `nom` VARCHAR(50), IN `nacionalitat` CHAR(2), OUT `RSP` VARCHAR(250))
BEGIN
declare valor_antic VARCHAR (10);
declare valor_nou VARCHAR(512);
declare codi_nacionalitat CHAR(2);
declare consulta VARCHAR(512);
declare state VARCHAR(50);
declare msg VARCHAR(250);


-- Control d'errors

DECLARE EXIT HANDLER FOR 1062 
BEGIN
GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
SET RSP := CONCAT('ERROR: Codi ',state,'. Possiblement el nom de l\'emissora (',nom,') en aquest pais (',codi_nacionalitat,') ja existeix a la taula \'emissora\'');
END;

DECLARE EXIT HANDLER FOR NOT FOUND
 BEGIN
 	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
   SET RSP := CONCAT('ERROR: Not found. Codi ',state,'. Possiblement el codi del pais (',nacionalitat,') no sigui correcte.' );
 END;

DECLARE EXIT HANDLER FOR SQLWARNING 
BEGIN 
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Warning. Codi ',state,'. Missatge original: ',msg);
END;

DECLARE CONTINUE HANDLER FOR SQLEXCEPTION
BEGIN
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Desconegut. Codi ',state,'. Missatge original: ',msg);
END;


	SET RSP := 'OK';
	SET consulta = CONCAT("alta_emissora ('",nom,"', '",nacionalitat,"')");
	SELECT p.codi_pais FROM pais p WHERE p.codi_pais = nacionalitat INTO codi_nacionalitat;
	
	
	-- Insertar a la taula
	
	INSERT INTO emissora(nom,pais) VALUES (nom, codi_nacionalitat);
	
		
	-- Actualitzar log
	
	SET valor_nou = CONCAT_WS(', ',LAST_INSERT_ID(),nom, codi_nacionalitat);
	SET consulta = CONCAT("alta_emissora ('",nom,"', '",nacionalitat,"')");
	SET valor_antic = " - ";

	INSERT INTO log(operacio, taula,antic_valor,nou_valor,data,consulta) VALUES ('ALTA', 'emissora', valor_antic, valor_nou, NOW(), consulta);

END//
DELIMITER ;


-- Volcando estructura para procedimiento fia.alta_equip
DELIMITER //
CREATE DEFINER=`root`@`localhost` PROCEDURE `alta_equip`(IN `nom` VARCHAR(50), IN `seu` VARCHAR(100), IN `debut` DATE, OUT `RSP` VARCHAR(250))
BEGIN
declare valor_antic VARCHAR (10);
declare valor_nou VARCHAR (512);
declare consulta VARCHAR(512);
declare state VARCHAR(50);
declare msg VARCHAR(250);

-- Control d'errors

DECLARE EXIT HANDLER FOR 1062 
BEGIN
GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
SET RSP := CONCAT('ERROR: Codi ',state,'. Possiblement el nom de l\'equip (',nom,') ja existeix a la taula \'equip\'');
END;

DECLARE EXIT HANDLER FOR SQLWARNING 
BEGIN 
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Warning. Codi ',state,'. Missatge original: ',msg);
END;

DECLARE CONTINUE HANDLER FOR SQLEXCEPTION
BEGIN
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Desconegut. Codi ',state,'. Missatge original: ',msg);
END;

	
	SET RSP := 'OK';
	
	
	-- Insertar a la taula
	
	INSERT INTO equip(nom,seu,data_debut) VALUES (nom, seu, debut);
	

	-- Actualitzar log
	
	set valor_nou = CONCAT_WS(', ',LAST_INSERT_ID(),nom, seu, debut);
	set valor_antic = " - ";
	SET consulta = CONCAT("alta_equip ('",nom,"', '",seu,"', ",debut,")");
	
	INSERT INTO log(operacio, taula,antic_valor,nou_valor,data,consulta) VALUES ('ALTA', 'equip', valor_antic, valor_nou, NOW(), consulta);

END//
DELIMITER ;


-- Volcando estructura para procedimiento fia.alta_fabricant
DELIMITER //
CREATE DEFINER=`root`@`localhost` PROCEDURE `alta_fabricant`(IN `nom` VARCHAR(50), IN `debut` YEAR, OUT `RSP` VARCHAR(250))
BEGIN
declare valor_antic VARCHAR (10);
declare valor_nou VARCHAR (512);
declare consulta VARCHAR(512);
declare state VARCHAR(50);
declare msg VARCHAR(250);


-- Control d'errors

DECLARE EXIT HANDLER FOR 1062 
BEGIN
GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
SET RSP := CONCAT('ERROR: Codi ',state,'. Possiblement el nom del fabricant (',nom,') ja existeix a la taula \'fabricant\'');
END;

DECLARE EXIT HANDLER FOR SQLWARNING 
BEGIN 
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Warning. Codi ',state,'. Missatge original: ',msg);
END;

DECLARE CONTINUE HANDLER FOR SQLEXCEPTION
BEGIN
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Desconegut. Codi ',state,'. Missatge original: ',msg);
END;

	SET RSP := 'OK';
	
	
	-- Insertar a la taula
	
	INSERT INTO fabricant(nom,any_debut) VALUES (nom, debut);
	
	
	-- Actualitzar log
	
	SET valor_antic = " - ";
	SET valor_nou = CONCAT_WS(', ',LAST_INSERT_ID(),nom, debut);
	SET consulta = CONCAT("alta_fabricant ('",nom,"', ",debut,")");
	
	INSERT INTO log(operacio, taula,antic_valor,nou_valor,data,consulta) VALUES ('ALTA', 'fabricant', valor_antic, valor_nou, NOW(), consulta);

END//
DELIMITER ;


-- Volcando estructura para procedimiento fia.alta_pais
DELIMITER //
CREATE DEFINER=`root`@`localhost` PROCEDURE `alta_pais`(IN `codi` CHAR(2), IN `nom` VARCHAR(50), OUT `RSP` VARCHAR(250))
BEGIN
declare valor_antic VARCHAR (10);
declare valor_nou VARCHAR (512);
declare consulta VARCHAR(512);
declare state VARCHAR(50);
declare msg VARCHAR(250);


-- Control d'errors

DECLARE EXIT HANDLER FOR 1062 
BEGIN
GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
SET RSP := CONCAT('ERROR: Codi ',state,'. Possiblement el nom del pais (',nom,') o el codi (',codi,') ja existeix a la taula \'pais\'.');
END;

DECLARE EXIT HANDLER FOR SQLWARNING 
BEGIN 
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Warning. Codi ',state,'. Missatge original: ',msg);
END;

DECLARE CONTINUE HANDLER FOR SQLEXCEPTION
BEGIN
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Desconegut. Codi ',state,'. Missatge original: ',msg);
END;

	SET RSP := 'OK';
	
	
	-- Insertar a la taula
	
	INSERT INTO pais VALUES (codi, nom);
	
	
	-- Actualitzar log
	
	SET consulta = CONCAT("alta_pais ('",codi,"', '",nom,"')");	
	SET valor_antic = " - ";
	SET valor_nou = CONCAT_WS(', ',LAST_INSERT_ID(),codi, nom);
	
	INSERT INTO log(operacio, taula,antic_valor,nou_valor,data,consulta) VALUES ('ALTA', 'pais', valor_antic, valor_nou, NOW(), consulta);
 END//
DELIMITER ;


-- Volcando estructura para procedimiento fia.alta_patrocinador
DELIMITER //
CREATE DEFINER=`root`@`localhost` PROCEDURE `alta_patrocinador`(IN `sector_patrocinador` SMALLINT, IN `nom_patrocinador` VARCHAR(50), OUT `RSP` VARCHAR(250))
BEGIN
declare valor_antic VARCHAR (10);
declare valor_nou VARCHAR (512);
declare codi_sector MEDIUMINT;
declare consulta VARCHAR(512);
declare state VARCHAR(50);
declare msg VARCHAR(250);


-- Control d'errors

DECLARE EXIT HANDLER FOR 1062 
BEGIN
GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
SET RSP := CONCAT('ERROR: Codi ',state,'. Possiblement nom del patrocinador (',nom_patrocinador,') ja existeix a la taula \'patrocinador\'');
END;

DECLARE EXIT HANDLER FOR NOT FOUND
 BEGIN
 	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
   SET RSP := CONCAT('ERROR: Not found. Codi ',state,'. Possiblement el sector del patrocinador (',sector_patrocinador,') no sigui correcte.' );
 END;

DECLARE EXIT HANDLER FOR SQLWARNING 
BEGIN 
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Warning. Codi ',state,'. Missatge original: ',msg);
END;

DECLARE CONTINUE HANDLER FOR SQLEXCEPTION
BEGIN
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Desconegut. Codi ',state,'. Missatge original: ',msg);
END;


	SET RSP := 'OK';
		
	SELECT sector.codi_sector FROM sector WHERE sector.codi_sector = sector_patrocinador INTO codi_sector;
	
	
	-- Insertar a la taula
	
	INSERT INTO patrocinador(sector,nom) VALUES (codi_sector, nom_patrocinador);
	
	
	-- Actualitzar log
	
	SET valor_antic = " - ";
	SET valor_nou = CONCAT_WS(', ',LAST_INSERT_ID(),codi_sector, nom_patrocinador);
	SET consulta = CONCAT("alta_patrocinador ('",sector_patrocinador,"', '",nom_patrocinador,"')");
	
	INSERT INTO log(operacio, taula,antic_valor,nou_valor,data,consulta) VALUES ('ALTA', 'patrocinador', valor_antic, valor_nou, NOW(), consulta);

END//
DELIMITER ;


-- Volcando estructura para procedimiento fia.alta_patrocini
DELIMITER //
CREATE DEFINER=`root`@`localhost` PROCEDURE `alta_patrocini`(IN `in_patrocinador` MEDIUMINT, IN `in_equip` SMALLINT, IN `temporada` YEAR, IN `cost` INT, OUT `RSP` VARCHAR(250))
BEGIN
declare codi_patrocinador MEDIUMINT(8);
declare codi_equip SMALLINT(5);
declare temporada_check YEAR;
declare valor_antic VARCHAR (10);
declare valor_nou VARCHAR (512);
declare consulta VARCHAR(512);
declare state VARCHAR(50);
declare msg VARCHAR(250);

declare aux_existeix INT;
declare aux_limit TINYINT;
declare aux_delete SMALLINT;


-- Control d'errors

DECLARE EXIT HANDLER FOR 1644 
BEGIN
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT("ERROR: Codi ",state,". ",msg);
END;

DECLARE EXIT HANDLER FOR 1062 
BEGIN
GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO;
SET RSP := CONCAT('ERROR: Codi ',state,'. Ja existeix aquest patrocini (',in_patrocinador,', ',in_equip,'), per aquesta temporada (',temporada,') a la taula \'patrocini\'.');
END;

DECLARE EXIT HANDLER FOR NOT FOUND
 BEGIN
 	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO;
   SET RSP := CONCAT('ERROR: Not found. Codi ',state,' .Possiblement l\'equip (\'',in_equip,'\'), el patrocinador (\'',in_patrocinador,'\'), o la temporada (',temporada,') no existeixen al sistema.');
 END;

DECLARE EXIT HANDLER FOR SQLWARNING 
BEGIN 
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Warning. Codi ',state,'. Missatge original: ',msg);
END;

DECLARE EXIT HANDLER FOR SQLEXCEPTION
BEGIN
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Desconegut. Codi ',state,'. Missatge original: ',msg);
END;


	SET RSP := 'OK';
		
	-- Es fa servir SELECT INTO per tal de llençar l'excepcio  NOT FOUND si no es troba el codi a la taula.
	SELECT p.codi_patrocinador 
	FROM patrocinador p 
	WHERE p.codi_patrocinador = in_patrocinador 
	INTO codi_patrocinador;
	
	SELECT e.codi_equip 
	FROM equip e 
	WHERE e.codi_equip = in_equip 
	INTO codi_equip;
	
	SELECT t.any 
	FROM temporada t 
	WHERE t.any = temporada 
	INTO temporada_check;
	
	INSERT INTO patrocini 
	VALUES (codi_patrocinador, codi_equip, temporada, cost);
	
	
	-- Actualitzar log
	
	SET consulta = CONCAT("alta_patrocini ('",in_patrocinador,"', '",in_equip,"', ",temporada,", ",cost,")");
	SET valor_antic = " - ";
	SET valor_nou = CONCAT_WS(', ', codi_patrocinador, codi_equip, temporada, cost);
	INSERT INTO log(operacio, taula,antic_valor,nou_valor,data,consulta) VALUES ('ALTA', 'patrocini', valor_antic, valor_nou, NOW(), consulta);
	

	-- Actualitzar estadistiques
	
	SET aux_existeix = (SELECT cost FROM dim_patrocini  WHERE dim_patrocini.patrocinador = codi_patrocinador);
	
	IF aux_existeix IS NULL THEN
		INSERT INTO dim_patrocini (patrocinador, cost) 
		VALUES (codi_patrocinador, cost);
	ELSE
		UPDATE dim_patrocini 
		SET dim_patrocini.cost = dim_patrocini.cost + cost 
		WHERE dim_patrocini.patrocinador = codi_patrocinador;
   END IF;	
   
   DELETE FROM fact_table_1 WHERE fact_table_1.camp_control = '0xPA';
   INSERT INTO fact_table_1(patrocinador, cost_patrocini, camp_control) 
   SELECT dp.patrocinador, dp.cost, '0xPA' 
	FROM dim_patrocini dp 
	ORDER BY dp.cost DESC 
	LIMIT 5;

END//
DELIMITER ;


-- Volcando estructura para procedimiento fia.alta_pilot
DELIMITER //
CREATE DEFINER=`root`@`localhost` PROCEDURE `alta_pilot`(IN `nom` VARCHAR(100), IN `nacionalitat` CHAR(2), IN `in_equip` MEDIUMINT, IN `llicencia` VARCHAR(8), IN `inici` YEAR, OUT `RSP` VARCHAR(250))
BEGIN

declare codi_equip SMALLINT(5);
declare codi_nacionalitat CHAR(2);
declare codi_persona_afegida MEDIUMINT(8);
declare valor_antic VARCHAR (10);
declare valor_nou VARCHAR (512);
declare consulta_pilot VARCHAR(512);
declare consulta_persona VARCHAR(512);
declare state VARCHAR(50);
declare msg VARCHAR(250);


-- Control d'errors

DECLARE EXIT HANDLER FOR 1062 
BEGIN
GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
SET RSP := CONCAT('ERROR: Codi ',state,'. Possiblement el pilot ',codi_persona_afegida,' (',nom,') ja existeix a la taula \'pilot\'');
END;

DECLARE EXIT HANDLER FOR NOT FOUND
 BEGIN
 	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
   SET RSP := CONCAT('ERROR: Not found. Codi ',state,'. Pot ser que el codi de l\'equip (',in_equip,') o el codi del pais (',nacionalitat,') no sigui correcte.');
 END;

DECLARE EXIT HANDLER FOR SQLWARNING 
BEGIN 
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Warning. Codi ',state,'. Missatge original: ',msg);
END;

DECLARE CONTINUE HANDLER FOR SQLEXCEPTION
BEGIN
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Desconegut. Codi ',state,'. Missatge original: ',msg);
END;


	SET RSP := 'OK';
	
	-- Es fa servir SELECT INTO per tal de llençar l'excepcio  NOT FOUND si no es troba el codi a la taula.
	SELECT e.codi_equip FROM equip e WHERE e.codi_equip = in_equip INTO codi_equip;
	SELECT p.codi_pais FROM pais p WHERE p.codi_pais = nacionalitat INTO codi_nacionalitat;
	
	-- Si la persona no existeix, cal afegir-la abans que director.
	IF nom NOT IN (SELECT nom FROM persona p WHERE p.nom = nom) THEN
		INSERT INTO persona(nom,nacionalitat) VALUES (nom, codi_nacionalitat);
		SET codi_persona_afegida = LAST_INSERT_ID();
	ELSE
		SET codi_persona_afegida = (SELECT p.codi_persona FROM persona p WHERE p.nom = nom);
	END IF;
	
	
	-- Actualitzar log
		
	SET valor_antic = " - ";
	SET consulta_persona = CONCAT("INDIRECTE -> alta_director ('",nom,"', '",nacionalitat,"', '",in_equip,"', '",llicencia,"', ",inici,")");
	SET valor_nou = CONCAT_WS(', ', codi_persona_afegida, nom, codi_nacionalitat);
	INSERT INTO log(operacio, taula,antic_valor,nou_valor,data,consulta) VALUES ('ALTA', 'persona', valor_antic, valor_nou, NOW(), consulta_persona);
	
	
	-- Insertar a la taula
	
	INSERT INTO pilot VALUES (codi_persona_afegida, codi_equip, llicencia, inici);
			
	
	-- Actualitzar log
	
	SET consulta_pilot = CONCAT("alta_director ('",nom,"', '",nacionalitat,"', '",in_equip,"', '",llicencia,"', ",inici,")");
	SET valor_nou = CONCAT_WS(', ', codi_persona_afegida, codi_equip, llicencia, inici);
	INSERT INTO log(operacio, taula,antic_valor,nou_valor,data,consulta) VALUES ('ALTA', 'pilot', valor_antic, valor_nou, NOW(), consulta_pilot);


END//
DELIMITER ;


-- Volcando estructura para procedimiento fia.alta_pilot_cotxe
DELIMITER //
CREATE DEFINER=`root`@`localhost` PROCEDURE `alta_pilot_cotxe`(IN `in_pilot` MEDIUMINT, IN `in_cotxe` MEDIUMINT, IN `in_temporada` YEAR, OUT `RSP` VARCHAR(250))
BEGIN

declare valor_antic VARCHAR (512);
declare valor_nou VARCHAR (512);
declare consulta VARCHAR(512);
declare msg VARCHAR(250);
declare state VARCHAR(50);

declare check_pilot MEDIUMINT;
declare check_cotxe MEDIUMINT;
declare check_temporada YEAR;


DECLARE EXIT HANDLER FOR 1062 
BEGIN
GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO;
SET RSP := CONCAT('ERROR: Codi ',state,'. Possiblement ja existeix aquesta entrada a la taula pilot_cotxe.');
END;

DECLARE EXIT HANDLER FOR NOT FOUND
BEGIN
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO;
	SET RSP := CONCAT('ERROR: Not found. Codi ',state,' .Possiblement el pilot (\'',in_pilot,'\') o el cotxe (\'',in_cotxe,'\') no siguin correctes.');
END;

DECLARE EXIT HANDLER FOR SQLWARNING 
BEGIN 
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Warning. Codi ',state,'. Missatge original: ',msg);
END;

DECLARE EXIT HANDLER FOR SQLEXCEPTION
BEGIN
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Desconegut. Codi ',state,'. Missatge original: ',msg);
END;


		
	SET RSP := 'OK';

	-- Es fa servir SELECT INTO per tal de llençar l'excepcio  NOT FOUND si no es troba el codi a la taula
	SELECT p.codi_persona
	FROM pilot p
	WHERE p.codi_persona = in_pilot 
	INTO check_pilot;
	
	SELECT c.codi_cotxe
	FROM cotxe c
	WHERE c.codi_cotxe = in_cotxe 
	INTO check_cotxe;
	
	SELECT t.any
	FROM temporada t
	WHERE t.any = in_temporada 
	INTO check_temporada;
	
	
	-- Insertar a la taula
	
	INSERT INTO pilot_cotxe VALUES (in_pilot, in_cotxe, in_temporada);
	
	
	-- Actualitzar log
	
	set valor_antic = " - ";
	SET valor_nou = CONCAT_WS(',',in_pilot, in_cotxe, in_temporada);
	SET consulta = CONCAT("alta_pilot_cotxe ('",in_pilot,"', '",in_cotxe,"', '",in_temporada,"')");
	
	INSERT INTO log(operacio, taula,antic_valor,nou_valor,data,consulta) VALUES ('ALTA', 'competicio_temporada', valor_antic, valor_nou, NOW(), consulta);
		
END//
DELIMITER ;


-- Volcando estructura para procedimiento fia.alta_retransmissio
DELIMITER //
CREATE DEFINER=`root`@`localhost` PROCEDURE `alta_retransmissio`(IN `in_emissora` MEDIUMINT, IN `pais_emissio` CHAR(2), IN `in_competicio` VARCHAR(50), OUT `RSP` VARCHAR(250))
BEGIN
declare valor_antic VARCHAR (10);
declare valor_nou VARCHAR (512);
declare codi_nacionalitat CHAR(2);
declare consulta VARCHAR(512);
declare codi_emissora SMALLINT(5);
declare codi_competicio SMALLINT(5);
declare state VARCHAR(50);
declare msg VARCHAR(250);


-- Control d'errors

DECLARE EXIT HANDLER FOR 1062 
BEGIN
GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO;
SET RSP := CONCAT('ERROR: Codi ',state,'. Ja existeix aquesta retransmissio (',in_emissora,', ',in_competicio,'), a la taula \'retransmissio\'');
END;

DECLARE EXIT HANDLER FOR NOT FOUND
 BEGIN
 	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO;
   SET RSP := CONCAT('ERROR: Not found. Codi ',state,' .Possiblement l\'emissora (\'',in_emissora,'\'), 
							el codi del pais (\'',pais_emissio,'\') o la competicio (\'',in_competicio,'\') no sigui correcte, o ',in_emissora,' no emet a ',pais_emissio,'.');
 END;

DECLARE EXIT HANDLER FOR SQLWARNING 
BEGIN 
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Warning. Codi ',state,'. Missatge original: ',msg);
END;

DECLARE EXIT HANDLER FOR SQLEXCEPTION
BEGIN
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Desconegut. Codi ',state,'. Missatge original: ',msg);
END;


	SET RSP := 'OK';
	
	-- Es fa servir SELECT INTO per tal de llençar l'excepcio  NOT FOUND si no es troba el codi a la taula
	SELECT c.codi_competicio FROM competicio c WHERE c.codi_competicio = in_competicio INTO codi_competicio;
	SELECT p.codi_pais FROM pais p WHERE p.codi_pais = pais_emissio INTO codi_nacionalitat;
	SELECT e.codi_emissora FROM emissora e WHERE e.codi_emissora = in_emissora AND e.pais = pais_emissio INTO codi_emissora;
	

	-- Insertar a la taula

	INSERT INTO retransmissio VALUES (codi_emissora, codi_competicio);
				
	
	-- Actualitzar log
	
	SET consulta = CONCAT("alta_retransmissio ('",in_emissora,"', '",codi_nacionalitat,"', '",in_competicio,")");
	SET valor_antic = " - ";
	SET valor_nou = CONCAT_WS(', ', codi_emissora, codi_competicio);
	
	INSERT INTO log(operacio, taula,antic_valor,nou_valor,data,consulta) VALUES ('ALTA', 'retransmissio', valor_antic, valor_nou, NOW(), consulta);

END//
DELIMITER ;


-- Volcando estructura para procedimiento fia.alta_sector
DELIMITER //
CREATE DEFINER=`root`@`localhost` PROCEDURE `alta_sector`(IN `nom` VARCHAR(50), OUT `RSP` VARCHAR(250))
BEGIN
declare valor_antic VARCHAR (10);
declare valor_nou VARCHAR (512);
declare consulta VARCHAR(512);
declare state VARCHAR(50);
declare msg VARCHAR(250);


-- Control d'errors

DECLARE EXIT HANDLER FOR 1062 
BEGIN
GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
SET RSP := CONCAT('ERROR: Codi ',state,'. Possiblement el nom del sector (',nom,') ja existeix a la taula \'sector\'.');
END;

DECLARE CONTINUE HANDLER FOR SQLWARNING 
BEGIN 
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Warning. Codi ',state,'. Missatge original: ',msg);
END;

DECLARE CONTINUE HANDLER FOR SQLEXCEPTION
BEGIN
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Desconegut. Codi ',state,'. Missatge original: ',msg);
END;

	SET RSP := 'OK';


	-- Insertar a la taula

	INSERT INTO sector(nom) VALUES (nom);

	
	-- Actualitzar log

	SET consulta = CONCAT("alta_sector ('",nom,"')");
	SET valor_antic = " - ";	
	SET valor_nou = CONCAT_WS(', ', LAST_INSERT_ID(), nom);
	
	INSERT INTO log VALUES (default, 'ALTA', 'patrocini', valor_antic, valor_nou, NOW(), consulta);

END//
DELIMITER ;


-- Volcando estructura para procedimiento fia.alta_telemetria
DELIMITER //
CREATE DEFINER=`root`@`localhost` PROCEDURE `alta_telemetria`(IN `in_component` MEDIUMINT, IN `in_moment` DATETIME(6), IN `in_ambit` VARCHAR(50), IN `in_valor` FLOAT, IN `in_unitats` VARCHAR(25), OUT `RSP` VARCHAR(512))
BEGIN

declare valor_antic VARCHAR (10);
declare valor_nou VARCHAR (512);
declare check_component MEDIUMINT;
declare consulta VARCHAR(512);
declare msg VARCHAR(250);
declare state VARCHAR(50);

declare aux_temps TIME;
declare current_dades MEDIUMINT;
declare current_temperatura FLOAT(8,2);
declare current_consum_cotxe FLOAT(10,2);
declare check_cotxe SMALLINT;
declare check_equip SMALLINT;

declare current_year YEAR;
declare current_competicio SMALLINT;
declare current_pilot SMALLINT;
declare current_circuit SMALLINT;

declare comp MEDIUMINT;
declare percentatge_defectuos DECIMAL(4,1);
declare equip_max_punts SMALLINT;


-- Control d'errors

DECLARE EXIT HANDLER FOR 1644 
BEGIN
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT, comp = COLUMN_NAME;
	SET RSP := CONCAT("ERROR: Codi ",state,". ",msg," Component: ",comp);
	
	
	-- Actualitzar estadistiques de component defectuos
	
	SELECT cotxe 
	FROM component c 
	WHERE c.codi_component = comp 
	INTO check_cotxe;
	
	UPDATE dim_defectuos 
	SET defectuos = 1
	WHERE component = comp
	AND cotxe = check_cotxe;
	
	SELECT (SUM(CASE WHEN defectuos = 1 THEN 1 ELSE 0 END)/COUNT(*)*100) 
	FROM dim_defectuos WHERE cotxe = check_cotxe
	INTO percentatge_defectuos;
	
	UPDATE fact_table_1
	SET components_defectuosos = percentatge_defectuos
	WHERE cotxe = check_cotxe
	AND camp_control = '0xCD';
 		
END;

DECLARE EXIT HANDLER FOR NOT FOUND
 BEGIN
 	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
   SET RSP := CONCAT('ERROR: Not found. Codi ',state,'. Possiblement el codi del component (',in_component,') o la data (',in_moment,') no siguin correctes.' );
 END;

DECLARE EXIT HANDLER FOR SQLWARNING 
BEGIN 
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Warning. Codi ',state,'. Missatge original: ',msg);
END;

DECLARE EXIT HANDLER FOR SQLEXCEPTION
BEGIN
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Desconegut. Codi ',state,'. Missatge original: ',msg);
END;


	SET RSP := 'OK';
	SET current_year = YEAR(in_moment);
	
	-- Es fa servir SELECT INTO per tal de llençar l'excepcio  NOT FOUND si no es troba el codi a la taula
	SELECT c.codi_component 
	FROM component c 
	WHERE c.codi_component = in_component 
	INTO check_component;
	
	SELECT cotxe 
	FROM component c 
	WHERE c.codi_component = in_component 
	INTO check_cotxe;
	
	SELECT equip 
	FROM cotxe_equip 
	WHERE cotxe = check_cotxe 
	INTO check_equip;
	
	
	-- Insertar a la taula
	
	INSERT INTO telemetria(component,moment,ambit,valor,unitat_de_mesura) 
	VALUES (in_component, in_moment, in_ambit, in_valor, in_unitats);


	-- Actualitzar log
	
	SET valor_antic = " - ";
	SET valor_nou = CONCAT_WS(', ', LAST_INSERT_ID(), in_component, in_moment, in_ambit, in_valor, in_unitats);
	SET consulta = CONCAT("alta_telemetria ('",in_component,"','",in_moment,"', '",in_ambit,"', '",in_valor,"', '",in_unitats,"')");
		
	INSERT INTO log(operacio, taula,antic_valor,nou_valor,data,consulta) VALUES ('ALTA', 'telemetria', valor_antic, valor_nou, NOW(),consulta);


	-- Actualitzar estadistiques de consum per competicio i temporada.
	
 	CASE in_ambit
      WHEN 'consum' THEN
			SELECT pilot 
			INTO current_pilot 
			FROM pilot_cotxe 
			WHERE temporada = current_year 
			AND cotxe = check_cotxe;
			
			SELECT competicio 
			INTO current_competicio 
			FROM cursa 
			WHERE data_cursa = DATE(in_moment) 
			AND pilot = current_pilot;
			
			SET current_consum_cotxe = (SELECT consum FROM dim_consum WHERE temporada = current_year AND competicio = current_competicio AND cotxe =  check_cotxe);
			
			IF current_consum_cotxe IS NULL THEN
				INSERT INTO dim_consum (competicio, temporada, cotxe, consum) 
				VALUES (current_competicio, current_year, check_cotxe, in_valor);
			ELSE				
				UPDATE dim_consum dc
				SET dc.consum =  dc.consum + in_valor
				WHERE dc.temporada = current_year 
				AND dc.competicio = current_competicio
				AND dc.cotxe = check_cotxe;
		   END IF;	
		   
		   DELETE FROM fact_table_1 
			WHERE camp_control = '0xCC' 
			AND temporada = current_year 
			AND competicio = current_competicio;
			
		   INSERT INTO fact_table_1(competicio, temporada, cotxe, consum_cotxe, camp_control) 
		   SELECT dc.competicio, dc.temporada, dc.cotxe, dc.consum, '0xCC' 
			FROM dim_consum dc 
			WHERE dc.competicio = current_competicio 
			AND dc.temporada = current_year 
			ORDER BY dc.consum DESC 
			LIMIT 1;
		   
		   
		-- Actualitzar estadistiques de temperatura maxima trobada al sistema (data, la cursa, el cotxe, el pilot i el valor.)
		
      WHEN 'temperatura' THEN
      	SELECT pilot 
			INTO current_pilot 
			FROM pilot_cotxe 
			WHERE temporada = current_year 
			AND cotxe = check_cotxe;
			
			SELECT competicio, circuit 
			INTO current_competicio, current_circuit 
			FROM cursa 
			WHERE data_cursa = DATE(in_moment) 
			AND pilot = current_pilot;
			
      	SET current_temperatura = (SELECT temperatura_maxima FROM fact_table_1 WHERE camp_control = '0xTM');
      	
      	IF current_temperatura IS NULL THEN
				INSERT INTO fact_table_1 (competicio, temporada, pilot, cotxe, circuit, data_cursa, temperatura_maxima, camp_control) 
				VALUES (current_competicio, current_year, current_pilot, check_cotxe, current_circuit, DATE(in_moment), in_valor, '0xTM');
			ELSE
				IF in_valor > (SELECT temperatura_maxima FROM fact_table_1 WHERE camp_control = '0xTM') THEN
					DELETE FROM fact_table_1
					WHERE camp_control = '0xTM';
					
					INSERT INTO fact_table_1 (competicio, temporada, pilot, cotxe, circuit, data_cursa, temperatura_maxima, camp_control) 
					VALUES (current_competicio, current_year, current_pilot, check_cotxe, current_circuit, DATE(in_moment), in_valor, '0xTM');
				END IF;
		   END IF;	
      
      
      -- Actualitzar estadistiques de volta rápida per circuit
      
      WHEN 'temps per volta' THEN
			
			SELECT pilot 
			INTO current_pilot 
			FROM pilot_cotxe 
			WHERE temporada = current_year 
			AND cotxe = check_cotxe;
			
			SELECT circuit 
			INTO current_circuit 
			FROM cursa 
			WHERE data_cursa = DATE(in_moment) 
			AND pilot = current_pilot;
			
			SET aux_temps = (TIME_FORMAT(SEC_TO_TIME(in_valor/1000),'%i:%s'));

			INSERT INTO dim_volta_rapida (circuit, pilot, equip,  data, temps) 
			VALUES (current_circuit, current_pilot, check_equip, DATE(in_moment), aux_temps);

			DELETE FROM fact_table_1 WHERE circuit = current_circuit AND camp_control = '0xVR';			
			
			INSERT INTO fact_table_1 (circuit, pilot, equip, data_cursa, volta_rapida, camp_control) 		
			SELECT dvr.circuit, dvr.pilot, dvr.equip,  dvr.data, dvr.temps, '0xVR'
			FROM dim_volta_rapida dvr
			WHERE dvr.circuit = current_circuit
			ORDER BY temps 
			LIMIT 10;
   ELSE
   	BEGIN
   	
   	END;
   END CASE;


	-- Actualitzar estadistiques de volum telemetria
	
	SET current_dades = (SELECT dades_any FROM dim_telemetria WHERE temporada = current_year AND equip = check_equip);
	
	IF current_dades IS NULL THEN
		INSERT INTO dim_telemetria (temporada, equip, dades_any) 
		VALUES (current_year, check_equip, 1);
	ELSE
		UPDATE dim_telemetria dt
		SET dt.dades_any =  dt.dades_any + 1 
		WHERE dt.temporada = current_year 
		AND dt.equip = check_equip;
   END IF;	
   	
   DELETE FROM fact_table_1 WHERE camp_control = '0xTE' AND temporada = current_year;
   
   SELECT dt.equip 
   INTO equip_max_punts
   FROM dim_telemetria dt
   WHERE dt.temporada = current_year 
   ORDER BY dt.dades_any DESC
	LIMIT 1;
   
   INSERT INTO fact_table_1(temporada, equip, dades_telemetriques, camp_control) 
   SELECT dt.temporada, equip_max_punts, SUM(dt.dades_any), '0xTE' 
	FROM dim_telemetria dt 
	WHERE dt.temporada = current_year 
	ORDER BY dt.dades_any DESC
	LIMIT 1;
	
END//
DELIMITER ;


-- Volcando estructura para procedimiento fia.alta_temporada
DELIMITER //
CREATE DEFINER=`root`@`localhost` PROCEDURE `alta_temporada`(IN `any_nou` YEAR, IN `in_competicio` SMALLINT, OUT `RSP` VARCHAR(250))
BEGIN

declare valor_antic VARCHAR (10);
declare valor_nou VARCHAR (512);
declare codi_competicio SMALLINT(5);
declare consulta_temporada VARCHAR(512);
declare consulta_competicio_any VARCHAR(512);
declare msg VARCHAR(250);
declare state VARCHAR(50);


-- Control d'errors

DECLARE EXIT HANDLER FOR 1062 
BEGIN
GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO;
SET RSP := CONCAT('ERROR: Codi ',state,'. Ja existeix aquesta competicio (\'',in_competicio,'\'), a aquesta temporada (',any_nou,')');
END;

DECLARE EXIT HANDLER FOR NOT FOUND
BEGIN
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO;
	SET RSP := CONCAT('ERROR: Not found. Codi ',state,' .Possiblement la competicio (\'',in_competicio,'\'), no sigui correcte.');
END;

DECLARE EXIT HANDLER FOR SQLWARNING 
BEGIN 
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Warning. Codi ',state,'. Missatge original: ',msg);
END;

DECLARE EXIT HANDLER FOR SQLEXCEPTION
BEGIN
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Desconegut. Codi ',state,'. Missatge original: ',msg);
END;

	SET RSP := 'OK';
	SET valor_antic = " - ";
	
	-- Si l'any no existeix a la taula temporada, s'afegeix.
	IF any_nou NOT IN (SELECT t.any FROM temporada t) THEN
		INSERT INTO temporada VALUES (any_nou);
		SET consulta_temporada = CONCAT("INDIRECTE->alta_temporada (",any_nou,"', ",in_competicio,"')");
		SET valor_nou = any_nou;
		INSERT INTO log(operacio, taula,antic_valor,nou_valor,data,consulta) VALUES ('ALTA', 'temporada', valor_antic, valor_nou, NOW(), consulta_temporada);
	END IF;
	
	SET consulta_competicio_any = CONCAT("alta_temporada (",any_nou,", ",in_competicio,"')");
	SELECT co.codi_competicio FROM competicio co WHERE co.codi_competicio = in_competicio INTO codi_competicio;
	SET valor_nou = CONCAT_WS(', ', any_nou, codi_competicio);
	INSERT INTO competicio_temporada VALUES (codi_competicio, any_nou);
	INSERT INTO log(operacio, taula,antic_valor,nou_valor,data,consulta) VALUES ('ALTA', 'competicio_temporada', valor_antic, valor_nou, NOW(), consulta_competicio_any );

END//
DELIMITER ;


-- Volcando estructura para procedimiento fia.baixa_circuit
DELIMITER //
CREATE DEFINER=`root`@`localhost` PROCEDURE `baixa_circuit`(IN `codi` MEDIUMINT, OUT `RSP` VARCHAR(250))
BEGIN

declare valor_antic VARCHAR (512);
declare valor_nou VARCHAR (10);
declare consulta VARCHAR(512);
declare msg VARCHAR(250);
declare state VARCHAR(50);

declare nom VARCHAR(50);
declare sobrenom VARCHAR(50);
declare longitud SMALLINT;
declare codi_pais CHAR(2);


-- Control d'errors

DECLARE EXIT HANDLER FOR NOT FOUND
BEGIN
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO;
	SET RSP := CONCAT('ERROR: Not found. Codi ',state,' .Possiblement el codi circuit (',codi,'), no sigui correcte.');
END;

DECLARE EXIT HANDLER FOR SQLWARNING 
BEGIN 
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Warning. Codi ',state,'. Missatge original: ',msg);
END;

DECLARE EXIT HANDLER FOR SQLEXCEPTION
BEGIN
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Desconegut. Codi ',state,'. Missatge original: ',msg);
END;

	SET RSP := 'OK';
	
	SELECT c.nom, c.sobrenom, c.longitud, c.pais 
	INTO nom, sobrenom, longitud, codi_pais 
	FROM circuit c 
	WHERE c.codi_circuit = codi;

	
	-- Esborra de la taula

	DELETE FROM cursa WHERE circuit = codi;
	

	-- Actualitzar log

	SET valor_nou = " - ";
	SET consulta = CONCAT("INDIRECTE -> baixa_circuit (",codi,")");
	SET valor_antic = CONCAT("curses relatives circuit ",codi);
	
	INSERT INTO log(operacio, taula,antic_valor,nou_valor,data,consulta) VALUES ('BAIXA', 'cursa', valor_antic, valor_nou, NOW(), consulta);
	
	
	-- Esborra de la taula
	
	DELETE FROM circuit  WHERE codi_circuit = codi; 
	

	-- Actualitzar log
	
	SET consulta = CONCAT("baixa_circuit (",codi,")");
	SET valor_antic = CONCAT_WS(',', codi, nom, sobrenom, longitud, codi_pais);

	INSERT INTO log(operacio, taula,antic_valor,nou_valor,data,consulta) VALUES ('BAIXA', 'circuit', valor_antic, valor_nou, NOW(), consulta);
	
END//
DELIMITER ;


-- Volcando estructura para procedimiento fia.baixa_competicio
DELIMITER //
CREATE DEFINER=`root`@`localhost` PROCEDURE `baixa_competicio`(IN `codi` SMALLINT, OUT `RSP` VARCHAR(512))
BEGIN

declare valor_antic VARCHAR (512);
declare valor_nou VARCHAR (10);
declare consulta VARCHAR(512);
declare msg VARCHAR(250);
declare state VARCHAR(50);

declare nom VARCHAR(50);
declare acronim VARCHAR(10);
declare descripcio TEXT;


-- Control d'errors

DECLARE EXIT HANDLER FOR NOT FOUND
BEGIN
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO;
	SET RSP := CONCAT('ERROR: Not found. Codi ',state,' .Possiblement el codi competicio (',codi,'), no sigui correcte.');
END;

DECLARE EXIT HANDLER FOR SQLWARNING 
BEGIN 
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Warning. Codi ',state,'. Missatge original: ',msg);
END;

DECLARE EXIT HANDLER FOR SQLEXCEPTION
BEGIN
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Desconegut. Codi ',state,'. Missatge original: ',msg);
END;

	SET RSP := 'OK';
	
	SELECT c.nom, c.acronim, c.descripcio
	INTO nom, acronim, descripcio 
	FROM competicio c 
	WHERE c.codi_competicio = codi;
	
	IF (SELECT competicio FROM competicio_temporada WHERE competicio = codi) IS NOT NULL THEN
			
		-- Esborra de la taula
	
		DELETE FROM competicio_temporada WHERE competicio = codi;
			

		-- Actualitzar log
	
		SET valor_nou = " - ";
		SET consulta = CONCAT("INDIRECTE -> baixa_competicio (",codi,")");
		SET valor_antic = CONCAT("files relatives a la competicio ",codi);
		
		INSERT INTO log(operacio, taula,antic_valor,nou_valor,data,consulta) VALUES ('BAIXA', 'competicio_temporada', valor_antic, valor_nou, NOW(), consulta);
	END IF;
	
	IF (SELECT competicio FROM retransmissio WHERE competicio = codi) IS NOT NULL THEN
			
	
		-- Esborra de la taula
	
		DELETE FROM retransmissio WHERE competicio = codi;
			

		-- Actualitzar log
		
		SET consulta = CONCAT("INDIRECTE -> baixa_competicio (",codi,")");
		SET valor_antic = CONCAT("retransmissions relatives a la competicio ",codi);
		
		INSERT INTO log(operacio, taula,antic_valor,nou_valor,data,consulta) VALUES ('BAIXA', 'retransmissio', valor_antic, valor_nou, NOW(), consulta);
	END IF;
	
	IF (SELECT competicio FROM cursa WHERE competicio = codi) IS NOT NULL THEN	
			
	
		-- Esborra de la taula
	
		DELETE FROM cursa WHERE competicio = codi;
			

		-- Actualitzar log
	
		SET consulta = CONCAT("INDIRECTE -> baixa_competicio (",codi,")");
		SET valor_antic = CONCAT("curses relatives a la competicio ",codi);
		
		INSERT INTO log(operacio, taula,antic_valor,nou_valor,data,consulta) VALUES ('BAIXA', 'cursa', valor_antic, valor_nou, NOW(), consulta);
	END IF;
		
	
	-- Esborra de la taula
	
	DELETE FROM competicio WHERE codi_competicio = codi;

	-- Actualitzar log
	
	SET consulta = CONCAT("baixa_competicio (",codi,")");
	SET valor_antic = CONCAT_WS(',', codi, nom, acronim, descripcio);
	
	INSERT INTO log(operacio, taula,antic_valor,nou_valor,data,consulta) VALUES ('BAIXA', 'competicio', valor_antic, valor_nou, NOW(), consulta);

END//
DELIMITER ;


-- Volcando estructura para procedimiento fia.baixa_component
DELIMITER //
CREATE DEFINER=`root`@`localhost` PROCEDURE `baixa_component`(IN `codi` MEDIUMINT, OUT `RSP` VARCHAR(512))
BEGIN


declare valor_antic VARCHAR (512);
declare valor_nou VARCHAR (10);
declare consulta VARCHAR(512);
declare msg VARCHAR(250);
declare state VARCHAR(50);

declare codi_fabricant MEDIUMINT;
declare codi_cotxe MEDIUMINT;
declare descripcio TEXT;
declare utilitat TEXT;
declare in_telemetria BIGINT;
declare percentatge_defectuos DECIMAL(4,1);


-- Control d'errors

DECLARE EXIT HANDLER FOR NOT FOUND
BEGIN
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO;
	SET RSP := CONCAT('ERROR: Not found. Codi ',state,' .Possiblement el codi component (',codi,'), no sigui correcte.');
END;

DECLARE EXIT HANDLER FOR SQLWARNING 
BEGIN 
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Warning. Codi ',state,'. Missatge original: ',msg);
END;

DECLARE EXIT HANDLER FOR SQLEXCEPTION
BEGIN
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Desconegut. Codi ',state,'. Missatge original: ',msg);
END;

	SET RSP := 'OK';
	
	SELECT c.fabricant, c.cotxe, c.descripcio, c.utilitat
	INTO codi_fabricant, codi_cotxe, descripcio, utilitat
	FROM component c 
	WHERE c.codi_component = codi;
	
	WHILE (SELECT codi_dada FROM telemetria WHERE component = codi LIMIT 1) IS NOT NULL DO
	   SELECT codi_dada FROM telemetria WHERE component = codi LIMIT 1 INTO in_telemetria;
	   call baixa_telemetria(in_telemetria, RSP);
	END WHILE;
				
	
	-- Esborra de la taula
	
	DELETE FROM component WHERE codi_component = codi;
	
	
	-- Actualitzar log
	
	SET valor_nou = " - ";
	SET consulta = CONCAT("baixa_component (",codi,")");
	SET valor_antic = CONCAT_WS(',', codi, codi_fabricant, codi_cotxe, descripcio, utilitat);
	
	INSERT INTO log(operacio, taula,antic_valor,nou_valor,data,consulta) VALUES ('BAIXA', 'component', valor_antic, valor_nou, NOW(), consulta);
	
	
	-- Actualitzar estadistiques de quantitat de components
	
	UPDATE dim_component dc
	SET dc.components = dc.components - 1
	WHERE dc.fabricant = codi_fabricant;
	
	DELETE FROM fact_table_1 WHERE camp_control = '0xQC';
   INSERT INTO fact_table_1(fabricant, quantitat_components, camp_control) 
   SELECT dc.fabricant, dc.components, '0xQC' FROM dim_component dc ORDER BY dc.components DESC LIMIT 5;
   
   
   -- Actualitzar estadistiques de components defectuosos
   
	DELETE FROM dim_defectuos
	WHERE component = codi
	AND cotxe = codi_cotxe;
	
	SELECT (SUM(CASE WHEN defectuos = 1 THEN 1 ELSE 0 END)/COUNT(*)*100) 
	FROM dim_defectuos WHERE cotxe = codi_cotxe
	INTO percentatge_defectuos;
	
	IF percentatge_defectuos IS NULL THEN
		DELETE FROM fact_table_1
		WHERE cotxe = codi_cotxe
		AND camp_control = '0xCD';
	END IF;
	
	UPDATE fact_table_1
	SET components_defectuosos = percentatge_defectuos
	WHERE cotxe = codi_cotxe
	AND camp_control = '0xCD';

END//
DELIMITER ;


-- Volcando estructura para procedimiento fia.baixa_cotxe
DELIMITER //
CREATE DEFINER=`root`@`localhost` PROCEDURE `baixa_cotxe`(IN `codi` MEDIUMINT, OUT `RSP` VARCHAR(512))
BEGIN


declare valor_antic VARCHAR (512);
declare valor_nou VARCHAR (10);
declare consulta VARCHAR(512);
declare in_component MEDIUMINT;
declare msg VARCHAR(250);
declare state VARCHAR(50);

declare model VARCHAR(25);
declare any_debut YEAR;


-- Control d'errors

DECLARE EXIT HANDLER FOR NOT FOUND
BEGIN
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO;
	SET RSP := CONCAT('ERROR: Not found. Codi ',state,' .Possiblement el codi cotxe (',codi,'), no sigui correcte.');
END;

DECLARE EXIT HANDLER FOR SQLWARNING 
BEGIN 
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Warning. Codi ',state,'. Missatge original: ',msg);
END;

DECLARE EXIT HANDLER FOR SQLEXCEPTION
BEGIN
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Desconegut. Codi ',state,'. Missatge original: ',msg);
END;

	SET RSP := 'OK';
	
	SELECT c.model, c.any_debut
	INTO model, any_debut
	FROM cotxe c 
	WHERE c.codi_cotxe = codi;
	
	WHILE (SELECT codi_component FROM component WHERE cotxe = codi LIMIT 1) IS NOT NULL DO
	   SELECT codi_component FROM component WHERE cotxe = codi LIMIT 1 INTO in_component;
	   call baixa_component(in_component, RSP);
	END WHILE;
					
	
	-- Esborra de la taula
	
	DELETE FROM cotxe_equip WHERE cotxe = codi;
		
	
	-- Actualitzar log

	SET valor_nou = " - ";
	SET consulta = CONCAT("INDIRECTE -> baixa_cotxe (",codi,")");
	SET valor_antic = CONCAT("files relatives al cotxe ",codi);
	INSERT INTO log(operacio, taula,antic_valor,nou_valor,data,consulta) VALUES ('BAIXA', 'cotxe_equip', valor_antic, '-', NOW(), consulta);
					
	
	-- Esborra de la taula
	
	DELETE FROM cotxe WHERE codi_cotxe = codi;
	SET consulta = CONCAT("baixa_cotxe (",codi,")");
	SET valor_antic = CONCAT_WS(',', codi, model, any_debut);
			
	
	-- Actualitzar log

	INSERT INTO log(operacio, taula,antic_valor,nou_valor,data,consulta) VALUES ('BAIXA', 'cotxe', valor_antic, valor_nou, NOW(), consulta);

END//
DELIMITER ;


-- Volcando estructura para procedimiento fia.baixa_cursa
DELIMITER //
CREATE DEFINER=`root`@`localhost` PROCEDURE `baixa_cursa`(IN `in_circuit` MEDIUMINT, IN `in_pilot` MEDIUMINT, IN `in_competicio` SMALLINT, IN `in_data` DATE, OUT `RSP` VARCHAR(512))
BEGIN

declare valor_antic VARCHAR (512);
declare valor_nou VARCHAR (10);

declare consulta VARCHAR(512);
declare msg VARCHAR(250);
declare state VARCHAR(50);

declare in_posicio SMALLINT;
declare check_circuit MEDIUMINT;
declare check_pilot MEDIUMINT;
declare check_competicio SMALLINT;
declare total_punts SMALLINT;

declare any_temporada YEAR;
declare pilot_retirat BOOL;


-- Control d'errors

DECLARE EXIT HANDLER FOR NOT FOUND
 BEGIN
 	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
   SET RSP := CONCAT('ERROR: Not found. Codi ',state,'. Possiblement el codi del circuit (',in_circuit,'), del pilot (',in_pilot,') o de la competicio (',in_competicio,') no sigui correcte.' );
 END;

DECLARE EXIT HANDLER FOR SQLWARNING 
BEGIN 
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Warning. Codi ',state,'. Missatge original: ',msg);
END;

DECLARE CONTINUE HANDLER FOR SQLEXCEPTION
BEGIN
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Desconegut. Codi ',state,'. Missatge original: ',msg);
END;


	SET RSP := 'OK';
	SET any_temporada = YEAR(in_data);
	SET pilot_retirat = 0;
	
	SELECT codi_competicio FROM competicio WHERE codi_competicio = in_competicio INTO check_competicio;
	SELECT codi_circuit FROM circuit WHERE codi_circuit = in_circuit INTO check_circuit;
	SELECT codi_persona FROM pilot WHERE codi_persona = in_pilot INTO check_pilot;
	
	SELECT c.posicio
	INTO in_posicio
	FROM cursa c 
	WHERE c.competicio = in_competicio 
	AND c.circuit = in_circuit
	AND c.pilot = in_pilot
	AND c.data_cursa = in_data;
		
	
	-- Esborra de la taula
	
	DELETE FROM cursa 
	WHERE competicio = in_competicio
	AND circuit = in_circuit
	AND pilot = in_pilot
	AND data_cursa = in_data;
	
	
	-- Actualitzar log
	
	SET valor_nou = " - ";
	SET consulta = CONCAT("baixa_cursa (",in_circuit,",",in_pilot,",",in_competicio,")");
	SET valor_antic = CONCAT_WS(',', in_circuit, in_pilot, in_competicio, in_posicio,in_data);
	
	INSERT INTO log(operacio, taula,antic_valor,nou_valor,data,consulta) VALUES ('BAIXA', 'cursa', valor_antic, valor_nou, NOW(), consulta);

	CASE in_posicio
      WHEN 1 THEN 
			SET total_punts = 25;
			
			-- Actualitzar estadistiques sobre curses guanyades
			
			UPDATE dim_resultat 
			SET guanyades = guanyades - 1 
			WHERE pilot = check_pilot;
			
		   DELETE FROM fact_table_1 
			WHERE camp_control = '0xCG';
			
			INSERT INTO fact_table_1(pilot, curses_guanyades, camp_control) 
			SELECT dr.pilot, dr.guanyades, '0xCG' 
			FROM dim_resultat dr 
			ORDER BY dr.guanyades DESC;

      WHEN 2 THEN SET total_punts = 15;
      WHEN 3 THEN SET total_punts = 25;
      WHEN 0 THEN 
			SET total_punts = 0;
			SET pilot_retirat = 1;      
   ELSE
   	BEGIN
   		SET total_punts = 0;
   	END;
   END CASE;
	
	
	-- Actualitzar estadistiques sobre classificacio i posicio
	
	UPDATE dim_classificacio 
	SET punts = punts - total_punts 
	WHERE pilot = in_pilot 
	AND competicio = in_competicio 
	AND temporada = any_temporada;
	
   DELETE FROM fact_table_1 
	WHERE camp_control = '0xCP' 
	AND competicio = in_competicio 
	AND temporada = any_temporada ;
   
	INSERT INTO fact_table_1(competicio, temporada, pilot, punts_pilot, posicio_pilot, camp_control) 
   SELECT dc.competicio, dc.temporada, dc.pilot, dc.punts, @rownum:=@rownum+1 posicio, '0xCP' 
	FROM (SELECT @rownum:=0) r, dim_classificacio dc 
	WHERE competicio = in_competicio 
	AND temporada = any_temporada 
	ORDER BY dc.punts DESC;
	
	
	-- Actualitzar estadistiques sobre curses finalitzades
	
	IF pilot_retirat = 0 THEN
		UPDATE dim_resultat 
		SET finalitzades = finalitzades - 1 
		WHERE pilot = in_pilot;

	   DELETE FROM fact_table_1 WHERE camp_control = '0xCF';
		INSERT INTO fact_table_1(pilot, curses_finalitzades, camp_control) 
		
		SELECT dr.pilot, dr.finalitzades, '0xCF' 
		FROM dim_resultat dr 
		ORDER BY dr.finalitzades DESC
		LIMIT 10;	
	END IF;
	
	
	-- Actualitzar estadistiques sobre la millor evolucio d'un pilot

	DELETE FROM fact_table_1 
	WHERE camp_control = '0xEP' 
	AND pilot = in_pilot 
	AND (data_cursa = in_data OR data_millora = in_data)
	AND YEAR(data_cursa) = any_temporada
	AND competicio = in_competicio;
	
	UPDATE fact_table_1 ft
	SET ft.punts_pilot = ft.punts_pilot - total_punts
	WHERE ft.camp_control = '0xEP' 
	AND ft.pilot = in_pilot
	AND ft.data_cursa < in_data
	AND ft.data_millora > in_data
	AND YEAR(ft.data_cursa) = any_temporada
	AND competicio = in_competicio; 
	
	UPDATE fact_table_1 ft
	SET ft.punts_pilot = ft.punts_pilot - total_punts
	WHERE ft.camp_control = '0xEP' 
	AND ft.pilot = in_pilot
	AND ft.data_cursa > in_data
	AND ft.data_millora IS NULL
	AND YEAR(ft.data_cursa) = any_temporada
	AND competicio = in_competicio; 
	
END//
DELIMITER ;


-- Volcando estructura para procedimiento fia.baixa_director
DELIMITER //
CREATE DEFINER=`root`@`localhost` PROCEDURE `baixa_director`(IN `in_persona` MEDIUMINT, OUT `RSP` VARCHAR(512))
BEGIN

declare valor_antic VARCHAR (512);
declare valor_nou VARCHAR (10);
declare in_equip SMALLINT;
declare in_estudis TEXT;
declare in_any_inici YEAR;
declare in_nom VARCHAR(100);
declare in_pais CHAR(2);
declare consulta VARCHAR(512);
declare msg VARCHAR(250);
declare state VARCHAR(50);


-- Control d'errors

DECLARE EXIT HANDLER FOR NOT FOUND
 BEGIN
 	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
   SET RSP := CONCAT('ERROR: Not found. Codi ',state,'. Possiblement el codi de la persona (',in_persona,') no sigui correcte.' );
 END;

DECLARE EXIT HANDLER FOR SQLWARNING 
BEGIN 
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Warning. Codi ',state,'. Missatge original: ',msg);
END;

DECLARE CONTINUE HANDLER FOR SQLEXCEPTION
BEGIN
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Desconegut. Codi ',state,'. Missatge original: ',msg);
END;


	SET RSP := 'OK';
	SET valor_nou = " - ";
	
	call baixa_pilot;
	SELECT d.equip, d.estudis, d.any_inici
	INTO in_equip, in_estudis, in_any_inici
	FROM director d
	WHERE d.codi_persona = in_persona;
	
	SELECT p.nom, p.nacionalitat
	INTO in_nom, in_pais
	FROM persona p
	WHERE p.codi_persona = in_persona;

	
	-- Esborra de la taula
	
	DELETE FROM director  
	WHERE codi_persona = in_persona;
		
	
	-- Actualitzar log
	
	SET consulta = CONCAT("baixa_director (",in_persona,")");
	SET valor_antic = CONCAT_WS(',',in_persona, in_equip, in_estudis, in_any_inici);
	
	INSERT INTO log(operacio, taula,antic_valor,nou_valor,data,consulta) VALUES ('BAIXA', 'director', valor_antic, valor_nou, NOW(), consulta);
		
	
	-- Esborra de la taula
	
	DELETE FROM persona  
	WHERE codi_persona = in_persona;
		
	
	-- Actualitzar log
	
	SET consulta = CONCAT("INDIRECTE->baixa_director (",in_persona,")");
	SET valor_antic = CONCAT_WS(',',in_persona, in_nom, in_pais);
	
	INSERT INTO log(operacio, taula,antic_valor,nou_valor,data,consulta) VALUES ('BAIXA', 'persona', valor_antic, valor_nou, NOW(), consulta);

END//
DELIMITER ;


-- Volcando estructura para procedimiento fia.baixa_emissora
DELIMITER //
CREATE DEFINER=`root`@`localhost` PROCEDURE `baixa_emissora`(IN `in_emissora` SMALLINT, OUT `RSP` VARCHAR(512))
BEGIN

declare valor_antic VARCHAR (512);
declare valor_nou VARCHAR (10);
declare in_competicio SMALLINT;
declare in_nom VARCHAR(50);
declare in_pais CHAR(2);
declare consulta VARCHAR(512);
declare msg VARCHAR(250);
declare state VARCHAR(50);

declare del_emissora SMALLINT;
declare del_competicio SMALLINT;


-- Control d'errors

DECLARE EXIT HANDLER FOR NOT FOUND
 BEGIN
 	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
   SET RSP := CONCAT('ERROR: Not found. Codi ',state,'. Possiblement el codi de la emissora (',in_emissora,') no sigui correcte.' );
 END;

DECLARE EXIT HANDLER FOR SQLWARNING 
BEGIN 
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Warning. Codi ',state,'. Missatge original: ',msg);
END;

DECLARE CONTINUE HANDLER FOR SQLEXCEPTION
BEGIN
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Desconegut. Codi ',state,'. Missatge original: ',msg);
END;

	SET RSP := 'OK';
		
	SELECT e.nom, e.pais
	INTO in_nom, in_pais
	FROM emissora e
	WHERE e.codi_emissora = in_emissora;
	
	WHILE (SELECT emissora FROM retransmissio WHERE emissora = in_emissora LIMIT 1) IS NOT NULL DO
	   SELECT emissora, competicio FROM retransmissio WHERE emissora = in_emissora INTO del_emissora, del_competicio;
	   call baixa_retransmissio(del_emissora, del_competicio, RSP);
	END WHILE;
	
	
	-- Esborra de la taula
	
	DELETE FROM emissora  
	WHERE codi_emissora = in_emissora;
		

	-- Actualitzar log
	
	SET valor_nou = " - ";
	SET consulta = CONCAT("baixa_emissora (",in_emissora,")");
	SET valor_antic = CONCAT_WS(',',in_emissora, in_nom, in_pais);

	INSERT INTO log(operacio, taula,antic_valor,nou_valor,data,consulta) VALUES ('BAIXA', 'emissora', valor_antic, valor_nou, NOW(), consulta);

END//
DELIMITER ;


-- Volcando estructura para procedimiento fia.baixa_equip
DELIMITER //
CREATE DEFINER=`root`@`localhost` PROCEDURE `baixa_equip`(IN `in_equip` SMALLINT, OUT `RSP` VARCHAR(512))
BEGIN

declare valor_antic VARCHAR (512);
declare valor_nou VARCHAR (10);
declare consulta VARCHAR(512);
declare msg VARCHAR(250);
declare state VARCHAR(50);

declare in_nom VARCHAR(50);
declare in_seu VARCHAR(100);
declare in_debut DATE;

declare del_director MEDIUMINT;
declare del_pilot MEDIUMINT;
declare del_patrocinador SMALLINT;
declare del_temporada YEAR;
declare del_cotxe MEDIUMINT;


-- Control d'errors

DECLARE EXIT HANDLER FOR NOT FOUND
 BEGIN
 	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
   SET RSP := CONCAT('ERROR: Not found. Codi ',state,'. Possiblement el codi de l\'equip (',in_equip,') no sigui correcte.' );
 END;

DECLARE EXIT HANDLER FOR SQLWARNING 
BEGIN 
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Warning. Codi ',state,'. Missatge original: ',msg);
END;

DECLARE CONTINUE HANDLER FOR SQLEXCEPTION
BEGIN
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Desconegut. Codi ',state,'. Missatge original: ',msg);
END;


	SET RSP := 'OK';

	-- Esborrar el director d'equip si en te
	
	IF (SELECT codi_persona FROM director WHERE equip = in_equip) IS NOT NULL THEN
		SELECT codi_persona FROM director WHERE equip = in_equip INTO del_director;
		CALL baixa_director(del_director, RSP);
	END IF;

	-- Esborrar els pilots d'equip si en te
	
	WHILE (SELECT codi_persona FROM pilot WHERE equip = in_equip LIMIT 1) IS NOT NULL DO
	   SELECT codi_persona FROM pilot WHERE equip = in_equip LIMIT 1 INTO del_pilot;
	   call baixa_pilot(del_pilot, RSP);
	END WHILE;

	-- Esborrar els patrocinis d'equip si en te
	
	WHILE (SELECT equip FROM patrocini WHERE equip = in_equip LIMIT 1) IS NOT NULL DO
	   SELECT patrocinador, temporada FROM patrocini WHERE equip = in_equip LIMIT 1 INTO del_patrocinador, del_temporada;
	   call baixa_patrocini(del_patrocinador, in_equip, del_temporada,  RSP);
	END WHILE;
	
	-- Esborrar els cotxes d'equip si en te
	
	WHILE (SELECT equip FROM cotxe_equip WHERE equip = in_equip LIMIT 1) IS NOT NULL DO
	   SELECT cotxe FROM cotxe_equip WHERE equip = in_equip LIMIT 1 INTO del_cotxe;
	   call baixa_cotxe(del_cotxe, RSP);
	END WHILE;

	
	SELECT e.nom, e.seu, e.data_debut
	INTO in_nom, in_seu, in_debut
	FROM equip e 
	WHERE e.codi_equip = in_equip;
	
	
	-- Esborra de la taula
	
	DELETE FROM equip 
	WHERE  codi_equip = in_equip;
			

	-- Actualitzar log

	SET valor_nou = " - ";
	SET consulta = CONCAT("baixa_equip (",in_equip,")");
	SET valor_antic = CONCAT_WS(',', in_equip, in_nom, in_seu, in_debut);
	INSERT INTO log(operacio, taula,antic_valor,nou_valor,data,consulta) VALUES ('BAIXA', 'equip', valor_antic, valor_nou, NOW(), consulta);
END//
DELIMITER ;


-- Volcando estructura para procedimiento fia.baixa_fabricant
DELIMITER //
CREATE DEFINER=`root`@`localhost` PROCEDURE `baixa_fabricant`(IN `in_fabricant` MEDIUMINT, OUT `RSP` VARCHAR(512))
BEGIN

declare valor_antic VARCHAR (512);
declare valor_nou VARCHAR (10);
declare consulta VARCHAR(512);
declare msg VARCHAR(250);
declare state VARCHAR(50);

declare in_nom VARCHAR(50);
declare in_debut YEAR;

declare del_component MEDIUMINT;


-- Control d'errors

DECLARE EXIT HANDLER FOR NOT FOUND
 BEGIN
 	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
   SET RSP := CONCAT('ERROR: Not found. Codi ',state,'. Possiblement el codi del fabricant (',in_fabricant,') no sigui correcte.');
 END;

DECLARE EXIT HANDLER FOR SQLWARNING 
BEGIN 
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Warning. Codi ',state,'. Missatge original: ',msg);
END;

DECLARE CONTINUE HANDLER FOR SQLEXCEPTION
BEGIN
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Desconegut. Codi ',state,'. Missatge original: ',msg);
END;

	SET RSP := 'OK';
		
	SELECT f.nom, f.any_debut
	INTO in_nom, in_debut
	FROM fabricant f
	WHERE `codi_fabricant` = in_fabricant;
			
			
	-- Esborrar els components si en te
	
	WHILE (SELECT `codi_component` FROM component WHERE `fabricant` = in_fabricant LIMIT 1) IS NOT NULL DO
	   SELECT `codi_component` FROM component WHERE `fabricant` = in_fabricant LIMIT 1 INTO del_component;
	   call baixa_component(del_component,  RSP);
	END WHILE;
	
	
	-- Esborra de la taula
		
	DELETE FROM fabricant
	WHERE `codi_fabricant` = in_fabricant;
	
	
	-- Actualitzar log
	
	SET valor_nou = " - ";
	SET consulta = CONCAT("baixa_fabricant (",in_fabricant,")");
	SET valor_antic = CONCAT_WS(',', in_fabricant, in_nom, in_debut);
	INSERT INTO log(operacio, taula,antic_valor,nou_valor,data,consulta) VALUES ('BAIXA', 'fabricant', valor_antic, valor_nou, NOW(), consulta);
END//
DELIMITER ;


-- Volcando estructura para procedimiento fia.baixa_pais
DELIMITER //
CREATE DEFINER=`root`@`localhost` PROCEDURE `baixa_pais`(IN `in_pais` CHAR(2), OUT `RSP` VARCHAR(512))
BEGIN

declare valor_antic VARCHAR (512);
declare valor_nou VARCHAR (10);
declare consulta VARCHAR(512);
declare msg VARCHAR(250);
declare state VARCHAR(50);

declare in_nom VARCHAR(50);


-- Control d'errors

DECLARE EXIT HANDLER FOR 1451
 BEGIN
 	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
   SET RSP := CONCAT('ERROR: Codi ',state,'. No es pot esborrar.  Possiblement el codi del pais (',in_pais,') s\'esta fent servir a altres taules (emissora, persona).' );
 END;

DECLARE EXIT HANDLER FOR NOT FOUND
 BEGIN
 	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
   SET RSP := CONCAT('ERROR: Not found. Codi ',state,'. Possiblement el codi del pais (',in_pais,') no sigui correcte.' );
 END;

DECLARE EXIT HANDLER FOR SQLWARNING 
BEGIN 
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Warning. Codi ',state,'. Missatge original: ',msg);
END;

DECLARE CONTINUE HANDLER FOR SQLEXCEPTION
BEGIN
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Desconegut. Codi ',state,'. Missatge original: ',msg);
END;

	SET RSP := 'OK';
	
	SELECT p.nom
	INTO in_nom
	FROM pais p
	WHERE p.codi_pais = in_pais;


	-- Esborrar de la taula

	DELETE FROM pais 
	WHERE codi_pais = in_pais;
	
	
	-- Actualitzar log

	SET valor_nou = " - ";	
	SET consulta = CONCAT("baixa_pais (",in_pais,")");
	SET valor_antic = CONCAT_WS(',', in_pais, in_nom);
	INSERT INTO log(operacio, taula,antic_valor,nou_valor,data,consulta) VALUES ('BAIXA', 'pais', valor_antic, valor_nou, NOW(), consulta);
END//
DELIMITER ;


-- Volcando estructura para procedimiento fia.baixa_patrocinador
DELIMITER //
CREATE DEFINER=`root`@`localhost` PROCEDURE `baixa_patrocinador`(IN `in_patrocinador` MEDIUMINT, OUT `RSP` VARCHAR(512))
BEGIN

declare valor_antic VARCHAR (512);
declare valor_nou VARCHAR (10);
declare consulta VARCHAR(512);
declare msg VARCHAR(250);
declare state VARCHAR(50);

declare in_sector SMALLINT;
declare in_nom VARCHAR(50);

declare del_equip SMALLINT;
declare del_temporada YEAR;


-- Control d'errors

DECLARE EXIT HANDLER FOR NOT FOUND
 BEGIN
 	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
   SET RSP := CONCAT('ERROR: Not found. Codi ',state,'. Possiblement el codi del patrocinador (',in_patrocinador,') no sigui correcte.');
 END;

DECLARE EXIT HANDLER FOR SQLWARNING 
BEGIN 
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Warning. Codi ',state,'. Missatge original: ',msg);
END;

DECLARE CONTINUE HANDLER FOR SQLEXCEPTION
BEGIN
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Desconegut. Codi ',state,'. Missatge original: ',msg);
END;

	SET RSP := 'OK';
	
	SELECT p.sector, p.nom
	INTO in_sector, in_nom
	FROM patrocinador p
	WHERE p.codi_patrocinador = in_patrocinador;
		
		
	-- Esborrar els patrocinis si en te
	
	WHILE (SELECT `patrocinador` FROM patrocini WHERE `patrocinador` = in_patrocinador LIMIT 1) IS NOT NULL DO
	   SELECT equip, temporada FROM patrocini WHERE `patrocinador` = in_patrocinador LIMIT 1 INTO del_equip, del_temporada;
	   call baixa_patrocini(in_patrocinador, del_equip, del_temporada,  RSP);
	END WHILE;
		
			
	-- Esborra de la taula
	
	DELETE FROM patrocinador
	WHERE codi_patrocinador = in_patrocinador;
		
	
	-- Actualitzar log
	
	SET valor_nou = " - ";
	SET consulta = CONCAT("baixa_patrocinador (",in_patrocinador,")");
	SET valor_antic = CONCAT_WS(',',in_patrocinador, in_nom);
	
	INSERT INTO log(operacio, taula,antic_valor,nou_valor,data,consulta) VALUES ('BAIXA', 'patrocinador', valor_antic, valor_nou, NOW(), consulta);
END//
DELIMITER ;


-- Volcando estructura para procedimiento fia.baixa_patrocini
DELIMITER //
CREATE DEFINER=`root`@`localhost` PROCEDURE `baixa_patrocini`(IN `in_patrocinador` MEDIUMINT, IN `in_equip` SMALLINT, IN `in_temporada` YEAR, OUT `RSP` VARCHAR(512))
BEGIN

declare valor_antic VARCHAR (512);
declare valor_nou VARCHAR (10);
declare consulta VARCHAR(512);
declare msg VARCHAR(250);
declare state VARCHAR(50);

declare in_cost INT;


-- Control d'errors

DECLARE EXIT HANDLER FOR NOT FOUND
 BEGIN
 	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
   SET RSP := CONCAT('ERROR: Not found. Codi ',state,'. Possiblement el codi del patrocinador (',in_patrocinador,'), de l\'equip (',in_equip,') o de l\'any (',in_temporada,') no sigui correcte.' );
 END;

DECLARE EXIT HANDLER FOR SQLWARNING 
BEGIN 
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Warning. Codi ',state,'. Missatge original: ',msg);
END;

DECLARE CONTINUE HANDLER FOR SQLEXCEPTION
BEGIN
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Desconegut. Codi ',state,'. Missatge original: ',msg);
END;

	SET RSP := 'OK';

	SELECT p.cost
	INTO in_cost
	FROM patrocini p
	WHERE p.patrocinador = in_patrocinador
	AND p.equip = in_equip 
	AND p.temporada = in_temporada;
		
		
	-- Esborra de la taula
	
	DELETE FROM patrocini 
	WHERE patrocinador = in_patrocinador
	AND equip = in_equip 
	AND temporada = in_temporada;
		
	
	-- Actualitzar log

	SET valor_nou = " - ";
	SET consulta = CONCAT("baixa_patrocini (",in_patrocinador,', ', in_equip,', ', in_temporada,")");
	SET valor_antic = CONCAT_WS(',', in_patrocinador, in_equip, in_temporada, in_cost);
	INSERT INTO log(operacio, taula,antic_valor,nou_valor,data,consulta) VALUES ('BAIXA', 'patrocini', valor_antic, valor_nou, NOW(), consulta);
	
	
	-- Actualitzar estadistiques de patrocini mes alt
	
	UPDATE dim_patrocini 
	SET dim_patrocini.cost = dim_patrocini.cost - in_cost 
	WHERE dim_patrocini.patrocinador = in_patrocinador;

   DELETE FROM fact_table_1 WHERE fact_table_1.camp_control = '0xPA';
   INSERT INTO fact_table_1(patrocinador, cost_patrocini, camp_control) 
   SELECT dp.patrocinador, dp.cost, '0xPA' FROM dim_patrocini dp ORDER BY dp.cost DESC LIMIT 5;
	
END//
DELIMITER ;


-- Volcando estructura para procedimiento fia.baixa_pilot
DELIMITER //
CREATE DEFINER=`root`@`localhost` PROCEDURE `baixa_pilot`(IN `in_persona` MEDIUMINT, OUT `RSP` VARCHAR(512))
BEGIN

declare valor_antic VARCHAR (512);
declare valor_nou VARCHAR (10);
declare in_equip SMALLINT;
declare in_llicencia VARCHAR (8);
declare in_any_inici YEAR;
declare in_nom VARCHAR(100);
declare in_pais CHAR(2);
declare consulta VARCHAR(512);
declare msg VARCHAR(250);
declare state VARCHAR(50);


-- Control d'errors

DECLARE EXIT HANDLER FOR NOT FOUND
 BEGIN
 	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
   SET RSP := CONCAT('ERROR: Not found. Codi ',state,'. Possiblement el codi de la persona (',in_persona,') no sigui correcte.' );
 END;

DECLARE EXIT HANDLER FOR SQLWARNING 
BEGIN 
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Warning. Codi ',state,'. Missatge original: ',msg);
END;

DECLARE CONTINUE HANDLER FOR SQLEXCEPTION
BEGIN
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Desconegut. Codi ',state,'. Missatge original: ',msg);
END;


	SET RSP := 'OK';
	
	SELECT p.equip, p.codi_llicencia, p.any_inici
	INTO in_equip, in_llicencia, in_any_inici
	FROM pilot p
	WHERE p.codi_persona = in_persona;
	
	SELECT p.nom, p.nacionalitat
	INTO in_nom, in_pais
	FROM persona p
	WHERE p.codi_persona = in_persona;
		
		
	-- Esborra de la taula
	
	DELETE FROM pilot  
	WHERE codi_persona = in_persona;
	SET consulta = CONCAT("baixa_pilot (",in_persona,")");
	SET valor_antic = CONCAT_WS(',',in_persona, in_equip, in_llicencia, in_any_inici);
	INSERT INTO log(operacio, taula,antic_valor,nou_valor,data,consulta) VALUES ('BAIXA', 'pilot', valor_antic, valor_nou, NOW(), consulta);
		
	
	-- Esborra de la taula
	
	DELETE FROM persona  
	WHERE codi_persona = in_persona;
		
	
	-- Actualitzar log
	
	SET valor_nou = " - ";
	SET consulta = CONCAT("INDIRECTE->baixa_pilot (",in_persona,")");
	SET valor_antic = CONCAT_WS(',',in_persona, in_nom, in_pais);
	INSERT INTO log(operacio, taula,antic_valor,nou_valor,data,consulta) VALUES ('BAIXA', 'persona', valor_antic, valor_nou, NOW(), consulta);


END//
DELIMITER ;


-- Volcando estructura para procedimiento fia.baixa_retransmissio
DELIMITER //
CREATE DEFINER=`root`@`localhost` PROCEDURE `baixa_retransmissio`(IN `in_emissora` SMALLINT, IN `in_competicio` SMALLINT, OUT `RSP` VARCHAR(512))
BEGIN

declare valor_antic VARCHAR (512);
declare valor_nou VARCHAR (10);
declare consulta VARCHAR(512);
declare msg VARCHAR(250);
declare state VARCHAR(50);

declare check_competicio SMALLINT;
declare check_emissora SMALLINT;


-- Control d'errors

DECLARE EXIT HANDLER FOR NOT FOUND
BEGIN
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO;
	SET RSP := CONCAT('ERROR: Not found. Codi ',state,' .Possiblement el codi de l\'emissora (',in_emissora,') o de la competicio (',in_competicio,') no sigui correcte.');
END;

DECLARE EXIT HANDLER FOR SQLWARNING 
BEGIN 
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Warning. Codi ',state,'. Missatge original: ',msg);
END;

DECLARE EXIT HANDLER FOR SQLEXCEPTION
BEGIN
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Desconegut. Codi ',state,'. Missatge original: ',msg);
END;

	SET RSP := 'OK';
	
	-- Comprovacio de si existeixn a la taula els arguments
	SELECT r.emissora, r.competicio
	INTO check_emissora, check_competicio
	FROM retransmissio r
	WHERE r.emissora = in_emissora 
	AND r.competicio = in_competicio;
		
	
	-- Esborra de la taula
	
	DELETE FROM retransmissio 
	WHERE competicio = check_competicio 
	AND emissora = check_emissora;
		
	
	-- Actualitzar log
	
	SET valor_nou = " - ";
	SET consulta = CONCAT("baixa_retransmissio (",in_emissora,", ",in_competicio,")");
	SET valor_antic = CONCAT_WS(',', in_emissora, in_competicio );
	INSERT INTO log(operacio, taula,antic_valor,nou_valor,data,consulta) VALUES ('BAIXA', 'retransmissio', valor_antic, valor_nou, NOW(), consulta);

END//
DELIMITER ;


-- Volcando estructura para procedimiento fia.baixa_sector
DELIMITER //
CREATE DEFINER=`root`@`localhost` PROCEDURE `baixa_sector`(IN `in_sector` SMALLINT, OUT `RSP` VARCHAR(512))
BEGIN

declare valor_antic VARCHAR (512);
declare valor_nou VARCHAR (10);
declare consulta VARCHAR(512);
declare msg VARCHAR(250);
declare state VARCHAR(50);

declare in_nom VARCHAR(50);


-- Control d'errors

DECLARE EXIT HANDLER FOR NOT FOUND
 BEGIN
 	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
   SET RSP := CONCAT('ERROR: Not found. Codi ',state,'. Possiblement el codi del sector (',in_sector,') no sigui correcte.' );
 END;

DECLARE EXIT HANDLER FOR SQLWARNING 
BEGIN 
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Warning. Codi ',state,'. Missatge original: ',msg);
END;

DECLARE CONTINUE HANDLER FOR SQLEXCEPTION
BEGIN
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Desconegut. Codi ',state,'. Missatge original: ',msg);
END;

	SET RSP := 'OK';
	
	SELECT s.nom
	INTO in_nom
	FROM sector s 
	WHERE s.codi_sector = in_sector;
		
	
	-- Esborra de la taula
	
	DELETE FROM sector 
	WHERE codi_sector = in_sector;
		
	
	-- Actualitzar log

	SET valor_nou = " - ";	
	SET consulta = CONCAT("baixa_sector (",in_sector,")");
	SET valor_antic = CONCAT_WS(',', in_sector, in_nom);
	
	INSERT INTO log(operacio, taula,antic_valor,nou_valor,data,consulta) VALUES ('BAIXA', 'sector', valor_antic, valor_nou, NOW(), consulta);
END//
DELIMITER ;


-- Volcando estructura para procedimiento fia.baixa_telemetria
DELIMITER //
CREATE DEFINER=`root`@`localhost` PROCEDURE `baixa_telemetria`(IN `in_dada` BIGINT, OUT `RSP` VARCHAR(512))
BEGIN

declare valor_antic VARCHAR (512);
declare valor_nou VARCHAR (10);
declare consulta VARCHAR(512);
declare msg VARCHAR(250);
declare state VARCHAR(50);

declare in_component MEDIUMINT;
declare in_moment DATETIME;
declare in_ambit VARCHAR(25);
declare in_valor FLOAT;
declare in_unitats VARCHAR(25);

declare current_cotxe SMALLINT;
declare current_circuit SMALLINT;
declare current_year YEAR;
declare current_equip SMALLINT;
declare current_competicio SMALLINT;
declare current_pilot SMALLINT;

declare temperatura_maxima FLOAT;
declare aux_temps TIME;


-- Control d'errors

DECLARE EXIT HANDLER FOR NOT FOUND
BEGIN
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO;
	SET RSP := CONCAT('ERROR: Not found. Codi ',state,' .Possiblement el codi de la dada (',in_dada,'), no sigui correcte.');
END;

DECLARE EXIT HANDLER FOR SQLWARNING 
BEGIN 
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Warning. Codi ',state,'. Missatge original: ',msg);
END;

DECLARE EXIT HANDLER FOR SQLEXCEPTION
BEGIN
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Desconegut. Codi ',state,'. Missatge original: ',msg);
END;

	SET RSP := 'OK';
	
	SELECT t.component, t.moment, t.ambit, t.valor, t.unitat_de_mesura
	INTO in_component, in_moment, in_ambit, in_valor, in_unitats
	FROM telemetria t 
	WHERE t.codi_dada = in_dada;
		
	
	-- Esborra de la taula
	
	DELETE FROM telemetria WHERE codi_dada = in_dada;
	SET consulta = CONCAT("baixa_telemetria (",in_dada,")");
		
	
	-- Actualitzar log
	
	SET valor_nou = " - ";	
	SET valor_antic = CONCAT_WS(',', in_component, in_moment, in_ambit, in_valor, in_unitats);
	INSERT INTO log(operacio, taula,antic_valor,nou_valor,data,consulta) VALUES ('BAIXA', 'telemetria', valor_antic, valor_nou, NOW(), consulta);
	
	
	
	-- Actualitzar estadistiques de volum telemetria global
	
	SET current_year = YEAR(in_moment);
	
	SELECT c.cotxe 
	FROM component c 
	WHERE c.codi_component = in_component 
	INTO current_cotxe;
	
	SELECT ce.equip 
	FROM cotxe_equip ce
	WHERE ce.cotxe = current_cotxe 
	INTO current_equip;

	UPDATE dim_telemetria dt
	SET dt.dades_any =  dt.dades_any - 1 
	WHERE dt.temporada = current_year 
	AND dt.equip = current_equip;
	
	SELECT pilot 
	INTO current_pilot 
	FROM pilot_cotxe 
	WHERE temporada = current_year 
	AND cotxe = current_cotxe;
	
	SELECT competicio, circuit
	INTO current_competicio, current_circuit 
	FROM cursa 
	WHERE data_cursa = DATE(in_moment) 
	AND pilot = current_pilot;
	
	DELETE FROM fact_table_1 WHERE camp_control = '0xTE' AND temporada = current_year;
  
   INSERT INTO fact_table_1(temporada, equip, dades_telemetriques, camp_control) 
   SELECT dt.temporada, dt.equip, SUM(dt.dades_any), '0xTE' 
	FROM dim_telemetria dt 
	WHERE dt.temporada = current_year 
	ORDER BY dt.dades_any DESC 
	LIMIT 1;
	
	
	-- Actualitzar estadistiques de consum global
	
	CASE in_ambit
      WHEN 'consum' THEN					
			UPDATE dim_consum dc
			SET dc.consum =  dc.consum - in_valor
			WHERE dc.temporada = current_year 
			AND dc.competicio = current_competicio
			AND dc.cotxe = current_cotxe;

		   
		   DELETE FROM fact_table_1 WHERE camp_control = '0xCC' AND temporada = current_year AND competicio = current_competicio;
		   INSERT INTO fact_table_1(competicio, temporada, cotxe, consum_cotxe, camp_control) 
		   SELECT dc.competicio, dc.temporada, dc.cotxe, dc.consum, '0xCC' FROM dim_consum dc WHERE dc.competicio = current_competicio AND dc.temporada = current_year ORDER BY dc.consum DESC LIMIT 1;
	
	
		-- Actualitzar estadistiques de temperatura maxima
		
	   WHEN 'temperatura' THEN		
			SELECT valor
			INTO temperatura_maxima 
			FROM telemetria 
			WHERE ambit = 'temperatura' 
			ORDER BY valor DESC 
			LIMIT 1;
			
			DELETE FROM fact_table_1
			WHERE camp_control = '0xTM';
			
			INSERT INTO fact_table_1 (competicio, temporada, pilot, cotxe, circuit, data_cursa, temperatura_maxima, camp_control) 
			VALUES (current_competicio, current_year, current_pilot, current_cotxe, current_circuit, DATE(in_moment), temperatura_maxima, '0xTM');


		-- Actualitzar estadistiques de volta rapida
		
	   WHEN 'temps per volta' THEN		
			SET aux_temps = (TIME_FORMAT(SEC_TO_TIME(in_valor/1000),'%i:%s'));

			DELETE FROM dim_volta_rapida 
			WHERE circuit = current_circuit 
			AND pilot = current_pilot
			AND equip = current_equip
			AND data = DATE(in_moment)
			AND temps = aux_temps;

			DELETE FROM fact_table_1 
			WHERE circuit = current_circuit 
			AND camp_control = '0xVR';			
			
			INSERT INTO fact_table_1 (circuit, pilot, equip, data_cursa, volta_rapida, camp_control) 		
			SELECT dvr.circuit, dvr.pilot, dvr.equip,  dvr.data, dvr.temps, '0xVR'
			FROM dim_volta_rapida dvr
			WHERE dvr.circuit = current_circuit
			ORDER BY temps 
			LIMIT 10;
	ELSE
   	BEGIN
   	
   	END;
   END CASE;
	

END//
DELIMITER ;


-- Volcando estructura para procedimiento fia.baixa_temporada
DELIMITER //
CREATE DEFINER=`root`@`localhost` PROCEDURE `baixa_temporada`(IN `in_temporada` YEAR, IN `in_competicio` SMALLINT, OUT `RSP` VARCHAR(512))
BEGIN

declare valor_antic VARCHAR (512);
declare valor_nou VARCHAR (10);
declare consulta VARCHAR(512);
declare msg VARCHAR(250);
declare state VARCHAR(50);

declare check_temporada YEAR;
declare check_competicio SMALLINT;


-- Control d'errors

DECLARE EXIT HANDLER FOR 1451
 BEGIN
 	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
   SET RSP := CONCAT('ERROR: Codi ',state,'. No es pot esborrar.  Possiblement la temporada (',in_temporada,') s\'esta fent servir a altres taules (patrocini, competicio).' );
 END;

DECLARE EXIT HANDLER FOR NOT FOUND
 BEGIN
 	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
   SET RSP := CONCAT('ERROR: Not found. Codi ',state,'. Possiblement el codi la temporada (',in_temporada,') o la competicio (',in_competicio,') no sigui correcte.' );
 END;

DECLARE EXIT HANDLER FOR SQLWARNING 
BEGIN 
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Warning. Codi ',state,'. Missatge original: ',msg);
END;

DECLARE CONTINUE HANDLER FOR SQLEXCEPTION
BEGIN
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Desconegut. Codi ',state,'. Missatge original: ',msg);
END;

	SET RSP := 'OK';
	
	SELECT `temporada`, `competicio`
	INTO check_temporada, check_competicio
	FROM competicio_temporada
	WHERE `temporada` = in_temporada
	AND `competicio` = in_competicio;	
		
	
	-- Esborra de la taula
	
	DELETE FROM competicio_temporada
	WHERE `temporada` = in_temporada
	AND `competicio` = in_competicio;
		
	
	-- Actualitzar log

	SET valor_nou = " - ";
	SET consulta = CONCAT("baixa_temporada (",in_temporada,", ",in_competicio,")");
	SET valor_antic = CONCAT(in_temporada,", ",in_competicio);
	INSERT INTO log(operacio, taula,antic_valor,nou_valor,data,consulta) VALUES ('BAIXA', 'competicio_temporada', valor_antic, valor_nou, NOW(), consulta);
		
END//
DELIMITER ;


-- Volcando estructura para procedimiento fia.estadistica_classificacio_pilots
DELIMITER //
CREATE DEFINER=`root`@`localhost` PROCEDURE `estadistica_classificacio_pilots`()
BEGIN

declare loop_pilot SMALLINT;
declare max_pilot SMALLINT;
declare current_pilot SMALLINT;

declare loop_competicio SMALLINT;
declare max_competicio SMALLINT;
declare current_competicio SMALLINT;

declare loop_temporada SMALLINT;
declare max_temporada SMALLINT;
declare current_temporada YEAR;

declare aux_punts SMALLINT;
declare total_punts SMALLINT;
declare current_equip SMALLINT (5);
declare current_cotxe MEDIUMINT (8);
declare current_circuit MEDIUMINT (8);


	DELETE FROM fact_table_1 WHERE camp_control = '0xCP';

	SELECT count(*) 
	INTO max_competicio 
	FROM (
		SELECT *
		FROM cursa
		GROUP BY competicio
	)count_competicio;

	SET loop_competicio = 0;
	
	WHILE loop_competicio < max_competicio DO
	
		SELECT competicio
		INTO current_competicio
		FROM cursa 
		GROUP BY competicio
		LIMIT loop_competicio,1;
		
		SELECT count(*) 
		INTO max_temporada 
		FROM (
			SELECT EXTRACT(YEAR from data_cursa)as temporada 
			FROM cursa  
			WHERE competicio = current_competicio 
			GROUP BY temporada
		)count_temporadas;
				
		SET loop_temporada = 0;
			
		WHILE loop_temporada < max_temporada DO
		
      	SELECT (EXTRACT(YEAR from data_cursa))as temporada
			INTO current_temporada
			FROM cursa 
			WHERE competicio = current_competicio 
			GROUP BY temporada
			LIMIT loop_temporada,1;

        	SELECT count(*) 
	  		INTO max_pilot 
	  		FROM (
	  			SELECT pilot
	  			FROM cursa
	  			WHERE EXTRACT(YEAR from data_cursa) = current_temporada
	  			AND competicio = current_competicio
	  			GROUP BY pilot
			  )count_data;

			SET loop_pilot = 0;
  
     		WHILE loop_pilot < max_pilot DO
     		
     			SELECT pilot 
  				INTO current_pilot
  				FROM cursa 
  				WHERE competicio = current_competicio 
  				AND EXTRACT(YEAR from data_cursa) = current_temporada
  				GROUP BY pilot
  				LIMIT loop_pilot,1;

       		SET total_punts = 0;
				SET aux_punts	=	(SELECT count(posicio)*25 as count
									FROM cursa c 
									WHERE c.pilot = current_pilot 
									AND c.competicio = current_competicio 
									AND EXTRACT(YEAR from data_cursa) = current_temporada 
									AND c.posicio = 1 
									GROUP BY posicio);
			
				SET total_punts = total_punts + (SELECT IFNULL(aux_punts,0));
				SET aux_punts	=	(SELECT count(posicio)*15 as count
									FROM cursa c 
									WHERE c.pilot = current_pilot 
									AND c.competicio = current_competicio 
									AND EXTRACT(YEAR from data_cursa) = current_temporada 
									AND c.posicio = 2 
									GROUP BY posicio);
			
				SET total_punts = total_punts + (SELECT IFNULL(aux_punts,0));
				SET aux_punts	=	aux_punts + (SELECT count(posicio)*5 as count
									FROM cursa c 
									WHERE c.pilot = current_pilot 
									AND c.competicio = current_competicio 
									AND EXTRACT(YEAR from data_cursa) = current_temporada 
									AND c.posicio = 3 
									GROUP BY posicio);
									
				SET total_punts = total_punts + (SELECT IFNULL(aux_punts,0));

				INSERT INTO dim_classificacio (competicio, temporada, pilot, punts) 
  				VALUES (current_competicio, current_temporada, current_pilot, total_punts);
				  
				SET loop_pilot = loop_pilot + 1;
			END WHILE; -- Loop pilot
			
			INSERT INTO fact_table_1(competicio, temporada, pilot, punts_pilot, posicio_pilot, camp_control) 
			SELECT dc.competicio, dc.temporada, dc.pilot, dc.punts, @rownum:=@rownum+1 posicio, '0xCP' 
			FROM (SELECT @rownum:=0) r, dim_classificacio dc 
			WHERE competicio = current_competicio 
			AND temporada = current_temporada 
			ORDER BY dc.punts DESC;
  			
			SET loop_temporada = (loop_temporada + 1);
		END WHILE; -- Loop temporada
		
		SET loop_competicio = (loop_competicio + 1);
	END WHILE; -- Loop competicio

END//
DELIMITER ;


-- Volcando estructura para procedimiento fia.estadistica_consum_total
DELIMITER //
CREATE DEFINER=`root`@`localhost` PROCEDURE `estadistica_consum_total`()
BEGIN

declare loop_pilot SMALLINT;
declare max_pilot SMALLINT;
declare current_pilot SMALLINT;

declare loop_competicio SMALLINT;
declare max_competicio SMALLINT;
declare current_competicio SMALLINT;

declare loop_temporada SMALLINT;
declare max_temporada SMALLINT;
declare current_temporada YEAR;

declare current_cotxe MEDIUMINT (8);
declare current_component MEDIUMINT (8);
declare current_consum FLOAT(10,2);

declare max_consum FLOAT;
declare max_consum_cotxe SMALLINT;
declare current_consum_global FLOAT(10,2);


	DELETE FROM fact_table_1 WHERE camp_control = '0xCC';

	SELECT count(*) 
	INTO max_competicio 
	FROM (
		SELECT *
		FROM cursa
		GROUP BY competicio
	)count_competicio;

	SET loop_competicio = 0;
	
	WHILE loop_competicio < max_competicio DO
	
		SELECT competicio
		INTO current_competicio
		FROM cursa 
		GROUP BY competicio
		LIMIT loop_competicio,1;
		
		SELECT count(*) 
		INTO max_temporada 
		FROM (
			SELECT EXTRACT(YEAR from data_cursa)as temporada 
			FROM cursa  
			WHERE competicio = current_competicio 
			GROUP BY temporada
		)count_temporadas;
				
		SET loop_temporada = 0;
			
		WHILE loop_temporada < max_temporada DO
		
      	SELECT (EXTRACT(YEAR from data_cursa))as temporada
			INTO current_temporada
			FROM cursa 
			WHERE competicio = current_competicio 
			GROUP BY temporada
			LIMIT loop_temporada,1;


		   DROP TEMPORARY TABLE IF EXISTS classificacio_temp;
	      CREATE TEMPORARY TABLE IF NOT EXISTS classificacio_temp 
			( 
				`competicio` SMALLINT (6),
				`temporada`YEAR,
				`cotxe` MEDIUMINT(8),
				`consum` FLOAT(10,2)
			); 

        	SELECT count(*) 
	  		INTO max_pilot 
	  		FROM (
	  			SELECT pilot
	  			FROM cursa
	  			WHERE EXTRACT(YEAR from data_cursa) = current_temporada
	  			AND competicio = current_competicio
	  			GROUP BY pilot
			  )count_pilots;

			SET loop_pilot = 0;
  
     		WHILE loop_pilot < max_pilot DO
     		
     			SELECT pilot 
  				INTO current_pilot
  				FROM cursa 
  				WHERE competicio = current_competicio 
  				AND YEAR(data_cursa) = current_temporada
  				GROUP BY pilot
  				LIMIT loop_pilot,1;

				SELECT cotxe
				INTO current_cotxe
				FROM pilot_cotxe
				WHERE pilot = current_pilot
				AND temporada = current_temporada;
								
				SELECT sum(valor) as consum 
				INTO current_consum
				FROM telemetria t 
				INNER JOIN component c ON  t.component = c.codi_component 
				WHERE EXTRACT(YEAR FROM t.moment) = current_temporada
				AND t.ambit = 'consum'
				AND c.cotxe = current_cotxe;
 				
  				IF current_consum IS NOT NULL THEN
					INSERT INTO classificacio_temp (competicio, temporada, cotxe, consum) 
					VALUES (current_competicio, current_temporada, current_cotxe, current_consum);
				END IF;
  				
				  
				SET loop_pilot = loop_pilot + 1;
			END WHILE; -- Loop pilot

		  	
		  	INSERT INTO dim_consum (competicio, temporada, cotxe, consum) 
		  	SELECT competicio, temporada, cotxe, consum
		  	FROM classificacio_temp
		  	ORDER BY consum DESC; 	
		  	
		  	DELETE FROM fact_table_1 WHERE camp_control = '0xCC' AND temporada = current_temporada AND competicio = current_competicio;
   		INSERT INTO fact_table_1(competicio, temporada, cotxe, consum_cotxe, camp_control) 
  			SELECT dc.competicio, dc.temporada, dc.cotxe, dc.consum, '0xCC' FROM dim_consum dc WHERE dc.competicio = current_competicio AND dc.temporada = current_temporada ORDER BY dc.consum DESC LIMIT 1;


			SET loop_temporada = (loop_temporada + 1);
		END WHILE; -- Loop temporada
		
		SET loop_competicio = (loop_competicio + 1);
	END WHILE; -- Loop competicio
		  


END//
DELIMITER ;


-- Volcando estructura para procedimiento fia.estadistica_curses_finalitzades
DELIMITER //
CREATE DEFINER=`root`@`localhost` PROCEDURE `estadistica_curses_finalitzades`()
BEGIN

	DELETE FROM fact_table_1 WHERE camp_control = '0xCF';
	DELETE FROM dim_resultat WHERE finalitzades IS NOT NULL;

   INSERT INTO dim_resultat (pilot, finalitzades) 
  	SELECT pilot, SUM(CASE WHEN posicio = 0 THEN 0 ELSE 1 END) finished 
	FROM cursa 
	GROUP BY pilot
	ORDER BY finished DESC;
	
	INSERT INTO fact_table_1(pilot, curses_finalitzades, camp_control) 
	SELECT dr.pilot, dr.finalitzades, '0xCF' 
	FROM dim_resultat dr 
	ORDER BY dr.finalitzades DESC
	LIMIT 10;
		
END//
DELIMITER ;


-- Volcando estructura para procedimiento fia.estadistica_curses_resultat
DELIMITER //
CREATE DEFINER=`root`@`localhost` PROCEDURE `estadistica_curses_resultat`()
BEGIN

	DELETE FROM fact_table_1 WHERE camp_control = '0xCG';
	DELETE FROM fact_table_1 WHERE camp_control = '0xCF';
	TRUNCATE TABLE dim_resultat;

   INSERT INTO dim_resultat (pilot, finalitzades, guanyades) 
  	SELECT pilot, SUM(CASE WHEN posicio = 0 THEN 0 ELSE 1 END) finished, SUM(CASE WHEN posicio = 1 THEN 1 ELSE 0 END) won
	FROM cursa 
	GROUP BY pilot
	ORDER BY finished, won DESC;
	
	INSERT INTO fact_table_1(pilot, curses_guanyades, camp_control) 
	SELECT dr.pilot, dr.guanyades, '0xCG' 
	FROM dim_resultat dr 
	ORDER BY dr.guanyades DESC
	LIMIT 10;
	
	INSERT INTO fact_table_1(pilot, curses_finalitzades, camp_control) 
	SELECT dr.pilot, dr.finalitzades, '0xCF' 
	FROM dim_resultat dr 
	ORDER BY dr.finalitzades DESC
	LIMIT 10;
	
	
	
		
END//
DELIMITER ;


-- Volcando estructura para procedimiento fia.estadistica_patrocini_maxim
DELIMITER //
CREATE DEFINER=`root`@`localhost` PROCEDURE `estadistica_patrocini_maxim`()
BEGIN

declare max_temporades TINYINT;
declare current_temporada YEAR;
declare loop_temporada TINYINT;

	DELETE FROM fact_table_1 WHERE camp_control = '0xPA';

	SELECT count(*) 
	INTO max_temporades 
	FROM (
		SELECT temporada
		FROM patrocini
		GROUP BY temporada
	  )count_temporada;

	SET loop_temporada = 0;

	WHILE loop_temporada < max_temporades DO
		SELECT temporada
		INTO current_temporada
		FROM patrocini
		GROUP BY temporada
		LIMIT loop_temporada,1;

	   INSERT INTO fact_table_1 (temporada, patrocinador, cost_patrocini, camp_control) 
	  	SELECT temporada, patrocinador, sum(cost)cost, '0xPA'
		FROM patrocini 
		WHERE temporada = current_temporada
		GROUP BY patrocinador 
		ORDER BY cost DESC
		LIMIT 5;
		
		SET loop_temporada = loop_temporada +1;
	END WHILE;

	INSERT INTO dim_patrocini (patrocinador, cost) 
  	SELECT patrocinador, sum(cost)cost
	FROM patrocini 
	GROUP BY patrocinador 
	ORDER BY cost DESC;
	
END//
DELIMITER ;


-- Volcando estructura para procedimiento fia.estadistica_quantitat_components
DELIMITER //
CREATE DEFINER=`root`@`localhost` PROCEDURE `estadistica_quantitat_components`()
BEGIN

	DELETE FROM fact_table_1 WHERE camp_control = '0xQC';
	TRUNCATE TABLE dim_component;

	INSERT INTO dim_component (fabricant, components) 
	SELECT co.fabricant, count(co.codi_component) as quantitat
	FROM component co
	JOIN pilot_cotxe pc ON pc.cotxe = co.cotxe
	JOIN (
		SELECT c.pilot FROM cursa c GROUP BY pilot
	)AS aux ON aux.pilot = pc.pilot
	GROUP BY co.fabricant
	ORDER BY quantitat DESC;
	
	INSERT INTO fact_table_1 (fabricant, quantitat_components, camp_control) 
	SELECT co.fabricant, co.components, '0xQC'
	FROM dim_component co
	ORDER BY co.components DESC
	LIMIT 5;

END//
DELIMITER ;


-- Volcando estructura para procedimiento fia.estadistica_temperatura_maxima
DELIMITER //
CREATE DEFINER=`root`@`localhost` PROCEDURE `estadistica_temperatura_maxima`()
BEGIN

declare current_pilot SMALLINT;
declare current_cotxe MEDIUMINT (8);
declare current_component MEDIUMINT (8);
declare current_data DATE;
declare current_temperatura FLOAT(8,2);
declare current_circuit MEDIUMINT (8); 


	DELETE FROM fact_table_1 WHERE camp_control = '0xTM';

	SELECT component, valor, DATE(moment)
	INTO current_component, current_temperatura, current_data
	FROM telemetria
	WHERE ambit = 'temperatura' 
	ORDER BY valor DESC
	LIMIT 1;

	SELECT cotxe 
	INTO current_cotxe
	FROM component
	WHERE codi_component = current_component;

	SELECT pilot 
	INTO current_pilot
	FROM pilot_cotxe 
	WHERE cotxe = current_cotxe;
	
	SELECT circuit 
	INTO current_circuit
	FROM cursa 
	WHERE pilot = current_pilot
	AND data_cursa = current_data;

   INSERT INTO fact_table_1 (temporada, pilot, cotxe, circuit, data_cursa, temperatura_maxima, camp_control) 
  	VALUES (YEAR(current_data), current_pilot, current_cotxe, current_circuit, current_data, current_temperatura, '0xTM');

END//
DELIMITER ;


-- Volcando estructura para procedimiento fia.estadistica_volta_rapida
DELIMITER //
CREATE DEFINER=`root`@`localhost` PROCEDURE `estadistica_volta_rapida`()
BEGIN


declare loop_circuit SMALLINT;
declare max_circuit SMALLINT;
declare current_circuit MEDIUMINT (8);

declare loop_pilot SMALLINT;
declare max_pilot SMALLINT;
declare current_pilot SMALLINT;

declare loop_cotxe SMALLINT;
declare max_cotxe SMALLINT;
declare current_cotxe MEDIUMINT (8);

declare loop_data SMALLINT;
declare max_data SMALLINT;
declare current_data DATE;

declare current_equip SMALLINT (5);
declare current_temps_volta TIME;

	DELETE FROM fact_table_1 WHERE camp_control = '0xVR';
	TRUNCATE TABLE dim_volta_rapida;
	
	SELECT count(*) 
	INTO max_circuit 
	FROM (
		SELECT circuit
		FROM cursa
		GROUP BY circuit
	)count_circuit;

	SET loop_circuit = 0;
	
	WHILE loop_circuit < max_circuit DO

    SELECT circuit 
		INTO current_circuit
		FROM cursa 
		GROUP BY circuit
		LIMIT loop_circuit,1;	

		SELECT count(*) 
  		INTO max_data 
  		FROM (
  			SELECT data_cursa
  			FROM cursa
  			WHERE circuit = current_circuit
  			GROUP BY data_cursa
		  )count_data;

		SET loop_data = 0;
  
  		WHILE loop_data < max_data DO
			SELECT data_cursa 
			INTO current_data
  			FROM cursa
  			WHERE circuit = current_circuit
      GROUP BY data_cursa
			LIMIT loop_data,1;

			SELECT count(*) 
	  		INTO max_pilot 
	  		FROM (
	  			SELECT pilot
	  			FROM cursa
	  			WHERE circuit = current_circuit
	  			AND data_cursa = current_data
	  			GROUP BY pilot
			  )count_pilot;
	
			SET loop_pilot = 0;
	  
	  		WHILE loop_pilot < max_pilot DO
				SELECT pilot 
				INTO current_pilot
	  			FROM cursa
	  			WHERE circuit = current_circuit
	  			AND data_cursa = current_data
	  			GROUP BY pilot
				LIMIT loop_pilot,1;
				
				SELECT equip
				INTO current_equip
				FROM pilot
				WHERE codi_persona = current_pilot;	
				
				SELECT cotxe 
		  		INTO current_cotxe 
		  		FROM pilot_cotxe
		  		WHERE pilot = current_pilot
		  		AND temporada = YEAR(current_data);
				
					INSERT INTO dim_volta_rapida (circuit, pilot, equip, data, temps) 
			  		SELECT current_circuit, current_pilot, current_equip, current_data, TIME_FORMAT(SEC_TO_TIME(valor/1000),'%i:%s')
					FROM telemetria 
					WHERE ambit = 'temps per volta' 
					AND DATE(moment) = current_data
					AND component = (
					  	SELECT codi_component 
						FROM component 
						WHERE descripcio = 'Velocimetre' 
						AND cotxe = current_cotxe
					)
					ORDER BY valor LIMIT 10;

				SET loop_pilot = (loop_pilot + 1);
			END WHILE; -- Loop pilot

     		SET loop_data = (loop_data + 1);
		END WHILE; -- loop data


		INSERT INTO fact_table_1 (circuit, pilot, equip, data_cursa, volta_rapida, camp_control) 		
		SELECT dvr.circuit, dvr.pilot, dvr.equip,  dvr.data, dvr.temps, '0xVR'
		FROM dim_volta_rapida dvr
		WHERE dvr.circuit = current_circuit
		ORDER BY temps 
		LIMIT 10;

		SET loop_circuit = (loop_circuit + 1);
	END WHILE; -- Loop circuit
	

END//
DELIMITER ;


-- Volcando estructura para procedimiento fia.estadistica_volum_telemetria
DELIMITER //
CREATE DEFINER=`root`@`localhost` PROCEDURE `estadistica_volum_telemetria`()
BEGIN

declare max_temporades TINYINT;
declare current_temporada YEAR;
declare loop_temporada TINYINT;

declare equip_max_punts SMALLINT;

	DELETE FROM fact_table_1 WHERE camp_control = '0xTE';
	TRUNCATE TABLE dim_telemetria;

	SELECT count(*) 
	INTO max_temporades 
	FROM (
		SELECT YEAR(t.moment)any 
		FROM telemetria t 
		GROUP BY any
	  )count_temporada;

	SET loop_temporada = 0;

	WHILE loop_temporada < max_temporades DO
		SELECT YEAR(t.moment)any 
		INTO current_temporada
		FROM telemetria t 
		GROUP BY any
		LIMIT loop_temporada,1;

		INSERT INTO dim_telemetria (temporada, equip, dades_any) 
		SELECT YEAR(t.moment), ce.equip, count(codi_dada)
		FROM telemetria t
		INNER JOIN component c ON c.codi_component = t.component
		INNER JOIN cotxe_equip ce ON ce.cotxe = c.cotxe
		WHERE YEAR(t.moment) = current_temporada
		GROUP BY ce.equip;
		
		INSERT INTO fact_table_1(temporada, equip, dades_telemetriques, camp_control) 
	   SELECT dt.temporada, dt.equip, SUM(dt.dades_any), '0xTE' 
		FROM dim_telemetria dt 
		WHERE dt.temporada = current_temporada 
		ORDER BY dt.dades_any DESC
		LIMIT 1;
		
		UPDATE fact_table_1
   	SET equip = (
			SELECT dt.equip
			FROM dim_telemetria dt 
			WHERE dt.temporada = current_temporada 
			ORDER BY dt.dades_any DESC
			LIMIT 1)
		WHERE camp_control = '0xTE'
		AND temporada = current_temporada;
	
		SET loop_temporada = loop_temporada +1;
	END WHILE;	
				
END//
DELIMITER ;


-- Volcando estructura para procedimiento fia.me_carreres_finalitzades
DELIMITER //
CREATE DEFINER=`root`@`localhost` PROCEDURE `me_carreres_finalitzades`()
BEGIN


SELECT pilot, curses_finalitzades FROM fact_table_1 WHERE camp_control = '0xCF';
		
END//
DELIMITER ;


-- Volcando estructura para procedimiento fia.me_classificacio
DELIMITER //
CREATE DEFINER=`root`@`localhost` PROCEDURE `me_classificacio`(IN in_competicio MEDIUMINT, IN in_temporada YEAR)
BEGIN


	SELECT pilot, punts_pilot 
	FROM fact_table_1 
	WHERE camp_control = '0xCP' 
	AND temporada = in_temporada 
	AND competicio = in_competicio;
		
END//
DELIMITER ;


-- Volcando estructura para procedimiento fia.me_components_defectuosos
DELIMITER //
CREATE DEFINER=`root`@`localhost` PROCEDURE `me_components_defectuosos`()
BEGIN


SELECT cotxe, components_defectuosos as percentatge  FROM fact_table_1 WHERE camp_control = '0xCD';
		
END//
DELIMITER ;


-- Volcando estructura para procedimiento fia.me_consum_maxim
DELIMITER //
CREATE DEFINER=`root`@`localhost` PROCEDURE `me_consum_maxim`(IN in_temporada YEAR, IN in_competicio MEDIUMINT)
BEGIN


	SELECT cotxe, consum_cotxe as consum
	FROM fact_table_1 
	WHERE camp_control = '0xCC' 
	AND competicio = in_competicio 
	and temporada = in_temporada;


		
END//
DELIMITER ;


-- Volcando estructura para procedimiento fia.me_curses_guanyades
DELIMITER //
CREATE DEFINER=`root`@`localhost` PROCEDURE `me_curses_guanyades`()
BEGIN


	SELECT pilot, curses_guanyades 
	FROM fact_table_1 
	WHERE camp_control = '0xCG';
		
END//
DELIMITER ;


-- Volcando estructura para procedimiento fia.me_dades_telemetriques
DELIMITER //
CREATE DEFINER=`root`@`localhost` PROCEDURE `me_dades_telemetriques`(IN in_temporada YEAR)
BEGIN


 SELECT equip, dades_telemetriques 
 FROM fact_table_1 
 WHERE camp_control = '0xTE' 
 AND temporada = in_temporada;
		
END//
DELIMITER ;


-- Volcando estructura para procedimiento fia.me_millor_evolucio
DELIMITER //
CREATE DEFINER=`root`@`localhost` PROCEDURE `me_millor_evolucio`(IN `in_data_cura` DATE, IN `in_data_millora` DATE)
BEGIN


	SELECT pilot, punts_pilot as evolucio_punts
	FROM fact_table_1 
	WHERE camp_control = '0xEP' 
	AND data_cursa >= in_data_cura 
	AND data_millora <= in_data_millora
	AND data_millora IS NOT NULL 
	ORDER BY punts_pilot DESC 
	LIMIT 1;

		
END//
DELIMITER ;


-- Volcando estructura para procedimiento fia.me_resultat_pilots
DELIMITER //
CREATE DEFINER=`root`@`localhost` PROCEDURE `me_resultat_pilots`()
BEGIN


SELECT competicio, temporada, pilot, posicio_pilot as posicio FROM fact_table_1 WHERE camp_control = '0xCP';
		
END//
DELIMITER ;


-- Volcando estructura para procedimiento fia.me_temperatura_maxima
DELIMITER //
CREATE DEFINER=`root`@`localhost` PROCEDURE `me_temperatura_maxima`()
BEGIN


	SELECT temporada, competicio, circuit, pilot, cotxe, data_cursa, temperatura_maxima as temperatura
	FROM fact_table_1 
	WHERE camp_control = '0xTM';
		
END//
DELIMITER ;


-- Volcando estructura para procedimiento fia.me_top_fabricants
DELIMITER //
CREATE DEFINER=`root`@`localhost` PROCEDURE `me_top_fabricants`()
BEGIN


	SELECT fabricant, quantitat_components as quantitat
	FROM fact_table_1 
	WHERE camp_control = '0xQC';
		
END//
DELIMITER ;


-- Volcando estructura para procedimiento fia.me_top_patrocinadors
DELIMITER //
CREATE DEFINER=`root`@`localhost` PROCEDURE `me_top_patrocinadors`()
BEGIN


	SELECT patrocinador, cost_patrocini as patrocini
	FROM fact_table_1 
	WHERE camp_control = '0xPA';
		
END//
DELIMITER ;


-- Volcando estructura para procedimiento fia.me_volta_rapida
DELIMITER //
CREATE DEFINER=`root`@`localhost` PROCEDURE `me_volta_rapida`(IN in_circuit MEDIUMINT)
BEGIN


	SELECT pilot, equip, data_cursa, volta_rapida as temps
	FROM fact_table_1 
	WHERE camp_control = '0xVR' 
	AND circuit = in_circuit;
		
END//
DELIMITER ;


-- Volcando estructura para procedimiento fia.modificacio_circuit
DELIMITER //
CREATE DEFINER=`root`@`localhost` PROCEDURE `modificacio_circuit`(IN `in_codi` MEDIUMINT, IN `in_nom` VARCHAR(50), IN `in_sobrenom` VARCHAR(50), IN `in_longitud` SMALLINT, IN `in_nacionalitat` CHAR(2), OUT `RSP` VARCHAR(512))
BEGIN

declare valor_antic VARCHAR (512);
declare valor_nou VARCHAR (512);
declare consulta VARCHAR(512);
declare msg VARCHAR(250);
declare state VARCHAR(50);
declare chk_circuit MEDIUMINT;

declare old_nom VARCHAR(50);
declare old_sobrenom VARCHAR(50);
declare old_longitud SMALLINT;
declare old_nacionalitat CHAR(2);


-- Control d'errors

DECLARE EXIT HANDLER FOR NOT FOUND
BEGIN
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO;
	SET RSP := CONCAT('ERROR: Not found. Codi ',state,' .Possiblement el codi circuit (',in_codi,'), no sigui correcte.');
END;

-- TRIGGER chk_circuit_long

DECLARE EXIT HANDLER FOR 1644 
BEGIN
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT("ERROR: Codi ",state,". ",msg);
END;

DECLARE EXIT HANDLER FOR 1062 
BEGIN
GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
SET RSP := CONCAT('ERROR: Codi ',state,'.Nom de circuit (',in_nom,') o sobrenom (',IFNULL(in_sobrenom,"NULL"),') duplicat.');
END;

DECLARE EXIT HANDLER FOR 1452
BEGIN
GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
SET RSP := CONCAT('ERROR: Codi ',state,'. La clau forana (',in_nacionalitat,') no es correcte.');
END;

DECLARE EXIT HANDLER FOR SQLEXCEPTION
BEGIN
GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
SET RSP := CONCAT('ERROR: Desconegut. Codi ',state,'. Missatge original: ',msg);
END;

DECLARE EXIT HANDLER FOR SQLWARNING 
BEGIN 
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Warning. Codi ',state,'. Missatge original: ',msg);
END;

	SET RSP := 'OK';

	SELECT c.nom, c.sobrenom, c.longitud, c.pais 
	INTO old_nom, old_sobrenom, old_longitud, old_nacionalitat 
	FROM circuit c 
	WHERE c.codi_circuit = in_codi;


	-- Modifica la taula
	
	UPDATE circuit c
	SET c.nom = in_nom, c.sobrenom = in_sobrenom, c.longitud = in_longitud, c.pais = in_nacionalitat
	WHERE c.codi_circuit = in_codi;
			
		
	-- Actualitzar log

	SET consulta = CONCAT("modificacio_circuit ('",in_codi,"', '",in_nom,"', '",(IFNULL(in_sobrenom,"NULL")),"', ",(IFNULL(in_longitud,"NULL")),", '",in_nacionalitat,"')");
	SET valor_nou = CONCAT_WS(', ',in_codi, in_nom, in_sobrenom, in_longitud, in_nacionalitat);
	SET valor_antic = CONCAT_WS(',', in_codi, old_nom, old_sobrenom, old_longitud, old_nacionalitat);
	
	INSERT INTO log(operacio, taula,antic_valor,nou_valor,data,consulta) VALUES ('MODIFICACIÓ', 'circuit', valor_antic, valor_nou, NOW(), consulta);

END//
DELIMITER ;


-- Volcando estructura para procedimiento fia.modificacio_competicio
DELIMITER //
CREATE DEFINER=`root`@`localhost` PROCEDURE `modificacio_competicio`(IN `in_codi` SMALLINT, IN `in_nom` VARCHAR(50), IN `in_acronim` VARCHAR(10), IN `in_descripcio` TEXT, OUT `RSP` VARCHAR(512))
BEGIN

declare valor_antic VARCHAR (512);
declare valor_nou VARCHAR (512);
declare consulta VARCHAR(512);
declare msg VARCHAR(250);
declare state VARCHAR(50);

declare old_nom VARCHAR(50);
declare old_acronim VARCHAR(10);
declare old_descripcio TEXT;


-- Control d'errors

DECLARE EXIT HANDLER FOR NOT FOUND
BEGIN
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO;
	SET RSP := CONCAT('ERROR: Not found. Codi ',state,' .Possiblement el codi de la competició (',in_codi,'), no sigui correcte.');
END;


-- TRIGGER chk_circuit_long
DECLARE EXIT HANDLER FOR 1644 
BEGIN
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT("ERROR: Codi ",state,". ",msg);
END;

DECLARE EXIT HANDLER FOR 1062 
BEGIN
GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
SET RSP := CONCAT('ERROR: Codi ',state,'.Nom de la competicio (',in_nom,') o acronim (',IFNULL(in_acronim,"NULL"),') duplicat.');
END;

DECLARE EXIT HANDLER FOR SQLEXCEPTION
BEGIN
GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
SET RSP := CONCAT('ERROR: Desconegut. Codi ',state,'. Missatge original: ',msg);
END;

DECLARE EXIT HANDLER FOR SQLWARNING 
BEGIN 
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Warning. Codi ',state,'. Missatge original: ',msg);
END;

	SET RSP := 'OK';

	SELECT c.nom, c.acronim, c.descripcio
	INTO old_nom, old_acronim, old_descripcio 
	FROM competicio c
	WHERE c.codi_competicio = in_codi;


	-- Modifica la taula
	
	UPDATE competicio c
	SET c.nom = in_nom, c.acronim = in_acronim, c.descripcio = in_descripcio
	WHERE c.codi_competicio = in_codi;
			
		
	-- Actualitzar log

	SET consulta = CONCAT("modificacio_competicio ('",in_codi,"', '",in_nom,"', '",in_acronim,", '",in_descripcio,"')");	
	SET valor_nou = CONCAT_WS(', ',in_codi, in_nom, in_acronim, in_descripcio);
	SET valor_antic = CONCAT_WS(',', in_codi, old_nom, old_acronim, old_descripcio);
	
	INSERT INTO log(operacio, taula,antic_valor,nou_valor,data,consulta) VALUES ('MODIFICACIÓ', 'competicio', valor_antic, valor_nou, NOW(), consulta);

END//
DELIMITER ;


-- Volcando estructura para procedimiento fia.modificacio_component
DELIMITER //
CREATE DEFINER=`root`@`localhost` PROCEDURE `modificacio_component`(IN `in_codi` SMALLINT, IN `in_fabricant` MEDIUMINT, IN `in_cotxe` MEDIUMINT, IN `in_descripcio` TEXT, IN `in_utilitat` TEXT, OUT `RSP` VARCHAR(512))
BEGIN

declare valor_antic VARCHAR (512);
declare valor_nou VARCHAR (512);
declare consulta VARCHAR(512);
declare msg VARCHAR(250);
declare state VARCHAR(50);

declare old_fabricant MEDIUMINT;
declare old_cotxe MEDIUMINT;
declare old_descripcio TEXT;
declare old_utilitat TEXT;


-- Control d'errors

DECLARE EXIT HANDLER FOR 1452
BEGIN
GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
SET RSP := CONCAT('ERROR: Codi ',state,'. Possiblement el fabricant (',in_fabricant,') o el cotxe (',in_cotxe,') no son correctes.');
END;

DECLARE EXIT HANDLER FOR NOT FOUND
 BEGIN
 	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
   SET RSP := CONCAT('ERROR: Not found. Codi ',state,'. El nom del fabricant o del cotxe no son correctes.' );
 END;

DECLARE EXIT HANDLER FOR SQLWARNING 
BEGIN 
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Warning. Codi ',state,'. Missatge original: ',msg);
END;

DECLARE CONTINUE HANDLER FOR SQLEXCEPTION
BEGIN
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Desconegut. Codi ',state,'. Missatge original: ',msg);
END;

	SET RSP := 'OK';

	SELECT c.fabricant, c.cotxe, c.descripcio, c.utilitat
	INTO old_fabricant, old_cotxe, old_descripcio, old_utilitat
	FROM component c
	WHERE  c.codi_component = in_codi;
		

	-- Modifica la taula
	
	UPDATE component c 
	SET c.fabricant = in_fabricant, c.cotxe = in_cotxe, c.descripcio = in_descripcio, c.utilitat = in_utilitat
	WHERE c.codi_component = in_codi;
		
		
	-- Actualitzar log

	SET consulta = CONCAT("modificacio_component ('",in_codi,"','",in_fabricant,"','",in_cotxe,"', '",in_descripcio,"', '",in_utilitat,"')");	
	SET valor_nou = CONCAT_WS(', ',in_codi, in_fabricant, in_cotxe, in_descripcio, in_utilitat);
	SET valor_antic = CONCAT_WS(',',in_codi, old_fabricant, old_cotxe, old_descripcio, old_utilitat);
	
	INSERT INTO log(operacio, taula,antic_valor,nou_valor,data,consulta) VALUES ('MODIFICACIÓ', 'component', valor_antic, valor_nou, NOW(), consulta);
	
	
	
	
	
	
	
	
	
END//
DELIMITER ;


-- Volcando estructura para procedimiento fia.modificacio_cotxe
DELIMITER //
CREATE DEFINER=`root`@`localhost` PROCEDURE `modificacio_cotxe`(IN `in_codi` MEDIUMINT, IN `in_model` VARCHAR(25), IN `in_any` YEAR, OUT `RSP` VARCHAR(512))
BEGIN

declare valor_antic VARCHAR (512);
declare valor_nou VARCHAR (512);
declare consulta VARCHAR(512);
declare msg VARCHAR(250);
declare state VARCHAR(50);

declare old_model VARCHAR(25);
declare old_any YEAR;
declare old_equip SMALLINT;


-- Control d'errors

DECLARE EXIT HANDLER FOR 1062 
BEGIN
GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
SET RSP := CONCAT('ERROR: Codi ',state,'. Possiblement el model de cotxe (',in_model,') ja existeix a la taula \'cotxe\'.');
END;

DECLARE EXIT HANDLER FOR NOT FOUND
 BEGIN
 	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
   SET RSP := CONCAT('ERROR: Not found. Codi ',state,'. El codi del cotxe no es correcte.' );
 END;

DECLARE EXIT HANDLER FOR SQLWARNING 
BEGIN 
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Warning. Codi ',state,'. Missatge original: ',msg);
END;

DECLARE CONTINUE HANDLER FOR SQLEXCEPTION
BEGIN
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Desconegut. Codi ',state,'. Missatge original: ',msg);
END;

	
	SET RSP := 'OK';
	
	SELECT c.model, c.any_debut
	INTO old_model, old_any
	FROM cotxe c
	WHERE  c.codi_cotxe = in_codi;


	-- Modifica la taula
	
	UPDATE cotxe c 
	SET c.model = in_model, c.any_debut = in_any
	WHERE c.codi_cotxe = in_codi;
			
		
	-- Actualitzar log
	
	SET consulta = CONCAT("modificacio_cotxe ('",in_codi,"','",in_model,"','",in_any,"')");
	SET valor_nou = CONCAT_WS(', ',in_codi, in_model, in_any);
	SET valor_antic = CONCAT_WS(',',in_codi, old_model, old_any);
	
	INSERT INTO log(operacio, taula,antic_valor,nou_valor,data,consulta) VALUES ('MODIFICACIÓ', 'cotxe', valor_antic, valor_nou, NOW(), consulta);
	
END//
DELIMITER ;


-- Volcando estructura para procedimiento fia.modificacio_cursa
DELIMITER //
CREATE DEFINER=`root`@`localhost` PROCEDURE `modificacio_cursa`(IN `in_circuit` MEDIUMINT, IN `in_pilot` MEDIUMINT, IN `in_competicio` SMALLINT, IN `in_posicio` SMALLINT, IN `in_data_cursa` DATE, OUT `RSP` VARCHAR(512))
BEGIN

declare valor_antic VARCHAR (512);
declare valor_nou VARCHAR (512);
declare consulta VARCHAR(512);
declare msg VARCHAR(250);
declare state VARCHAR(50);
declare check_temporada YEAR;
declare any_temporada YEAR;

declare old_posicio SMALLINT;
declare old_data_cursa DATE;


-- Control d'errors

DECLARE EXIT HANDLER FOR NOT FOUND
 BEGIN
 	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
   SET RSP := CONCAT('ERROR: Not found. Codi ',state,'. Possiblement el circuit (',in_circuit,'), el pilot (',in_pilot,') o la parella competicio-any (',in_competicio,', ',any_temporada,') no son correctes.' );
 END;

DECLARE EXIT HANDLER FOR SQLWARNING 
BEGIN 
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Warning. Codi ',state,'. Missatge original: ',msg);
END;

DECLARE CONTINUE HANDLER FOR SQLEXCEPTION
BEGIN
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Desconegut. Codi ',state,'. Missatge original: ',msg);
END;

	
	SET RSP := 'OK';
	SET any_temporada = EXTRACT(YEAR from in_data_cursa);
	
	SELECT temporada FROM competicio_temporada WHERE temporada = any_temporada AND competicio = in_competicio INTO check_temporada;
	
	SELECT c.posicio, c.data_cursa
	INTO old_posicio, old_data_cursa
	FROM cursa c
	WHERE c.circuit = in_circuit 
	AND c.pilot = in_pilot 
	AND c.competicio = in_competicio;
		

	-- Modifica la taula
	
	UPDATE cursa c 
	SET c.posicio = in_posicio, c.data_cursa = in_data_cursa
	WHERE c.circuit = in_circuit 
	AND c.pilot = in_pilot 
	AND c.competicio = in_competicio;
		
		
	-- Actualitzar log

	SET consulta = CONCAT("modificacio_cursa ('",in_circuit,"','",in_pilot,"','",in_competicio,"','",in_posicio,"','",in_data_cursa,"')");
	SET valor_nou = CONCAT_WS(', ',in_circuit, in_pilot, in_competicio, in_posicio, in_data_cursa);
	SET valor_antic = CONCAT_WS(',',in_circuit, in_pilot, in_competicio, old_posicio, old_data_cursa);
	
	INSERT INTO log(operacio, taula,antic_valor,nou_valor,data,consulta) VALUES ('MODIFICACIÓ', 'cursa', valor_antic, valor_nou, NOW(), consulta);
END//
DELIMITER ;


-- Volcando estructura para procedimiento fia.modificacio_director
DELIMITER //
CREATE DEFINER=`root`@`localhost` PROCEDURE `modificacio_director`(IN `in_codi` MEDIUMINT, IN `in_nom` VARCHAR(100), IN `in_pais` CHAR(2), IN `in_equip` SMALLINT, IN `in_estudis` TEXT, IN `in_inici` YEAR, OUT `RSP` VARCHAR(512))
BEGIN

declare valor_antic VARCHAR (512);
declare valor_nou VARCHAR (512);
declare consulta_director VARCHAR(512);
declare consulta_persona VARCHAR(512);
declare msg VARCHAR(250);
declare state VARCHAR(50);

declare old_nom VARCHAR(100);
declare old_pais CHAR(2);
declare old_equip SMALLINT;
declare old_estuds TEXT;
declare old_inici YEAR;

declare check_equip SMALLINT;
declare check_nacionalitat CHAR(2);


-- Control d'errors

DECLARE EXIT HANDLER FOR 1062 
BEGIN
GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
SET RSP := CONCAT('ERROR: Codi ',state,'. Possiblement l\'equip (',in_equip,') ja te director assignat.');
END;

DECLARE EXIT HANDLER FOR NOT FOUND
 BEGIN
 	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
   SET RSP := CONCAT('ERROR: Not found. Codi ',state,'. Possiblement el codi de la persona (',in_codi,'), el codi de l\'equip (',in_equip,') o el codi del pais (',in_pais,') no sigui correcte.');
 END;

DECLARE EXIT HANDLER FOR SQLWARNING 
BEGIN 
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Warning. Codi ',state,'. Missatge original: ',msg);
END;

DECLARE CONTINUE HANDLER FOR SQLEXCEPTION
BEGIN
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Desconegut. Codi ',state,'. Missatge original: ',msg);
END;

	
	SET RSP := 'OK';
	
	-- Es fa servir SELECT INTO per tal de llençar l'excepcio  NOT FOUND si no es troba el codi a la taula.
	SELECT e.codi_equip FROM equip e WHERE e.codi_equip = in_equip INTO check_equip;
	SELECT p.codi_pais FROM pais p WHERE p.codi_pais = in_pais INTO check_nacionalitat;
	
	SELECT p.nom, p.nacionalitat, d.equip, d.estudis, d.any_inici
	INTO old_nom, old_pais, old_equip, old_estuds, old_inici
	FROM persona p 
	JOIN director d 
	ON p.codi_persona = d.codi_persona
	AND d.codi_persona = in_codi;
		

	-- Modifica la taula
	
	UPDATE director d
	SET d.equip = in_equip, d.estudis = in_estudis, d.any_inici = in_inici
	WHERE d.codi_persona = in_codi;
					
		
	-- Actualitzar log director
	
	SET consulta_director = CONCAT("modificacio_director ('",in_nom,"', '",in_pais,"', '",in_equip,"', '",in_estudis,"', ",in_inici,"')");
	SET valor_nou = CONCAT_WS(', ',in_codi, in_equip, in_estudis, in_inici);
	SET valor_antic = CONCAT_WS(',',in_codi, old_equip, old_estuds, old_inici);
	INSERT INTO log(operacio, taula,antic_valor,nou_valor,data,consulta) VALUES ('MODIFICACIÓ', 'director', valor_antic, valor_nou, NOW(), consulta_director);
		

	-- Modifica la taula
	
	UPDATE persona p
	SET p.nom = in_nom, p.nacionalitat = in_pais
	WHERE p.codi_persona = in_codi;
			
		
	-- Actualitzar log persona
	
	SET consulta_persona = CONCAT("INDIRECTE -> modificacio_director ('",in_nom,"', '",in_pais,"', '",in_equip,"', '",in_estudis,"', ",in_inici,"')");
	SET valor_nou = CONCAT_WS(', ',in_codi, in_nom, in_pais);
	SET valor_antic = CONCAT_WS(',',in_codi, old_nom, old_pais);
	
	INSERT INTO log(operacio, taula,antic_valor,nou_valor,data,consulta) VALUES ('MODIFICACIÓ', 'persona', valor_antic, valor_nou, NOW(), consulta_persona);
		
END//
DELIMITER ;


-- Volcando estructura para procedimiento fia.modificacio_emissora
DELIMITER //
CREATE DEFINER=`root`@`localhost` PROCEDURE `modificacio_emissora`(IN `in_codi` SMALLINT, IN `in_nom` VARCHAR(50), IN `in_pais` CHAR(2), OUT `RSP` VARCHAR(512))
BEGIN

declare valor_antic VARCHAR (512);
declare valor_nou VARCHAR (512);
declare consulta VARCHAR(512);
declare msg VARCHAR(250);
declare state VARCHAR(50);

declare old_nom VARCHAR(100);
declare old_pais CHAR(2);

declare check_nacionalitat CHAR(2);


-- Control d'errors

DECLARE EXIT HANDLER FOR 1062 
BEGIN
GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
SET RSP := CONCAT('ERROR: Codi ',state,'. Possiblement l\'emissora (',in_nom,') ja existeix a aquest pais (',in_pais,') a la taula \'emissora\'');
END;

DECLARE EXIT HANDLER FOR NOT FOUND
 BEGIN
 	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
   SET RSP := CONCAT('ERROR: Not found. Codi ',state,'.Possiblement el codi del pais (',in_pais,') no sigui correcte.');
 END;

DECLARE EXIT HANDLER FOR SQLWARNING 
BEGIN 
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Warning. Codi ',state,'. Missatge original: ',msg);
END;

DECLARE CONTINUE HANDLER FOR SQLEXCEPTION
BEGIN
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Desconegut. Codi ',state,'. Missatge original: ',msg);
END;

	
	SET RSP := 'OK';

	-- Es fa servir SELECT INTO per tal de llençar l'excepcio  NOT FOUND si no es troba el codi a la taula.
	SELECT p.codi_pais FROM pais p WHERE p.codi_pais = in_pais INTO check_nacionalitat;
		
	SELECT e.nom, e.pais
	INTO old_nom, old_pais
	FROM emissora e 
	WHERE e.codi_emissora = in_codi;
		

	-- Modifica la taula
	
	UPDATE emissora e
	SET e.nom = in_nom, e.pais = in_pais
	WHERE e.codi_emissora = in_codi;
			
		
	-- Actualitzar log
	
	SET consulta = CONCAT("modificacio_emissora ('",in_codi,"', '",in_nom,"', '",in_pais,"')");
	SET valor_nou = CONCAT_WS(', ',in_codi, in_nom, in_pais);
	SET valor_antic = CONCAT_WS(',',in_codi, old_nom, old_pais);
	
	INSERT INTO log(operacio, taula,antic_valor,nou_valor,data,consulta) VALUES ('MODIFICACIÓ', 'emissora', valor_antic, valor_nou, NOW(), consulta);
		
END//
DELIMITER ;


-- Volcando estructura para procedimiento fia.modificacio_equip
DELIMITER //
CREATE DEFINER=`root`@`localhost` PROCEDURE `modificacio_equip`(IN `in_codi` SMALLINT, IN `in_nom` VARCHAR(50), IN `in_seu` VARCHAR(100), IN `in_debut` DATE, OUT `RSP` VARCHAR(512))
BEGIN

declare valor_antic VARCHAR (512);
declare valor_nou VARCHAR (512);
declare consulta VARCHAR(512);
declare msg VARCHAR(250);
declare state VARCHAR(50);

declare old_nom VARCHAR(50);
declare old_seu VARCHAR(100);
declare old_debut DATE;


-- Control d'errors

DECLARE EXIT HANDLER FOR 1062 
BEGIN
GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
SET RSP := CONCAT('ERROR: Codi ',state,'. Possiblement l\'equip (',in_nom,') ja existeix a la taula \'equip\'.');
END;

DECLARE EXIT HANDLER FOR NOT FOUND
 BEGIN
 	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
   SET RSP := CONCAT('ERROR: Not found. Codi ',state,'.Possiblement el codi de l\'equip (',in_codi,') no sigui correcte.');
 END;

DECLARE EXIT HANDLER FOR SQLWARNING 
BEGIN 
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Warning. Codi ',state,'. Missatge original: ',msg);
END;

DECLARE CONTINUE HANDLER FOR SQLEXCEPTION
BEGIN
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Desconegut. Codi ',state,'. Missatge original: ',msg);
END;

	
	SET RSP := 'OK';
			
	SELECT e.nom, e.seu, e.data_debut
	INTO old_nom, old_seu, old_debut
	FROM equip e 
	WHERE e.codi_equip = in_codi;
		

	-- Modifica la taula
	
	UPDATE equip e
	SET e.nom = in_nom, e.seu = in_seu, e.data_debut = in_debut
	WHERE e.codi_equip = in_codi;
			
		
	-- Actualitzar log
	
	SET consulta = CONCAT("modificacio_equip ('",in_codi,"', '",in_nom,"', '",in_seu,"', '",in_debut,"')");	
	SET valor_nou = CONCAT_WS(', ',in_codi, in_nom, in_seu, in_debut);
	SET valor_antic = CONCAT_WS(',',in_codi, old_nom, old_seu, old_debut);
	
	INSERT INTO log(operacio, taula,antic_valor,nou_valor,data,consulta) VALUES ('MODIFICACIÓ', 'equip', valor_antic, valor_nou, NOW(), consulta);
		
END//
DELIMITER ;


-- Volcando estructura para procedimiento fia.modificacio_fabricant
DELIMITER //
CREATE DEFINER=`root`@`localhost` PROCEDURE `modificacio_fabricant`(IN `in_codi` MEDIUMINT, IN `in_nom` VARCHAR(50), IN `in_debut` YEAR, OUT `RSP` VARCHAR(512))
BEGIN

declare valor_antic VARCHAR (512);
declare valor_nou VARCHAR (512);
declare consulta VARCHAR(512);
declare msg VARCHAR(250);
declare state VARCHAR(50);

declare old_nom VARCHAR(50);
declare old_debut YEAR;


-- Control d'errors

DECLARE EXIT HANDLER FOR 1062 
BEGIN
GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
SET RSP := CONCAT('ERROR: Codi ',state,'. Possiblement el nom del fabricant (',in_nom,') ja existeix a la taula \'fabricant\'');
END;

DECLARE EXIT HANDLER FOR NOT FOUND
 BEGIN
 	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
   SET RSP := CONCAT('ERROR: Not found. Codi ',state,'.Possiblement el codi del fabricant (',in_codi,') no sigui correcte.');
 END;

DECLARE EXIT HANDLER FOR SQLWARNING 
BEGIN 
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Warning. Codi ',state,'. Missatge original: ',msg);
END;

DECLARE CONTINUE HANDLER FOR SQLEXCEPTION
BEGIN
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Desconegut. Codi ',state,'. Missatge original: ',msg);
END;

	
	SET RSP := 'OK';
			
	SELECT f.nom, f.any_debut
	INTO old_nom, old_debut
	FROM fabricant f
	WHERE f.codi_fabricant = in_codi;
	

	-- Modifica la taula
		
	UPDATE fabricant f
	SET f.nom = in_nom, f.any_debut = in_debut
	WHERE f.codi_fabricant = in_codi;
			
		
	-- Actualitzar log
	
	SET consulta = CONCAT("modificacio_fabricant ('",in_codi,"', '",in_nom,"', '",in_debut,")");
	SET valor_nou = CONCAT_WS(', ',in_codi, in_nom, in_debut);
	SET valor_antic = CONCAT_WS(',',in_codi, old_nom, old_debut);
	
	INSERT INTO log(operacio, taula,antic_valor,nou_valor,data,consulta) VALUES ('MODIFICACIÓ', 'fabricant', valor_antic, valor_nou, NOW(), consulta);
		
END//
DELIMITER ;


-- Volcando estructura para procedimiento fia.modificacio_pais
DELIMITER //
CREATE DEFINER=`root`@`localhost` PROCEDURE `modificacio_pais`(IN `in_codi` CHAR(2), IN `in_nom` VARCHAR(50), OUT `RSP` VARCHAR(512))
BEGIN

declare valor_antic VARCHAR (512);
declare valor_nou VARCHAR (512);
declare consulta VARCHAR(512);
declare msg VARCHAR(250);
declare state VARCHAR(50);

declare old_nom VARCHAR(50);


-- Control d'errors

DECLARE EXIT HANDLER FOR 1062 
BEGIN
GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
SET RSP := CONCAT('ERROR: Codi ',state,'. Possiblement el nom del pais (',in_nom,') ja existeix a la taula \'pais\'');
END;

DECLARE EXIT HANDLER FOR NOT FOUND
 BEGIN
 	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
   SET RSP := CONCAT('ERROR: Not found. Codi ',state,'.Possiblement el codi del pais (',in_codi,') no sigui correcte.');
 END;

DECLARE EXIT HANDLER FOR SQLWARNING 
BEGIN 
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Warning. Codi ',state,'. Missatge original: ',msg);
END;

DECLARE CONTINUE HANDLER FOR SQLEXCEPTION
BEGIN
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Desconegut. Codi ',state,'. Missatge original: ',msg);
END;

	
	SET RSP := 'OK';
			
	SELECT p.nom
	INTO old_nom
	FROM pais p
	WHERE p.codi_pais = in_codi;
		

	-- Modifica la taula
	
	UPDATE pais p
	SET p.nom = in_nom
	WHERE p.codi_pais = in_codi;
			
		
	-- Actualitzar log

	SET consulta = CONCAT("modificacio_pais ('",in_codi,"', '",in_nom,"')");	
	SET valor_nou = CONCAT_WS(', ',in_codi, in_nom);
	SET valor_antic = CONCAT_WS(',',in_codi, old_nom);
	
	INSERT INTO log(operacio, taula,antic_valor,nou_valor,data,consulta) VALUES ('MODIFICACIÓ', 'pais', valor_antic, valor_nou, NOW(), consulta);
		
END//
DELIMITER ;


-- Volcando estructura para procedimiento fia.modificacio_patrocinador
DELIMITER //
CREATE DEFINER=`root`@`localhost` PROCEDURE `modificacio_patrocinador`(IN `in_codi` MEDIUMINT, IN `in_sector` SMALLINT, IN `in_nom` VARCHAR(50), OUT `RSP` VARCHAR(512))
BEGIN

declare valor_antic VARCHAR (512);
declare valor_nou VARCHAR (512);
declare consulta VARCHAR(512);
declare msg VARCHAR(250);
declare state VARCHAR(50);

declare old_nom VARCHAR(50);
declare old_sector SMALLINT;

declare check_sector SMALLINT;


-- Control d'errors

DECLARE EXIT HANDLER FOR 1062 
BEGIN
GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
SET RSP := CONCAT('ERROR: Codi ',state,'. Possiblement nom del patrocinador (',in_nom,') ja existeix a la taula \'patrocinador\'.');
END;

DECLARE EXIT HANDLER FOR NOT FOUND
 BEGIN
 	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
   SET RSP := CONCAT('ERROR: Not found. Codi ',state,'. Possiblement el codi del patrocinador (',in_codi,') o el sector del patrocinador (',in_sector,') no sigui correcte.' );
 END;

DECLARE EXIT HANDLER FOR SQLWARNING 
BEGIN 
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Warning. Codi ',state,'. Missatge original: ',msg);
END;

DECLARE CONTINUE HANDLER FOR SQLEXCEPTION
BEGIN
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Desconegut. Codi ',state,'. Missatge original: ',msg);
END;

	
	SET RSP := 'OK';
	SELECT codi_sector FROM sector WHERE codi_sector = in_sector INTO check_sector;
	
	SELECT p.sector, p.nom
	INTO old_sector, old_nom
	FROM patrocinador p
	WHERE p.codi_patrocinador = in_codi;
		

	-- Modifica la taula
	
	UPDATE patrocinador p
	SET p.nom = in_nom, p.sector = in_sector
	WHERE p.codi_patrocinador = in_codi;
			
		
	-- Actualitzar log

	SET consulta = CONCAT("modificacio_patrocinador ('",in_codi,"', '",in_sector,"', '",in_nom,"')");
	SET valor_nou = CONCAT_WS(', ',in_codi, in_sector, in_nom);
	SET valor_antic = CONCAT_WS(',',in_codi, old_sector, old_nom);
	
	INSERT INTO log(operacio, taula,antic_valor,nou_valor,data,consulta) VALUES ('MODIFICACIÓ', 'patrocinador', valor_antic, valor_nou, NOW(), consulta);
		
END//
DELIMITER ;


-- Volcando estructura para procedimiento fia.modificacio_patrocini
DELIMITER //
CREATE DEFINER=`root`@`localhost` PROCEDURE `modificacio_patrocini`(IN `in_patrocinador` MEDIUMINT, IN `in_equip` SMALLINT, IN `in_temporada` YEAR, IN `in_cost` INT, OUT `RSP` VARCHAR(512))
BEGIN

declare valor_antic VARCHAR (512);
declare valor_nou VARCHAR (512);
declare consulta VARCHAR(512);
declare msg VARCHAR(250);
declare state VARCHAR(50);

declare old_temporada YEAR;
declare old_cost INT;

declare check_temporada YEAR;


-- Control d'errors

DECLARE EXIT HANDLER FOR 1644 
BEGIN
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT("ERROR: Codi ",state,". ",msg);
END;

DECLARE EXIT HANDLER FOR 1062 
BEGIN
GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO;
SET RSP := CONCAT('ERROR: Codi ',state,'. Ja existeix aquest patrocini (',in_patrocinador,', ',in_equip,'), per aquesta temporada (',in_temporada,') a la taula \'patrocini\'.');
END;

DECLARE EXIT HANDLER FOR NOT FOUND
 BEGIN
 	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO;
   SET RSP := CONCAT('ERROR: Not found. Codi ',state,' .Possiblement l\'equip (\'',in_equip,'\'), el patrocinador (\'',in_patrocinador,'\'), o la temporada (',in_temporada,') no existeixen al sistema.');
 END;

DECLARE EXIT HANDLER FOR SQLWARNING 
BEGIN 
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Warning. Codi ',state,'. Missatge original: ',msg);
END;

DECLARE EXIT HANDLER FOR SQLEXCEPTION
BEGIN
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Desconegut. Codi ',state,'. Missatge original: ',msg);
END;
	
	SET RSP := 'OK';
	
	SELECT t.any FROM temporada t
	WHERE t.any = in_temporada 
	INTO check_temporada;
		
	SELECT p.temporada, p.cost
	INTO old_temporada, old_cost
	FROM patrocini p
	WHERE p.patrocinador = in_patrocinador
	AND p.equip = in_equip;
		

	-- Modifica la taula
	
	UPDATE patrocini p
	SET p.temporada = in_temporada, p.cost = in_cost
	WHERE p.patrocinador = in_patrocinador
	AND p.equip = in_equip;
			
		
	-- Actualitzar log
	
	SET consulta = CONCAT("modificacio_patrocini ('",in_patrocinador,"', '",in_equip,"', '",in_temporada,"', '",in_cost,"')");
	SET valor_nou = CONCAT_WS(', ',in_patrocinador, in_equip, in_temporada, in_cost);
	SET valor_antic = CONCAT_WS(',',in_patrocinador, in_equip, old_temporada, old_cost);
	
	INSERT INTO log(operacio, taula,antic_valor,nou_valor,data,consulta) VALUES ('MODIFICACIÓ', 'patrocini', valor_antic, valor_nou, NOW(), consulta);
		
END//
DELIMITER ;


-- Volcando estructura para procedimiento fia.modificacio_pilot
DELIMITER //
CREATE DEFINER=`root`@`localhost` PROCEDURE `modificacio_pilot`(IN `in_persona` MEDIUMINT, IN `in_nom` VARCHAR(100), IN `in_pais` CHAR(2), IN `in_equip` SMALLINT, IN `in_llicencia` VARCHAR(8), IN `in_inici` YEAR, OUT `RSP` VARCHAR(512))
BEGIN

declare valor_antic VARCHAR (512);
declare valor_nou VARCHAR (512);
declare consulta_pilot VARCHAR(512);
declare consulta_persona VARCHAR(512);
declare msg VARCHAR(250);
declare state VARCHAR(50);

declare old_nom  VARCHAR(100);
declare old_pais CHAR(2);
declare old_equip SMALLINT;
declare old_llicencia VARCHAR(8);
declare old_inici YEAR;

declare check_pais CHAR(2);
declare check_equip SMALLINT(5);
declare check_temporada YEAR;


-- Control d'errors

DECLARE EXIT HANDLER FOR 1062 
BEGIN
GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
SET RSP := CONCAT('ERROR: Codi ',state,'. Possiblement la llicencia  (',in_llicencia,') ja existeix a la taula \'pilot\'');
END;

DECLARE EXIT HANDLER FOR NOT FOUND
 BEGIN
 	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
   SET RSP := CONCAT('ERROR: Not found. Codi ',state,'. Pot ser que el codi de la persona (',in_persona,'), de l\'equip (',in_equip,') o el codi del pais (',in_pais,') no sigui correcte.');
 END;

DECLARE EXIT HANDLER FOR SQLWARNING 
BEGIN 
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Warning. Codi ',state,'. Missatge original: ',msg);
END;

DECLARE CONTINUE HANDLER FOR SQLEXCEPTION
BEGIN
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Desconegut. Codi ',state,'. Missatge original: ',msg);
END;


	SET RSP := 'OK';

	SELECT e.codi_equip FROM equip e WHERE e.codi_equip = in_equip INTO check_equip;
	SELECT p.codi_pais FROM pais p WHERE p.codi_pais = in_pais INTO check_pais;


	SELECT pe.nom, pe.nacionalitat
	INTO old_nom, old_pais
	FROM persona pe
	WHERE pe.codi_persona = in_persona;
		
	SELECT pi.equip, pi.codi_llicencia, pi.any_inici
	INTO old_equip, old_llicencia, old_inici
	FROM pilot pi
	WHERE pi.codi_persona = in_persona;	
		

	-- Modifica la taula
			
	UPDATE persona pe
	SET pe.nom = in_nom, pe.nacionalitat = in_pais 
	WHERE pe.codi_persona = in_persona;
			
		
	-- Actualitzar log
	
	SET consulta_persona = CONCAT("INDIRECTE-> modificacio_pilot ('",in_persona,"', '",in_nom,"', '",in_pais,"', '",in_equip,"', ",in_llicencia,"', ",in_inici,")");
	SET valor_antic = CONCAT_WS(', ',in_persona, old_nom, old_pais);
	SET valor_nou = CONCAT_WS(',',in_persona, in_nom, in_pais);
	
	INSERT INTO log(operacio, taula,antic_valor,nou_valor,data,consulta) VALUES ('MODIFICACIÓ', 'persona', valor_antic, valor_nou, NOW(), consulta_persona);
			
	UPDATE pilot pi
	SET pi.equip = in_equip, pi.codi_llicencia = in_llicencia, pi.any_inici = in_inici
	WHERE pi.codi_persona = in_persona;	
			
		
	-- Actualitzar log
	
	SET valor_antic = CONCAT_WS(', ',in_persona, old_equip, old_llicencia, old_inici);
	SET valor_nou = CONCAT_WS(',',in_persona, in_equip, in_llicencia, in_inici);
	SET consulta_pilot = CONCAT("modificacio_pilot ('",in_persona,"', '",in_nom,"', '",in_pais,"', '",in_equip,"', ",in_llicencia,"', ",in_inici,")");
	
	INSERT INTO log(operacio, taula,antic_valor,nou_valor,data,consulta) VALUES ('MODIFICACIÓ', 'pilot', valor_antic, valor_nou, NOW(), consulta_pilot);
		
END//
DELIMITER ;


-- Volcando estructura para procedimiento fia.modificacio_sector
DELIMITER //
CREATE DEFINER=`root`@`localhost` PROCEDURE `modificacio_sector`(IN `in_sector` SMALLINT, IN `in_nom` VARCHAR(50), OUT `RSP` VARCHAR(512))
BEGIN

declare valor_antic VARCHAR (512);
declare valor_nou VARCHAR (512);
declare consulta VARCHAR(512);
declare msg VARCHAR(250);
declare state VARCHAR(50);

declare old_nom  VARCHAR(50);


-- Control d'errors

DECLARE EXIT HANDLER FOR NOT FOUND
 BEGIN
 	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
   SET RSP := CONCAT('ERROR: Not found. Codi ',state,'. Possiblement el codi del sector (',in_sector,') no sigui correcte.');
 END;

DECLARE EXIT HANDLER FOR SQLWARNING 
BEGIN 
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Warning. Codi ',state,'. Missatge original: ',msg);
END;

DECLARE CONTINUE HANDLER FOR SQLEXCEPTION
BEGIN
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Desconegut. Codi ',state,'. Missatge original: ',msg);
END;


	SET RSP := 'OK';

	SELECT s.nom
	INTO old_nom
	FROM sector s
	WHERE s.codi_sector = in_sector;


	-- Modifica la taula
				
	UPDATE sector s
	SET s.nom = in_nom
	WHERE s.codi_sector = in_sector;
			
		
	-- Actualitzar log

	SET consulta = CONCAT("modificacio_sector ('",in_sector,"', '",in_nom,")");
	SET valor_antic = CONCAT_WS(', ',in_sector, old_nom);
	SET valor_nou = CONCAT_WS(',',in_sector, in_nom);
		
	INSERT INTO log(operacio, taula,antic_valor,nou_valor,data,consulta) VALUES ('MODIFICACIÓ', 'sector', valor_antic, valor_nou, NOW(), consulta);
		
END//
DELIMITER ;


-- Volcando estructura para procedimiento fia.modificacio_telemetria
DELIMITER //
CREATE DEFINER=`root`@`localhost` PROCEDURE `modificacio_telemetria`(IN `in_dada` BIGINT, IN `in_component` MEDIUMINT, IN `in_moment` DATETIME(6), IN `in_ambit` VARCHAR(50), IN `in_valor` FLOAT, IN `in_unitats` VARCHAR(25), OUT `RSP` VARCHAR(512))
BEGIN

declare valor_antic VARCHAR (512);
declare valor_nou VARCHAR (512);
declare consulta VARCHAR(512);
declare msg VARCHAR(250);
declare state VARCHAR(50);

declare old_component  MEDIUMINT;
declare old_moment DATETIME(6);
declare old_ambit VARCHAR(50);
declare old_valor FLOAT;
declare old_unitats VARCHAR(25);

declare check_component MEDIUMINT;


-- Control d'errors

DECLARE EXIT HANDLER FOR 1644 
BEGIN
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT("ERROR: Codi ",state,". ",msg);
END;

DECLARE EXIT HANDLER FOR NOT FOUND
 BEGIN
 	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
   SET RSP := CONCAT('ERROR: Not found. Codi ',state,'. Possiblement el codi del component (',in_component,') no sigui correcte.' );
 END;

DECLARE EXIT HANDLER FOR SQLWARNING 
BEGIN 
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Warning. Codi ',state,'. Missatge original: ',msg);
END;

DECLARE EXIT HANDLER FOR SQLEXCEPTION
BEGIN
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Desconegut. Codi ',state,'. Missatge original: ',msg);
END;

		
	SET RSP := 'OK';

	-- Es fa servir SELECT INTO per tal de llençar l'excepcio  NOT FOUND si no es troba el codi a la taula
	SELECT c.codi_component FROM component c WHERE c.codi_component = in_component INTO check_component;

	SELECT t.component, t.moment, t.ambit, t.valor, t.unitat_de_mesura
	INTO old_component, old_moment, old_ambit, old_valor, old_unitats
	FROM telemetria t
	WHERE t.codi_dada = in_dada;

		
	-- Modifica la taula
			
	UPDATE telemetria t
	SET t.component = in_component, t.moment = in_moment, t.ambit = in_ambit, t.valor = in_valor, t.unitat_de_mesura = in_unitats
	WHERE t.codi_dada = in_dada;
			
		
	-- Actualitzar log

	SET consulta = CONCAT("modificacio_telemetria ('",in_dada,"','",in_component,"','",in_moment,"', '",in_ambit,"', '",in_valor,"', '",in_unitats,"')");
	SET valor_antic = CONCAT_WS(', ',in_dada, old_component, old_moment, old_ambit, old_valor, old_unitats);
	SET valor_nou = CONCAT_WS(',',in_dada, in_component, in_moment, in_ambit, in_valor, in_unitats);
		
	INSERT INTO log(operacio, taula,antic_valor,nou_valor,data,consulta) VALUES ('MODIFICACIÓ', 'telemetria', valor_antic, valor_nou, NOW(), consulta);
	
			
END//
DELIMITER ;


-- Volcando estructura para procedimiento fia.modificacio_temporada
DELIMITER //
CREATE DEFINER=`root`@`localhost` PROCEDURE `modificacio_temporada`(IN `in_old_any` YEAR, IN `in_old_competicio` SMALLINT, IN `in_nou_any` YEAR, IN `in_nou_competicio` SMALLINT, OUT `RSP` VARCHAR(250))
BEGIN

declare valor_antic VARCHAR (512);
declare valor_nou VARCHAR (512);
declare consulta VARCHAR(512);
declare msg VARCHAR(250);
declare state VARCHAR(50);

declare check_temporada YEAR;
declare check_competicio SMALLINT;


-- Control d'errors

DECLARE EXIT HANDLER FOR 1062 
BEGIN
GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO;
SET RSP := CONCAT('ERROR: Codi ',state,'. Ja existeix aquesta competicio (\'',in_nou_competicio,'\'), a aquesta temporada (',in_nou_any,')');
END;

DECLARE EXIT HANDLER FOR NOT FOUND
BEGIN
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO;
	SET RSP := CONCAT('ERROR: Not found. Codi ',state,' .Possiblement la parella competicio - temporada (\'',in_old_competicio,' - ',in_old_any,'\') no sigui correcte.');
END;

DECLARE EXIT HANDLER FOR SQLWARNING 
BEGIN 
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Warning. Codi ',state,'. Missatge original: ',msg);
END;

DECLARE EXIT HANDLER FOR SQLEXCEPTION
BEGIN
	GET DIAGNOSTICS CONDITION 1 state = MYSQL_ERRNO, msg = MESSAGE_TEXT;
	SET RSP := CONCAT('ERROR: Desconegut. Codi ',state,'. Missatge original: ',msg);
END;


	SET RSP := 'OK';

	-- Es fa servir SELECT INTO per tal de llençar l'excepcio  NOT FOUND si no es troba el codi a la taula
	SELECT co.competicio, co.temporada 
	FROM competicio_temporada co
	WHERE co.competicio = in_old_competicio 
	AND co.temporada = in_old_any 
	INTO check_competicio, check_temporada;


	-- Modifica la taula
	
	UPDATE competicio_temporada co
	SET co.competicio = in_nou_competicio, co.temporada = in_nou_any
	WHERE co.competicio = in_old_competicio 
	AND co.temporada = in_old_any;
			
		
	-- Actualitzar log
	
	SET consulta = CONCAT("modificacio_temporada ('",in_old_any,"','",in_old_competicio,"','",in_nou_any,"', '",in_nou_competicio,"')");
	SET valor_antic = CONCAT_WS(', ',in_old_any, in_old_competicio);
	SET valor_nou = CONCAT_WS(',',in_nou_any, in_nou_competicio);
		
	INSERT INTO log(operacio, taula,antic_valor,nou_valor,data,consulta) VALUES ('MODIFICACIÓ', 'temporada', valor_antic, valor_nou, NOW(), consulta);
		
END//
DELIMITER ;


-- Volcando estructura para procedimiento fia.simulacio_consum
DELIMITER //
CREATE DEFINER=`root`@`localhost` PROCEDURE `simulacio_consum`(IN `in_component` MEDIUMINT, IN `in_inici` DATETIME(6), IN `in_consum_maxim` INT)
BEGIN

-- 130 litres per carrera
-- 300 km per carrera
-- de 3000 a 6000 dades per carrera
-- 0.022 litres per segon de mitjana 


declare moment_actual DATETIME(6);
declare RSP VARCHAR(512);
declare in_fi DATETIME(6);
	
	SET moment_actual = in_inici;
	SET in_fi = DATE_ADD(moment_actual, INTERVAL (FLOOR(RAND()*(6000-3000)+3000)) SECOND);
	
	WHILE moment_actual < in_fi DO
		call alta_telemetria(in_component,moment_actual,'consum', FLOOR(RAND()*(in_consum_maxim-210)+210)/10000,'lps', RSP);
		SET moment_actual = DATE_ADD(moment_actual, INTERVAL 1 SECOND);
	END WHILE;


END//
DELIMITER ;


-- Volcando estructura para procedimiento fia.simulacio_temperatura
DELIMITER //
CREATE DEFINER=`root`@`localhost` PROCEDURE `simulacio_temperatura`(IN `in_component` MEDIUMINT, IN `in_inici` DATETIME(6))
BEGIN

-- 1200ºC temp maxima
-- 0ºC temp maxima
-- de 3000 a 6000 dades per carrera

declare moment_actual DATETIME(6);
declare in_fi DATETIME(6);
declare RSP VARCHAR(512);
	
	SET moment_actual = in_inici;
	SET in_fi = DATE_ADD(moment_actual, INTERVAL (FLOOR(RAND()*(6000-3000)+3000)) SECOND);
	
	WHILE moment_actual < in_fi DO
		call alta_telemetria(in_component,moment_actual,'temperatura', FLOOR(RAND()*(1200-0)+0),'ºC', RSP);
		SET moment_actual = DATE_ADD(moment_actual, INTERVAL 1 SECOND);
	END WHILE;


END//
DELIMITER ;


-- Volcando estructura para procedimiento fia.simulacio_temps_volta
DELIMITER //
CREATE DEFINER=`root`@`localhost` PROCEDURE `simulacio_temps_volta`(IN `in_component` MEDIUMINT, IN `in_inici` DATETIME(6), IN `in_temps_maxim` TIME(6))
BEGIN


-- 6000 segons per carrera
-- 60 voltes per carrera

declare temps_volta SMALLINT(5);
declare loop_moment DATETIME(6);
declare voltes SMALLINT(5);
declare RSP VARCHAR(512);
	
	SET loop_moment = in_inici;
	SET voltes = 0;
	
	WHILE voltes < 60 DO
		SET temps_volta = FLOOR(RAND()*(in_temps_maxim-90)+90);
		call alta_telemetria(in_component,loop_moment,'temps per volta', temps_volta*1000,'ms', RSP);
		SET loop_moment = DATE_ADD(loop_moment, INTERVAL temps_volta SECOND);
		SET voltes = voltes + 1;
	END WHILE;


END//
DELIMITER ;
/*!40101 SET SQL_MODE=IFNULL(@OLD_SQL_MODE, '') */;
/*!40014 SET FOREIGN_KEY_CHECKS=IF(@OLD_FOREIGN_KEY_CHECKS IS NULL, 1, @OLD_FOREIGN_KEY_CHECKS) */;
/*!40101 SET CHARACTER_SET_CLIENT=@OLD_CHARACTER_SET_CLIENT */;






-- --------------------------------------------------------
-- Host:                         127.0.0.1
-- Versión del servidor:         5.6.21 - MySQL Community Server (GPL)
-- SO del servidor:              Win32
-- HeidiSQL Versión:             9.2.0.4947
-- --------------------------------------------------------

/*!40101 SET @OLD_CHARACTER_SET_CLIENT=@@CHARACTER_SET_CLIENT */;
/*!40101 SET NAMES utf8mb4 */;
/*!40014 SET @OLD_FOREIGN_KEY_CHECKS=@@FOREIGN_KEY_CHECKS, FOREIGN_KEY_CHECKS=0 */;
/*!40101 SET @OLD_SQL_MODE=@@SQL_MODE, SQL_MODE='NO_AUTO_VALUE_ON_ZERO' */;

-- Volcando estructura de base de datos para fia

USE `fia`;


-- Volcando estructura para disparador fia.chk_circuit_long_insert
SET @OLDTMP_SQL_MODE=@@SQL_MODE, SQL_MODE='NO_ENGINE_SUBSTITUTION';
DELIMITER //
CREATE TRIGGER `chk_circuit_long_insert` BEFORE INSERT ON `circuit` FOR EACH ROW begin  
      if  NEW.longitud<1000 then
       SIGNAL SQLSTATE '45000'   
	       SET MESSAGE_TEXT = 'Longitud massa petita.';
       end if; 
    end//
DELIMITER ;
SET SQL_MODE=@OLDTMP_SQL_MODE;


-- Volcando estructura para disparador fia.chk_circuit_long_update
SET @OLDTMP_SQL_MODE=@@SQL_MODE, SQL_MODE='NO_ENGINE_SUBSTITUTION';
DELIMITER //
CREATE TRIGGER `chk_circuit_long_update` BEFORE UPDATE ON `circuit` FOR EACH ROW begin  
      if  NEW.longitud<1000 then
       SIGNAL SQLSTATE '45000'   
	       SET MESSAGE_TEXT = 'Longitud massa petita.';
       end if; 
    end//
DELIMITER ;
SET SQL_MODE=@OLDTMP_SQL_MODE;


-- Volcando estructura para disparador fia.chk_cost_patrocini_insert
SET @OLDTMP_SQL_MODE=@@SQL_MODE, SQL_MODE='NO_ENGINE_SUBSTITUTION';
DELIMITER //
CREATE TRIGGER `chk_cost_patrocini_insert` BEFORE INSERT ON `patrocini` FOR EACH ROW begin  
      if  NEW.cost<10000 then
       SIGNAL SQLSTATE '45000'   
	       SET MESSAGE_TEXT = 'Cost massa petit.';
       end if; 
    end//
DELIMITER ;
SET SQL_MODE=@OLDTMP_SQL_MODE;


-- Volcando estructura para disparador fia.chk_cost_patrocini_update
SET @OLDTMP_SQL_MODE=@@SQL_MODE, SQL_MODE='NO_ENGINE_SUBSTITUTION';
DELIMITER //
CREATE TRIGGER `chk_cost_patrocini_update` BEFORE UPDATE ON `patrocini` FOR EACH ROW begin  
      if  NEW.cost<10000 then
       SIGNAL SQLSTATE '45000'   
	       SET MESSAGE_TEXT = 'Cost massa petit.';
       end if; 
    end//
DELIMITER ;
SET SQL_MODE=@OLDTMP_SQL_MODE;


-- Volcando estructura para disparador fia.chk_telemetria_value_insert
SET @OLDTMP_SQL_MODE=@@SQL_MODE, SQL_MODE='NO_ENGINE_SUBSTITUTION';
DELIMITER //
CREATE TRIGGER `chk_telemetria_value_insert` BEFORE INSERT ON `telemetria` FOR EACH ROW begin  
      if  NEW.ambit='velocitat' and NEW.valor>500 then
       SIGNAL SQLSTATE '45000'   
	       SET MESSAGE_TEXT = 'Camp velocitat massa gran.', COLUMN_NAME = NEW.component;
       end if; 
       
      if  NEW.ambit='temps per volta' and NEW.valor=0 then
       SIGNAL SQLSTATE '45000'   
	       SET MESSAGE_TEXT = 'Temps per volta igual a 0.', COLUMN_NAME = NEW.component;
       end if;
       
      if  NEW.valor<0 then
       SIGNAL SQLSTATE '45000'   
	       SET MESSAGE_TEXT = 'Valor negatiu.', COLUMN_NAME = NEW.component;
       end if;
    end//
DELIMITER ;
SET SQL_MODE=@OLDTMP_SQL_MODE;


-- Volcando estructura para disparador fia.chk_telemetria_value_update
SET @OLDTMP_SQL_MODE=@@SQL_MODE, SQL_MODE='NO_ENGINE_SUBSTITUTION';
DELIMITER //
CREATE TRIGGER `chk_telemetria_value_update` BEFORE UPDATE ON `telemetria` FOR EACH ROW begin  
      if  NEW.ambit='velocitat' and NEW.valor>500 then
       SIGNAL SQLSTATE '45000'   
	       SET MESSAGE_TEXT = 'Camp velocitat massa gran.', COLUMN_NAME = NEW.component;
       end if; 
       
      if  NEW.ambit='temps per volta' and NEW.valor=0 then
       SIGNAL SQLSTATE '45000'   
	       SET MESSAGE_TEXT = 'Temps per volta igual a 0.', COLUMN_NAME = NEW.component;
       end if;
       
      if  NEW.valor<0 then
       SIGNAL SQLSTATE '45000'   
	       SET MESSAGE_TEXT = 'Valor negatiu.', COLUMN_NAME = NEW.component;
       end if;
    end//
DELIMITER ;
SET SQL_MODE=@OLDTMP_SQL_MODE;
/*!40101 SET SQL_MODE=IFNULL(@OLD_SQL_MODE, '') */;
/*!40014 SET FOREIGN_KEY_CHECKS=IF(@OLD_FOREIGN_KEY_CHECKS IS NULL, 1, @OLD_FOREIGN_KEY_CHECKS) */;
/*!40101 SET CHARACTER_SET_CLIENT=@OLD_CHARACTER_SET_CLIENT */;
